package com.at.video.atlas.ui.classify

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.atlas.entity.ConditionsInfo
import com.at.video.atlas.http.ApiService
import com.at.video.atlas.ui.classify.binder.ClassifyFreeBinder
import com.at.video.atlas.ui.classify.binder.ClassifyPayBinder
import com.at.video.atlas.ui.classify.binder.ConditionsBinder
import com.melancholy.network.ApiFactory
import com.video.module_widget.binder.*
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvPosition
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch

/**
 * @Create_time: 2022/5/26 20:39
 * @Description: 筛选
 */

data class ClassifyUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf()
) : IUiState

sealed class ClassifyUiEvent : IUiEvent {
    object InitLoadData : ClassifyUiEvent()
    object RefreshData : ClassifyUiEvent()
    object LoadDataMore : ClassifyUiEvent()
    data class Switch(val info: ConditionsBinder.Conditions, val position: Int) : ClassifyUiEvent()
    data class InitCategoryId(val categoryId: String) : ClassifyUiEvent()
    data class InitStyle(val style: HomeStyle?) : ClassifyUiEvent()
}

sealed class ClassifyUiEffect : IUiEffect

class ClassifyViewModel(application: Application) : BaseAndroidViewModel<ClassifyUiState,
    ClassifyUiEvent, ClassifyUiEffect>(application) {
    private var mOffset: Int = 0
    private var mLimit: Int = 9
    private var mCategoryId: String? = null
    private var mCost: String? = null
    private var mSort: String? = null
    private var mType: String? = null
    private var mSubcategory: String? = null
    private var style: HomeStyle? = null
    private var dataPosition: Int = 0
    private var hasMore: Boolean = false
    override fun providerInitialState(): ClassifyUiState {
        return ClassifyUiState()
    }

    override fun handleEvent(event: ClassifyUiEvent) {
        when (event) {
            is ClassifyUiEvent.InitLoadData -> initLoadData()
            is ClassifyUiEvent.RefreshData -> initLoadData(true)
            is ClassifyUiEvent.LoadDataMore -> loadDataMore()
            is ClassifyUiEvent.Switch -> switch(event.info, event.position)
            is ClassifyUiEvent.InitCategoryId -> mCategoryId = event.categoryId
            is ClassifyUiEvent.InitStyle -> style = event.style
        }
    }

    private fun initLoadData(isRefresh: Boolean = false) {
        if (isRefresh) {
            mOffset = 0
            hasMore = false
        }
        mLimit = getLimit()
        viewModelScope.launch(Dispatchers.IO) {
            combine(
                listOf(
                    getFilter(),
                    if (!mCategoryId.isNullOrEmpty()) {
                        getSubCategory(mCategoryId!!)
                    } else {
                        flow { emit(listOf()) }
                    },
                    getList()
                )
            ) {
                val filter: List<Any>? = dealFilter(it[0], it[1])
                val atlas = it[2]
                if (filter.isNullOrEmpty() && atlas.isNullOrEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty) }
                } else if (!filter.isNullOrEmpty() && !atlas.isNullOrEmpty()) {
                    if (isRefresh) {
                        setState {
                            copy(
                                refreshStatus = RefreshStatus.RefreshSuccess,
                                data = dataTransfer(filter, atlas),
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                            )
                        }
                    } else {
                        setState {
                            copy(
                                pageStatus = PageStatus.Success,
                                data = dataTransfer(filter, atlas),
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                            )
                        }
                    }
                } else if (!filter.isNullOrEmpty() && atlas.isNullOrEmpty()) {
                    setState {
                        copy(
                            refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle,
                            pageStatus = PageStatus.Success,
                            data = dataTransfer(filter, mutableListOf(NoDataBinder.Data("暂无数据"))),
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }.onStart {
                if (isRefresh) {
                    setState {
                        copy(
                            refreshStatus = RefreshStatus.RefreshLoading,
                            loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                        )
                    }
                } else {
                    setState { copy(pageStatus = PageStatus.Loading) }
                }
            }.catch {
                if (isRefresh) {
                    setState { copy(refreshStatus = RefreshStatus.RefreshFail(it)) }
                } else {
                    setState { copy(pageStatus = PageStatus.Error(it)) }
                }
            }.collect()
        }
    }

    /**
     * 数据整合
     */
    private fun dataTransfer(condition: List<Any>, data: List<Any>): MutableList<Any> {
        val datas: MutableList<Any> = mutableListOf()
        style?.let {
            it.content?.forEach { item ->
                when (item!!.type) {
                    StyleConsts.SWIPER_ADV -> {
                        val carourseAdvs = JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.CAROUSEL_ADV)?.let { it1 ->
                            DataUtils.filterAdvPosition(it1, AdvPosition.AtlasCategory)
                        }
                        carourseAdvs?.let { advs ->
                            datas.add(GapLineBinder.Data(""))
                            datas.add(BannerBinder.BannerInfo(advs))
                            dataPosition += 1
                        }
                    }
                    StyleConsts.BANNER_ADV -> {
                        val bannerAdvs = JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.BANNER_ADV)?.let { it1 ->
                            DataUtils.filterAdvPosition(it1, AdvPosition.AtlasCategory)
                        }
                        val adBinderData: MutableList<Any> = mutableListOf()
                        bannerAdvs?.let { advs ->
                            advs.forEach { item ->
                                run {
                                    adBinderData.add(GapLineBinder.Data(""))
                                    if (item.image_type != 3) {
                                        adBinderData.add(AdImageBinder.Data(item))
                                    } else {
                                        adBinderData.add(AdTextBinder.Data(item))
                                    }
                                }
                            }
                            dataPosition += adBinderData.size
                            datas.addAll(adBinderData)
                        }
                    }
                    StyleConsts.FILTER_BAR -> {
                        datas.addAll(condition)
                    }
                }
            }
        }
        datas.add(GapLineBinder.Data(""))
        datas.addAll(data)
        return datas
    }

    private fun getLimit(): Int {
        val num = if (style != null) {
            StyleUtils.getHomeContentStyle(style!!, StyleConsts.RESULT)?.config?.layout?.limit
        } else 9
        return num ?: 9
    }

    private fun dealFilter(filter1: List<Any>?, filter2: List<Any>?): List<Any>? {
        val newFilter = mutableListOf<Any>()
        // 去除以前的子分类
        filter1?.apply {
            if (!mCategoryId.isNullOrEmpty()) {
                forEach {
                    if (it is ConditionsBinder.Conditions && it.type != 1) {
                        newFilter.add(it)
                    }
                }
            } else {
                newFilter.addAll(filter1)
            }
        }

        if (filter2.isNullOrEmpty()) {
            return newFilter
        }

        if (newFilter.size > 1) {
            filter2.forEach {
                if (it is ConditionsBinder.Conditions && it.type == 1) {
                    newFilter.add(1, it)
                }
            }
        }
        return newFilter
    }

    private fun loadDataMore() {
        mOffset += getLimit()
        viewModelScope.launch(Dispatchers.IO) {
            getList()
                .onStart {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
                }.catch {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(it)) }
                }.collect {
                    if (it.isNullOrEmpty()) {
                        setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                    } else {
                        setState {
                            copy(
                                data = data + it,
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                            )
                        }
                    }
                }
        }
    }

    /**
     * 获取筛选条件
     */
    private fun getFilter(): Flow<List<Any>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["site_id"] = GlobalConsts.SITE_ID
            // if (!mCategoryId.isNullOrEmpty()) {
            //     map["categoryId"] = mCategoryId!!.toInt()
            // }
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .getCondition(map)
            if (bean.success()) {
                val list: MutableList<Any> = mutableListOf()
                val filter = bean.category_list!!
                if (filter.list.isNotEmpty()) {
                    val category: MutableList<ConditionsInfo> = mutableListOf()
                    filter.list.forEach {
                        category.add(
                            ConditionsInfo(
                                it.name!!,
                                it.id,
                                isSelected = mCategoryId == it.id
                            )
                        )
                    }
                    // 自添加选项
                    if (category.size > 0) {
                        category.add(0, ConditionsInfo("全部", null, isSelected = true))
                        list.add(ConditionsBinder.Conditions(category, 0))
                    }
                }
                if (bean.sub_category_list != null) {
                    val subcategory: MutableList<ConditionsInfo> = mutableListOf()
                    bean.sub_category_list.list?.forEach {
                        subcategory.add(
                            ConditionsInfo(
                                it.name!!,
                                it.id,
                                isSelected = mSubcategory == it.id
                            )
                        )
                    }
                    if (subcategory.size > 0) {
                        subcategory.add(
                            0,
                            ConditionsInfo("全部", isSelected = mSubcategory == null)
                        )
                        list.add(ConditionsBinder.Conditions(subcategory, 1))
                    }
                }
                if (bean.types_list != null) {
                    val typeList: MutableList<ConditionsInfo> = mutableListOf()
                    bean.types_list.list?.forEach {
                        typeList.add(
                            ConditionsInfo(
                                it.name!!,
                                it.id,
                                isSelected = mSubcategory == it.id
                            )
                        )
                    }
                    if (typeList.size > 0) {
                        typeList.add(
                            0,
                            ConditionsInfo("全部", isSelected = mSubcategory == null)
                        )
                        list.add(ConditionsBinder.Conditions(typeList, 2))
                    }
                }
                val mSortMutableList = mutableListOf<ConditionsInfo>()
                mSortMutableList.add(0, ConditionsInfo("全部", null))
                mSortMutableList.add(1, ConditionsInfo("最新", "created_at"))
                mSortMutableList.add(2, ConditionsInfo("最热", "popularity"))
                list.add(ConditionsBinder.Conditions(mSortMutableList, 3))

                val mCostMutableList = mutableListOf<ConditionsInfo>()
                mCostMutableList.add(0, ConditionsInfo("全部", null))
                mCostMutableList.add(1, ConditionsInfo("金币", "2"))
                mCostMutableList.add(2, ConditionsInfo("免费", "1"))
                list.add(ConditionsBinder.Conditions(mCostMutableList, 4))
                emit(list)
            } else {
                throw Error(bean.message)
            }
        }
    }

    /**
     * 获取数据
     */
    private fun getList(): Flow<List<Any>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["site_id"] = GlobalConsts.SITE_ID
            map["offset"] = mOffset
            map["limit"] = mLimit
            if (!mCategoryId.isNullOrEmpty()) {
                map["category_id"] = mCategoryId!!
            }
            if (!mCost.isNullOrEmpty()) {
                map["price_type"] = mCost!!.toInt()
            }
            if (!mSort.isNullOrEmpty()) {
                map["sort"] = mSort!!
            } else {
                map["sort"] = "updated_at"
            }
            if (!mSubcategory.isNullOrEmpty()) {
                map["sub_category_id"] = mSubcategory!!
            }
            if (!mType.isNullOrEmpty()) {
                map["types"] = mType!!
            }
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .getPictureList(map)
            if (bean.success()) {
                if (bean.list.isNullOrEmpty()) {
                    hasMore = false
                    emit(null)
                } else {
                    val list: MutableList<Any> = mutableListOf()
                    hasMore = bean.total!!.toInt() > mOffset + bean.list.size

                    bean.list.forEach {
                        if (it.price_type == 1) {
                            list.add(ClassifyFreeBinder.Free(it))
                        } else {
                            list.add(ClassifyPayBinder.Pay(it))
                        }
                    }
                    emit(list)
                }
            } else {
                throw Error(bean.msg())
            }
        }
    }

    // type 0分类 1费用类型 2排序方式 3子分类
    private fun switch(info: ConditionsBinder.Conditions, position: Int) {
        viewModelScope.launch(Dispatchers.IO) {
            val oldData = state.value.data.toMutableList()
            val index: Int = oldData.indexOf(info)
            if (index >= 0) {
                when (info.type) {
                    0 -> mCategoryId = info.list[position].`val`
                    4 -> mCost = info.list[position].`val`
                    3 -> mSort = info.list[position].`val`
                    1 -> mSubcategory = info.list[position].`val`
                    2 -> mType = info.list[position].`val`
                }

                oldData[index] = info.copy(index = position, isSwitch = true)
                setState { copy(data = oldData.toList()) }
                mOffset = 0
//                initLoadTestData()
//                if (info.type == 0) {//父级分类重新获取所有
//                    reacquireData(mCategoryId?.toInt() ?: -1)
//                    return@launch
//                }
                combine(
                    listOf(
                        if (info.type == 0) {
                            getSubCategory(mCategoryId)
                        } else {
                            flow { emit(listOf()) }
                        },
                        getList()
                    )
                ) {
                    val subCategory = it[0]
                    val searchData = it[1]

                    if (subCategory.isNullOrEmpty() && info.type == 0 && !mCategoryId.isNullOrEmpty()) {
                        setState { copy(pageStatus = PageStatus.Error(Throwable("服务器异常"))) }
                    } else {
                        val condition = if (!subCategory.isNullOrEmpty()) {
                            subCategory
                        } else {
                            val oldData = state.value.data.toMutableList()
                            val newData = mutableListOf<Any>()
                            // 去除以前的子分类
                            oldData.forEach { it1 ->
                                if (it1 is ConditionsBinder.Conditions) {
                                    newData.add(it1)
                                }
                            }
                            newData
                        }
                        if (searchData.isNullOrEmpty()) {
                            setState {
                                copy(
                                    data = dataTransfer(condition, mutableListOf(NoDataBinder.Data("暂无数据"))),
                                    loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                                )
                            }
                        } else {
                            setState {
                                copy(
                                    data = dataTransfer(condition, searchData),
                                    loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                                )
                            }
                        }
                    }
                }.onStart {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
                }.catch {
                    setState { copy(pageStatus = PageStatus.Error(it)) }
                }.collect()
            }
        }
    }

    // 获取二级分类
    private fun getSubCategory(mCategoryId: String?): Flow<List<Any>?> {
        return flow {
            val oldData = state.value.data.toMutableList()
            val newData = mutableListOf<Any>()
            // 去除以前的子分类
            oldData.forEach {
                if (it is ConditionsBinder.Conditions && it.type != 1) {
                    newData.add(it)
                }
            }
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["site_id"] = GlobalConsts.SITE_ID
            map["type"] = 2
            map["module"] = 2
            if (!mCategoryId.isNullOrEmpty()) {
                map["parent_id"] = mCategoryId
            }
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .getSubCondition(map)
            if (bean.success()) {
                val subcategory: MutableList<ConditionsInfo> = mutableListOf()
                bean.list?.forEach {
                    subcategory.add(
                        ConditionsInfo(
                            it.name!!,
                            it.id,
                            isSelected = mSubcategory == it.id
                        )
                    )
                }
                if (subcategory.size > 0) {
                    subcategory.add(
                        0,
                        ConditionsInfo("全部", isSelected = mSubcategory == null)
                    )
                    newData.add(if (newData.size > 1) 1 else 0, ConditionsBinder.Conditions(subcategory, 1))
                }
            }
            emit(newData)
        }
    }
}
