package com.at.video.atlas.ui.detail

import android.app.Application
import android.util.Log
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.atlas.entity.PictureDetail
import com.at.video.atlas.http.ApiService
import com.at.video.export.mine.IMineService
import com.melancholy.network.ApiFactory
import com.video.module_widget.api.CommonApiService
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.getFloatDigits
import com.video.module_widget.strToFloat
import com.video.module_widget.utils.DataUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/7 下午8:27
 */
data class DetailUiState(
    val msg: String = "",
    val data: PictureDetail? = null,
    val datas: List<Any>? = null,
    val pos: Int = 0,
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle
) : IUiState

sealed class DetailUiEvent : IUiEvent {
    data class AtlasInfo(val id: String) : DetailUiEvent()
    object Collect : DetailUiEvent()
    object ThumbUp : DetailUiEvent()
    object Share : DetailUiEvent()
    data class UpdateBrowser(val pos: Int = 0) : DetailUiEvent()
    object BrowserAble : DetailUiEvent()
    object BuyByGold : DetailUiEvent()
    object Refresh : DetailUiEvent()
    data class InitStyle(val style: HomeStyle?) : DetailUiEvent()
}

sealed class DetailUiEffect : IUiEffect {
    data class Loading(val msg: String) : DetailUiEffect()
    data class Toast(val toast: String, val finish: Boolean = false) : DetailUiEffect()
    object Dismiss : DetailUiEffect()
    data class Success(val atlas: PictureDetail?) : DetailUiEffect()
    data class CollectChange(val atlas: PictureDetail?) : DetailUiEffect()
    data class ThumbUpChange(val atlas: PictureDetail?) : DetailUiEffect()
    data class BuyAtlasState(val state: Boolean) : DetailUiEffect()
}

class DetailViewModel(application: Application) :
    BaseAndroidViewModel<DetailUiState, DetailUiEvent, DetailUiEffect>(application) {
    private var id = ""
    private var style: HomeStyle? = null

    override fun handleEvent(event: DetailUiEvent) {
        when (event) {
            is DetailUiEvent.AtlasInfo -> {
                id = event.id
                atlasInfo()
            }
            is DetailUiEvent.Collect -> collect()
            is DetailUiEvent.ThumbUp -> thumbUp()
            is DetailUiEvent.UpdateBrowser -> updateBrowser(event.pos)
            is DetailUiEvent.BuyByGold -> buyByGold()
            is DetailUiEvent.Refresh -> atlasInfo(true)
            is DetailUiEvent.InitStyle -> style = event.style
            is DetailUiEvent.Share -> shareRes()
            else -> {}
        }
    }

    private fun shareRes() {
        val data = state.value.data
        if (data == null) {
            setEffect {
                DetailUiEffect.Toast("操作失败")
            }
            return
        }
        viewModelScope.launch(Dispatchers.IO) {
            val map = mutableMapOf<String, Any>()
            map["resource_id"] = data.picture?.id.toString()
            map["pop_share_count"] = 1
            map["module"] = 2
            ApiFactory.getInstance()
                .create(CommonApiService::class.java)
                .shareRes(map = map)
        }
    }

    // 金币购买
    private fun buyByGold() {
        val data = state.value.data
        viewModelScope.launch {
            val info =
                ApiFactory.getInstance().create(ApiService::class.java).buyAtlas(1)
            if (info.success()) {
//                data.useAuth=1
                setState { copy(data = data) }
                setEffect {
                    DetailUiEffect.BuyAtlasState(true)
                }
            } else {
                setEffect {
                    DetailUiEffect.Toast(info.msg ?: "购买失败")
                }
                setEffect {
                    DetailUiEffect.BuyAtlasState(false)
                }
            }
        }
    }

    private var curTime = 0L

    // 更新播放浏览
    private fun updateBrowser(pos: Int) {
        val time = System.currentTimeMillis()
        val data = state.value.data
        if (time - curTime > 1000) {
            curTime = time
            return
        }
//        if (data?.watchHistory!=null){
//            return
//        }
//        setState { copy(pos = pos) }

        viewModelScope.launch {
            val map = mutableMapOf<String, Any>()
            map["type"] = 2
            map["resource_id"] = data?.picture?.id.toString()
            map["progress"] = pos.toString()
            val percent = ((pos.toFloat() / (state.value.datas?.size ?: pos)).getFloatDigits(2))
            map["percentage"] = if (percent.toFloat() > 1f) "1" else percent
            Log.i("history percentage:", "${percent}")
            map["episode"] = 1
            try {
                val res = ApiFactory.getInstance().create(CommonApiService::class.java)
                    .watchHistory(map = map)
                if (res.success()) {
                    Log.d("heat", "updateBrowser:更新数据成功,有历史了$pos ")
                } else {
                    Log.d("heat", "updateBrowser:更新数据失败")
                }
            } catch (e: Exception) {
                DetailUiEffect.Toast("服务器异常!")
            }
        }
    }

    // 收藏
    private fun collect() {
        val data = state.value.data
        if (data == null) {
            setEffect {
                DetailUiEffect.Toast("操作失败")
            }
            return
        }
        viewModelScope.launch {
            setEffect {
                DetailUiEffect.Loading("处理中...")
            }
            val map = mutableMapOf<String, Any>()
            map["type"] = 2
//            map["operateState"] = if (data.isCollection) {
//                2
//            } else 1

            var res = if (!data.picture!!.is_collect) {
                map["resource_id"] = data.picture.id.toString()
                ApiFactory.getInstance()
                    .create(CommonApiService::class.java)
                    .collect(map = map)
            } else {
                map["resource_id_list"] = arrayListOf(data.picture.id.toString())
                ApiFactory.getInstance()
                    .create(CommonApiService::class.java)
                    .deleteCollect(map = map)
            }
            setEffect {
                DetailUiEffect.Dismiss
            }
            if (res.success()) {
                setEffect {
                    DetailUiEffect.Toast(if (data.picture.is_collect) "已取消" else "收藏成功,可在\"我的\"页面查看")
                }
                data.picture.is_collect = !data.picture.is_collect
//                data.isCollection = !data.isCollection
                setState { copy(data = data) }
                setEffect {
                    DetailUiEffect.CollectChange(data)
                }
            } else {
                setEffect {
                    DetailUiEffect.Toast(
                        if (data.picture.is_collect) {
                            "取消收藏失败"
                        } else "收藏失败"
                    )
                }
            }
        }
    }

    // 点赞
    private fun thumbUp() {
        val data = state.value.data
        if (data == null) {
            setEffect {
                DetailUiEffect.Toast("操作失败")
            }
            return
        }
        viewModelScope.launch {
            setEffect {
                DetailUiEffect.Loading("处理中...")
            }
            val map = mutableMapOf<String, Any>()
            map["resource_id"] = data.picture!!.id.toString()
            map["type"] = 2

            val res = if (!data.picture.is_star) ApiFactory.getInstance()
                .create(CommonApiService::class.java)
                .star(map = map) else ApiFactory.getInstance().create(CommonApiService::class.java)
                .deleteStar(map = map)
            setEffect {
                DetailUiEffect.Dismiss
            }
            if (res.success()) {
                setEffect {
                    DetailUiEffect.Toast(if (data.picture.is_star) "已取消" else "谢谢你的喜欢")
                }
                data.picture.is_star = !data.picture.is_star
                setState { copy(data = data) }
                setEffect {
                    DetailUiEffect.ThumbUpChange(data)
                }
            } else {
                setEffect {
                    DetailUiEffect.Toast(
                        if (data.picture.is_star) {
                            "取消点赞失败"
                        } else "点赞失败"
                    )
                }
            }
        }
    }

    // 获取图集详情
    private fun atlasInfo(isRefresh: Boolean = false) {
        viewModelScope.launch(Dispatchers.IO) {
            if (isRefresh) {
                setState { copy(refreshStatus = RefreshStatus.RefreshLoading) }
            } else {
                setState { copy(pageStatus = PageStatus.Loading) }
            }
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["site_id"] = GlobalConsts.SITE_ID
            map["id"] = id
            val sourceInfo =
                ApiFactory.getInstance().create(ApiService::class.java).getPictureDetail(map)

            if (sourceInfo.success()) {
                val data = sourceInfo.picture_chapter_list
                val status = sourceInfo.status
                dataReport(sourceInfo.picture?.id)
                status?.let {
                    sourceInfo.picture!!.is_star = it.is_like == 1
                    sourceInfo.picture.is_collect = it.is_collect == 1
                }
                val imagesList = data?.list?.toMutableList()
                if (imagesList.isNullOrEmpty()) {
                    setState {
                        copy(
                            data = sourceInfo,
                            pageStatus = PageStatus.Empty,
                            refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                        )
                    }
                } else {
                    val list = mutableListOf<Any>()
                    imagesList.forEach {
                        if (!it.url.isNullOrEmpty()) {
                            list.addAll(it.url.split("\n"))
                        }
                    }
                    val posVal = sourceInfo.watch_history?.percentage?.strToFloat()
                        ?: 0f
                    if (sourceInfo.watch_history != null && posVal != 0f
                    ) {
                        val pos = (list.size ?: 0) * posVal
                        Log.d("heat", "watchHistory:观看到：$pos")
                        setState {
                            copy(
                                pos = pos.toInt()
                            )
                        }
                    } else {
                        sendEvent(DetailUiEvent.BrowserAble)
                    }
                    setState {
                        copy(
                            data = sourceInfo,
                            datas = list,
                            pageStatus = PageStatus.Success,
                            refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                        )
                    }
                }
                setEffect {
                    DetailUiEffect.Success(sourceInfo)
                }

            } else {
                setState {
                    copy(
                        pageStatus = PageStatus.Error(Error(sourceInfo.message)),
                        refreshStatus = if (isRefresh) RefreshStatus.RefreshFail(Error(sourceInfo.message)) else RefreshStatus.RefreshIdle,
                        loadMoreStatus = LoadMoreStatus.LoadMoreFail(Error(sourceInfo.message))
                    )
                }
                setEffect { DetailUiEffect.Toast(sourceInfo.msg() ?: "获取图集失败,请稍后重试", true) }
            }
        }
    }

    /**
     * 数据上报
     */
    private fun dataReport(resId: String?) {
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            val readRecord = DataUtils.dataReport(3).play_record!!
            readRecord.user_id = IMineService.getMineService().getUser().value?.getUserId()
            readRecord.resource_type = 1
            readRecord.resource_id = resId
            map["read_record"] = readRecord
            try {
                val res = ApiFactory.getInstance().create(CommonApiService::class.java).dataReport(
                    GlobalConsts.DATA_REPORT + "/v1/basic_record/create",
                    map
                )
                if (res.success()) {
                    Log.d("DATA_REPORT", "上报成功")
                } else {
                    Log.d("DATA_REPORT", "上报失败")
                }
            } catch (e: Exception) {
                Log.e("Exception", e.message.toString())
            }
        }
    }

    override fun providerInitialState(): DetailUiState {
        return DetailUiState()
    }
}
