package com.at.video.atlas.ui.more

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.atlas.http.ApiService
import com.at.video.atlas.ui.atlas.binder.AtlasFreeBinder
import com.at.video.atlas.ui.atlas.binder.AtlasPayBinder
import com.melancholy.network.ApiFactory
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.GapLineBinder
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvPosition
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/23 下午2:36
 */
data class MoreUiState(
    val msg: String = "",
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf()
) : IUiState

sealed class MoreUiEvent : IUiEvent {
    data class SetParameter(val param: String = "") : MoreUiEvent()
    object InitLoadData : MoreUiEvent()
    object RefreshData : MoreUiEvent()
    object LoadDataMore : MoreUiEvent()
    data class InitStyle(val style: HomeStyle?) : MoreUiEvent()

}

sealed class MoreUiEffect : IUiEffect
class AtlasMoreViewModel(application: Application) :
    BaseAndroidViewModel<MoreUiState, MoreUiEvent, MoreUiEffect>(application) {

    private var mOffset: Int = 0
    private var mLimit: Int = 9
    private var param: String = ""
    private var style: HomeStyle? = null
    private var hasMore: Boolean = false
    override fun handleEvent(event: MoreUiEvent) {
        when (event) {
            is MoreUiEvent.SetParameter -> {
                param = event.param
                sendEvent(MoreUiEvent.InitLoadData)
            }
            is MoreUiEvent.InitLoadData -> initLoadData()
            is MoreUiEvent.RefreshData -> initLoadData(true)
            is MoreUiEvent.LoadDataMore -> loadDataMore()
            is MoreUiEvent.InitStyle -> style = event.style
        }
    }

    private fun getLimit(): Int {
        val num = if (style != null) {
            StyleUtils.getHomeContentStyle(style!!, StyleConsts.RESULT)?.config?.layout?.limit
        } else 9
        return num ?: 9
    }

    private fun loadDataMore() {
        mOffset += getLimit()
        viewModelScope.launch(Dispatchers.IO) {
            getList()
                .onStart {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
                }.catch {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(it)) }
                }.collect {
                    if (it.isNullOrEmpty()) {
                        setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                    } else {
                        setState {
                            copy(
                                data = data + it,
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                            )
                        }
                    }
                }
        }
    }

    private fun initLoadData(isRefresh: Boolean = false) {
        viewModelScope.launch(Dispatchers.IO) {
            getList().onStart {
                if (isRefresh) {
                    setState {
                        copy(
                            refreshStatus = RefreshStatus.RefreshLoading,
                            loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                        )
                    }
                } else {
                    setState { copy(pageStatus = PageStatus.Loading) }
                }
            }.catch {
                setState {
                    copy(
                        pageStatus = PageStatus.Error(it),
                        refreshStatus = if (isRefresh) RefreshStatus.RefreshFail(it) else RefreshStatus.RefreshIdle
                    )
                }
            }.collect {
                if (it.isNullOrEmpty()) {
                    setState {
                        copy(
                            pageStatus = PageStatus.Empty,
                            refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle
                        )
                    }
                } else {
                    val datas: MutableList<Any> = mutableListOf()
                    style?.let { it1 ->
                        it1.content?.forEach { item ->
                            when (item!!.type) {
                                StyleConsts.SWIPER_ADV -> {
                                    val carourseAdvs =
                                        JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.CAROUSEL_ADV)
                                            ?.let { it1 ->
                                                DataUtils.filterAdvPosition(
                                                    it1,
                                                    AdvPosition.AtlasList
                                                )
                                            }
                                    carourseAdvs?.let { advs ->
                                        datas.add(GapLineBinder.Data(""))
                                        datas.add(BannerBinder.BannerInfo(advs))
                                    }
                                }
                                StyleConsts.BANNER_ADV -> {
                                    val bannerAdvs =
                                        JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.BANNER_ADV)
                                            ?.let { it1 ->
                                                DataUtils.filterAdvPosition(
                                                    it1,
                                                    AdvPosition.AtlasList
                                                )
                                            }
                                    val adBinderData: MutableList<Any> = mutableListOf()
                                    bannerAdvs?.let { advs ->
                                        advs.forEach { item ->
                                            run {
                                                adBinderData.add(GapLineBinder.Data(""))
                                                if (item.image_type != 3) {
                                                    adBinderData.add(AdImageBinder.Data(item))
                                                } else {
                                                    adBinderData.add(AdTextBinder.Data(item))
                                                }
                                            }
                                        }
                                        datas.addAll(adBinderData)
                                    }
                                }
                                StyleConsts.ATLAS_GRID -> {
                                    datas.add(GapLineBinder.Data(""))
                                    datas.addAll(it)
                                }
                            }
                        }
                    }

                    setState {
                        copy(
                            pageStatus = PageStatus.Success, data = datas,
                            refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    /**
     * 获取数据
     */
    private fun getList(): Flow<List<Any>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["offset"] = mOffset
            map["limit"] = mLimit
            if (param.contains("免费")) {
                map["price_type"] = 1
                map["sort"] = "updated_at"
            } else if (param.contains("热门")) {
                map["sort"] = "popularity"
            } else {
                map["sort"] = "updated_at"
            }
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .getPictureList(map)
            if (bean.success()) {
                if (bean.list.isNullOrEmpty()) {
                    hasMore = false
                    emit(null)
                } else {
                    val list: MutableList<Any> = mutableListOf()
                    hasMore = bean.total!!.toInt() > mOffset + bean.list.size

                    bean.list.forEach {
                        if (it.price_type == 1) {
                            list.add(AtlasFreeBinder.Free(it))
                        } else {
                            list.add(AtlasPayBinder.Pay(it))
                        }
                    }
                    emit(list)
                }
            } else {
                throw Error(bean.msg())
            }
        }
    }

    override fun providerInitialState(): MoreUiState {
        return MoreUiState()
    }

}