package com.at.video.atlas.ui.search.result

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.atlas.entity.PictureBean
import com.at.video.atlas.entity.SearchInfo
import com.at.video.atlas.http.ApiService
import com.melancholy.network.ApiFactory
import com.video.module_widget.constants.GlobalConsts
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @Create_time: 2022/4/28 19:25
 * @Description:
 */

data class SearchResultUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<PictureBean> = listOf()
): IUiState

sealed class SearchResultUiEvent: IUiEvent {
    object InitLoadData: SearchResultUiEvent()
    object RefreshData: SearchResultUiEvent()
    object LoadDataMore: SearchResultUiEvent()
    data class SearchData(val keyword:String): SearchResultUiEvent()
}

sealed class SearchResultUiEffect: IUiEffect{
    data class SendSearchHis(val keyword: String?):SearchResultUiEffect()
}

@HiltViewModel
class SearchResultViewModel @Inject constructor(application: Application):
    BaseAndroidViewModel<SearchResultUiState, SearchResultUiEvent,
            SearchResultUiEffect>(application) {
    private val mLimit: Int = 9
    private var mOffset: Int = 0
    private var mKeyword: String = ""
    private var hasMore:Boolean = false
    override fun providerInitialState(): SearchResultUiState {
        return SearchResultUiState()
    }

    override fun handleEvent(event: SearchResultUiEvent) {
        when(event){
            is SearchResultUiEvent.InitLoadData -> {
                mKeyword = ""
                setState { copy(pageStatus = PageStatus.Empty,
                    refreshStatus = RefreshStatus.RefreshIdle,
                    loadMoreStatus = LoadMoreStatus.LoadMoreIdle, data = listOf()) }
            }
            is SearchResultUiEvent.RefreshData -> refreshData()
            is SearchResultUiEvent.LoadDataMore -> loadDataMore()
            is SearchResultUiEvent.SearchData -> {
                mKeyword = event.keyword
                initLoadData()
            }
        }
    }

    private fun initLoadData() {
        viewModelScope.launch {
            getSearchList().onStart {
                setState { copy(pageStatus = PageStatus.Loading) }
            }.catch {
                setState { copy(pageStatus = PageStatus.Error(it)) }
            }.collect {
                if(it.isNullOrEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty) }
                } else {
                    setState { copy(pageStatus = PageStatus.Success, data = it,
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
                }
            }
        }
    }

    private fun refreshData() {
        viewModelScope.launch {
            mOffset = 0
            hasMore = false
            getSearchList().onStart {
                setState { copy(refreshStatus = RefreshStatus.RefreshLoading,
                    loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
            }.catch {
                setState { copy(refreshStatus = RefreshStatus.RefreshFail(it)) }
            }.collect {
                if(it.isNullOrEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty,
                        refreshStatus = RefreshStatus.RefreshSuccess) }
                } else {
                    setState { copy(refreshStatus = RefreshStatus.RefreshSuccess, data = it,
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
                }
            }
        }
    }

    private fun loadDataMore() {
        mOffset += mLimit
        viewModelScope.launch {
            getSearchList().onStart {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            }.catch {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(it)) }
            }.collect {
                if(it.isNullOrEmpty()) {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    setState { copy(data = data + it,
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
                }
            }
        }
    }

    private fun getSearchList(): Flow<List<PictureBean>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["keyword"] = mKeyword
            map["limit"] = mLimit
            map["offset"] = mOffset
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .getPictureList(map)
            if(bean.success()) {
                hasMore = bean.total!!.toInt() > mOffset + bean.list?.size!!

                emit(bean.list)
                // if (!bean.data.isNullOrEmpty()){
                //     setEffect {
                //         SearchResultUiEffect.SendSearchHis(mKeyword)
                //     }
                // }
            } else {
                throw Error(bean.message)
            }
        }
    }

}