package com.at.video.game.ui.game

import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.GridLayoutManager
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.game.databinding.GameFragmentBinding
import com.at.video.game.ui.game.binder.AdTextBinder
import com.chad.library.adapter.base.BaseBinderAdapter
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.at.video.game.ui.game.binder.AdImageBinder
import dagger.hilt.android.AndroidEntryPoint

/**
 * @Create_time: 2022/4/27 13:31
 * @Description: 游戏页面
 */

@AndroidEntryPoint
@Route(path = "/game/GameFragment")
class GameFragment : BaseMviFragment<GameFragmentBinding, GameViewModel>(),
    OnRefreshLoadMoreListener, OnItemClickListener {
    private val mAdapter: BaseBinderAdapter = BaseBinderAdapter()

    override fun createViewBinding(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): GameFragmentBinding {
        return GameFragmentBinding.inflate(inflater, container, false)
    }

    override fun getViewLifecycleOwner(): LifecycleOwner = requireActivity()

    override fun getViewModelClass(): Class<GameViewModel> {
        return GameViewModel::class.java
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            setTitleColor(Color.WHITE)
            setTitle("发现")
        }
        requireBinding().apply {
            srlRefresh.setOnRefreshLoadMoreListener(this@GameFragment)
            srlRefresh.setEnableRefresh(false)
            srlRefresh.setEnableLoadMore(false)
            rvList.layoutManager = GridLayoutManager(requireContext(), 1)
            rvList.itemAnimator = null
            rvList.setHasFixedSize(true)
            rvList.adapter = mAdapter
        }
        mAdapter.addItemBinder(
            AdImageBinder(
                showMarginHorizontal = true, showMarginVertical = true
            )
        )
        mAdapter.addItemBinder(AdTextBinder(showMarginHorizontal = true, showMarginVertical = true))
        mAdapter.setOnItemClickListener(this@GameFragment)
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(GameUiState::pageStatus) {
                requireBinding().apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.LOADING
                        }
                        is PageStatus.Error -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.getView(MultiStateView.ViewState.ERROR)!!
                                .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                it.error.message
                            msvView.viewState = MultiStateView.ViewState.ERROR
                        }
                        is PageStatus.Empty -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.EMPTY
                        }
                        is PageStatus.Success -> {
                            srlRefresh.setEnableRefresh(true)
                            srlRefresh.setEnableLoadMore(true)
                            msvView.viewState = MultiStateView.ViewState.CONTENT
                        }
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(GameUiState::refreshStatus) {
                requireBinding().apply {
                    when (it) {
                        is RefreshStatus.RefreshSuccess -> {
                            srlRefresh.finishRefresh()
                        }
                        is RefreshStatus.RefreshLoading -> {
                            if (!srlRefresh.isRefreshing) {
                                srlRefresh.autoRefreshAnimationOnly()
                            }
                        }
                        is RefreshStatus.RefreshFail -> {
                            srlRefresh.finishRefresh(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(GameUiState::loadMoreStatus) {
                requireBinding().apply {
                    when (it) {
                        is LoadMoreStatus.LoadMoreSuccess -> {
                            if (it.hasMore) {
                                srlRefresh.finishLoadMore()
                            } else {
                                srlRefresh.finishLoadMoreWithNoMoreData()
                            }
                        }
                        is LoadMoreStatus.LoadMoreLoading -> {
                            if (!srlRefresh.isLoading) {
                                srlRefresh.autoLoadMoreAnimationOnly()
                            }
                        }
                        is LoadMoreStatus.LoadMoreFail -> {
                            srlRefresh.finishLoadMore(false)
                        }
                        else -> {}
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(GameUiState::data) {
                mAdapter.setDiffNewData(it.toMutableList())
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(GameUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(GameUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(GameUiEvent.LoadDataMore)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val data = adapter.data[position]
        var jumpAddr: String? = null
        when (data) {
            is AdImageBinder.Data -> {
                jumpAddr = data.data.jump_addr
            }
            is AdTextBinder.Data -> {
                jumpAddr = data.data.jump_addr
            }
        }
        jumpAddr?.apply {
            Router.getInstance().build("/webview/WebViewActivity")
                .withString(Constants.KEY_DATA, this).navigation()
        }
    }
}
