package com.at.video.game.ui.game

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.game.ui.game.binder.AdImageBinder
import com.at.video.game.ui.game.binder.AdTextBinder
import com.melancholy.network.ApiFactory
import com.video.module_widget.api.ApiService
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.entity.AdvType
import com.video.module_widget.entity.AdvertisingBean
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.onStart
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @Create_time: 2022/4/27 13:31
 * @Description:
 */

data class GameUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf()
) : IUiState

sealed class GameUiEvent : IUiEvent {
    object InitLoadData : GameUiEvent()
    object RefreshData : GameUiEvent()
    object LoadDataMore : GameUiEvent()
}

sealed class GameUiEffect : IUiEffect

@HiltViewModel
class GameViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<GameUiState, GameUiEvent, GameUiEffect>(application) {
    private val mLimit: Int = 9
    private var mOffset: Int = 0
    private var hasMore: Boolean = false
    override fun providerInitialState(): GameUiState {
        return GameUiState()
    }

    override fun handleEvent(event: GameUiEvent) {
        when (event) {
            is GameUiEvent.InitLoadData -> initLoadData()
            is GameUiEvent.RefreshData -> initLoadData(true)
            is GameUiEvent.LoadDataMore -> loadDataMore()
        }
    }

    private fun initLoadData(isRefresh: Boolean = false) {
        viewModelScope.launch(Dispatchers.IO) {
            if (isRefresh) {
                mOffset = 0
                hasMore = false
                setState {
                    copy(
                        refreshStatus = RefreshStatus.RefreshLoading,
                        loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                    )
                }
            } else {
                setState { copy(pageStatus = PageStatus.Loading) }
            }
            getData().catch {
                setState { copy(pageStatus = PageStatus.Error(Error(it.message))) }
            }.collect {
                val list = it
                if (it.isEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty) }
                } else {
                    val newData: MutableList<Any> = mutableListOf()
                    list.forEach { it1 ->
                        if (it1.image_type == 1 || it1.image_type == 2) {
                            newData.add(AdImageBinder.Data(it1))
                        } else {
                            newData.add(AdTextBinder.Data(it1))
                        }
                    }
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            data = newData,
                            refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    private fun getData(): Flow<List<AdvertisingBean>> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["type"] = AdvType.ColumnAdv.ordinal + 1
            map["library_id"] = GlobalConsts.LIBRARY_ID
            map["limit"] = mLimit
            map["offset"] = mOffset
            val info =
                ApiFactory.getInstance().create(ApiService::class.java).getAdvertisingList(map)
            if (info.success() && info.list != null) {
                hasMore = info.total.toInt() > mOffset + info.list!!.size
                emit(info.list!!)
            } else if (info.code == 502) {
                emit(mutableListOf())
            } else {
                throw Error(info.msg())
            }
        }
    }

    private fun loadDataMore() {
        mOffset += mLimit
        viewModelScope.launch(Dispatchers.IO) {
            getData()
                .onStart {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
                }.catch {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(it)) }
                }.collect {
                    if (it.isEmpty()) {
                        setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                    } else {
                        setState {
                            copy(
                                data = data + it,
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                            )
                        }
                    }
                }
        }
    }
}
