package com.at.video.home.ui.classify

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.common.utils.ToastUtils
import com.at.video.home.entity.*
import com.at.video.home.http.ApiService
import com.at.video.home.ui.classify.binder.ClassifyFreeBinder
import com.at.video.home.ui.classify.binder.ClassifyPayBinder
import com.at.video.home.ui.classify.binder.ConditionsBinder
import com.melancholy.network.ApiFactory
import com.video.module_widget.api.CommonApiService
import com.video.module_widget.binder.*
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.Content
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch

/**
 * @Create_time: 2022/5/26 20:39
 * @Description: 筛选
 */

data class ClassifyUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf(),
    val collectStatus: UpdateMoreCollectStatus = UpdateMoreCollectStatus()
) : IUiState

sealed class ClassifyUiEvent : IUiEvent {
    data class HeaderData(val id: String, val reType: Int, val style: HomeStyle?) :
        ClassifyUiEvent()

    object InitLoadData : ClassifyUiEvent()
    object RefreshData : ClassifyUiEvent()
    object LoadDataMore : ClassifyUiEvent()
    data class Switch(val info: ConditionsBinder.Conditions, val position: Int) : ClassifyUiEvent()
    data class Collect(val info: Any, val position: Int) : ClassifyUiEvent()
}

sealed class ClassifyUiEffect : IUiEffect {
    data class Toast(val toast: String = "") : ClassifyUiEffect()
}

class ClassifyViewModel(application: Application) :
    BaseAndroidViewModel<ClassifyUiState, ClassifyUiEvent, ClassifyUiEffect>(application) {
    private var mIsFirst: Boolean = true
    private var mOffset: Int = 0
    private var mLimit: Int = 9
    private var mId: String = ""
    private var mReType: Int = -1
    private var mArgs: MutableMap<String, Any?> = mutableMapOf()
    private var style: HomeStyle? = null
    private var hasMore: Boolean = false

    override fun providerInitialState(): ClassifyUiState {
        return ClassifyUiState()
    }

    override fun handleEvent(event: ClassifyUiEvent) {
        when (event) {
            is ClassifyUiEvent.HeaderData -> {
                mId = event.id
                mReType = event.reType
                style = event.style
                val resultStyle: Content? = JsonUtils.getHomeContentStyle(style, StyleConsts.RESULT)
                resultStyle?.let {
                    it.config?.apply {
                        mLimit = layout!!.limit
                    }
                }
            }
            is ClassifyUiEvent.InitLoadData -> {
                initLoadData()
            }
            is ClassifyUiEvent.RefreshData -> initLoadData(true)
            is ClassifyUiEvent.LoadDataMore -> loadDataMore()
            is ClassifyUiEvent.Switch -> switch(event.info, event.position)
            is ClassifyUiEvent.Collect -> collect(event.info, event.position)
        }
    }

    @Suppress("UNCHECKED_CAST")
    private fun initLoadData(isRefresh: Boolean = false) {
        if (isRefresh) {
            mOffset = 0
            hasMore = false
        }
        mLimit = getLimit()
        viewModelScope.launch {
            combine(listOf(getConditions(), getList())) {
                val filter: List<Any>? = it[0]
                val data: List<Any>? = it[1]
                if (filter.isNullOrEmpty() && data.isNullOrEmpty()) {
                    setState {
                        copy(
                            pageStatus = PageStatus.Empty,
                            refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle
                        )
                    }
                } else if (!filter.isNullOrEmpty() && !data.isNullOrEmpty()) {
                    if (isRefresh) {
                        setState {
                            copy(
                                refreshStatus = RefreshStatus.RefreshSuccess,
                                data = dataTransfer(filter, data),
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                            )
                        }
                    } else {
                        setState {
                            copy(
                                pageStatus = PageStatus.Success,
                                data = dataTransfer(filter, data),
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                            )
                        }
                    }
                } else if (!filter.isNullOrEmpty() && data.isNullOrEmpty()) {
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            data = dataTransfer(filter, arrayListOf(NoDataBinder.Data("暂无数据"))),
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                        )
                    }
                }
            }.onStart {
                if (isRefresh) {
                    setState {
                        copy(
                            refreshStatus = RefreshStatus.RefreshLoading,
                            loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                        )
                    }
                } else {
                    setState { copy(pageStatus = PageStatus.Loading) }
                }
            }.catch {
                if (isRefresh) {
                    setState { copy(refreshStatus = RefreshStatus.RefreshFail(it)) }
                } else {
                    setState { copy(pageStatus = PageStatus.Error(it)) }
                }
            }.collect()
        }
    }

    private fun loadDataMore() {
        mOffset += getLimit()
        viewModelScope.launch(Dispatchers.IO) {
            getList().onStart {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
            }.catch {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(it)) }
            }.collect {
                if (it.isNullOrEmpty()) {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    setState {
                        copy(
                            data = data + it,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    /**
     * 数据整合
     */
    private fun dataTransfer(condition: List<Any>, data: List<Any>): MutableList<Any> {
        val datas: MutableList<Any> = mutableListOf()
        style?.let {
            it.content?.forEach { item ->
                when (item!!.type) {
//                    StyleConsts.SWIPER_ADV -> {
//                        val carourseAdvs =
//                            JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.CAROUSEL_ADV)
//                        carourseAdvs?.let { advs ->
//                            datas.add(GapLineBinder.Data(""))
//                            datas.add(BannerBinder.BannerInfo(advs))
//                        }
//                    }
//                    StyleConsts.BANNER_ADV -> {
//                        val bannerAdvs =
//                            JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.BANNER_ADV)
//                        val adBinderData: MutableList<Any> = mutableListOf()
//                        bannerAdvs?.let { advs ->
//                            advs.forEach { item ->
//                                run {
//                                    adBinderData.add(GapLineBinder.Data(""))
//                                    if (item.image_type != 3) {
//                                        adBinderData.add(AdImageBinder.Data(item))
//                                    } else {
//                                        adBinderData.add(AdTextBinder.Data(item))
//                                    }
//
//                                }
//                            }
//                            datas.addAll(adBinderData)
//                        }
//                    }
                    StyleConsts.FILTER_BAR -> {
                        datas.addAll(condition)
                    }
                }
            }
        }
        datas.add(GapLineBinder.Data(""))
        datas.addAll(data)
        return datas
    }

    private fun getLimit(): Int {
        val num = if (style != null) {
            StyleUtils.getHomeContentStyle(style!!, StyleConsts.RESULT)?.config?.layout?.limit
        } else 9
        return num ?: 9
    }

    private fun getConditions(): Flow<List<Any>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["category_id"] = mId
            val bean =
                ApiFactory.getInstance().create(ApiService::class.java).getVideoConditions(map)
            if (bean.success()) {
                val list: MutableList<Any> = mutableListOf()
                val mSortMutableList = mutableListOf<ConditionsInfo>()
                mSortMutableList.add(0, ConditionsInfo("全部", null))
                mSortMutableList.add(1, ConditionsInfo("最新", "created_at"))
                mSortMutableList.add(2, ConditionsInfo("最热", "popularity"))
                mSortMutableList.add(3, ConditionsInfo("好评", "score"))
                list.add(ConditionsBinder.Conditions(mSortMutableList, "sort"))

                if (!bean.area_list?.list.isNullOrEmpty()) {
                    val area: MutableList<ConditionsInfo> = mutableListOf()
                    area.add(ConditionsInfo("全部", null))
                    bean.area_list?.list?.forEach {
                        area.add(ConditionsInfo(it.name!!, `val` = it.id))
                    }
                    list.add(ConditionsBinder.Conditions(area, "area_id"))
                }

                if (!bean.sub_category_list?.list.isNullOrEmpty()) {
                    val category: MutableList<ConditionsInfo> = mutableListOf()
                    category.add(ConditionsInfo("全部", null))
                    bean.sub_category_list?.list?.forEach {
                        category.add(ConditionsInfo(it.name!!, it.id))
                    }
                    list.add(ConditionsBinder.Conditions(category, "sub_category_id"))
                }
                if (bean.types_list != null && bean.types_list.list?.size!! > 0) {
                    val typeList: MutableList<ConditionsInfo> = mutableListOf()
                    bean.types_list.list.forEach {
                        typeList.add(
                            ConditionsInfo(
                                it.name!!,
                                it.id
                            )
                        )
                    }
                    if (typeList.size > 0) {
                        typeList.add(
                            0,
                            ConditionsInfo("全部", null)
                        )
                        list.add(ConditionsBinder.Conditions(typeList, "types"))
                    }
                }

                if (!bean.year?.value.isNullOrEmpty()) {
                    val year: MutableList<ConditionsInfo> = mutableListOf()
                    val years = bean.year?.value?.split(",") ?: arrayListOf()
                    year.add(ConditionsInfo("全部", null))
                    years.forEach { it1 ->
                        year.add(ConditionsInfo(it1, it1))
                    }
                    list.add(ConditionsBinder.Conditions(year, "year"))
                }

                if (!bean.year?.value.isNullOrEmpty()) {
                    val lang: MutableList<ConditionsInfo> = mutableListOf()
                    val langs = bean.lang?.value?.split(",") ?: arrayListOf()
                    lang.add(ConditionsInfo("全部", null))
                    langs.forEach { it1 ->
                        lang.add(ConditionsInfo(it1, it1))
                    }
                    list.add(ConditionsBinder.Conditions(lang, "lang"))
                }
                val mCostMutableList = mutableListOf<ConditionsInfo>()
                mCostMutableList.add(0, ConditionsInfo("全部", null))
                mCostMutableList.add(1, ConditionsInfo("金币", "2"))
                mCostMutableList.add(2, ConditionsInfo("免费", "1"))
                list.add(ConditionsBinder.Conditions(mCostMutableList, "price_type"))
                emit(list)
            } else {
                throw Error(bean.message)
            }
        }
    }

    /**
     * 获取数据
     */
    private fun getList(): Flow<List<Any>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["offset"] = mOffset
            map["limit"] = mLimit
            map["category_id"] = mId
            // 添加默认排序
            if (mArgs.isNullOrEmpty()) {
                map["sort"] = "updated_at"
            } else {
                mArgs.forEach {
                    if (it.value != null) {
                        map[it.key] = it.value!!
                    }
                    // 添加默认排序
                    if (it.key == "sort" && it.value == null) {
                        map["sort"] = "updated_at"
                    }
                }
            }

            val bean = ApiFactory.getInstance().create(ApiService::class.java).getFilmList(map)
            if (bean.success()) {
                if (bean.list.isNullOrEmpty()) {
                    hasMore = false
                    emit(null)
                } else {
                    val list: MutableList<Any> = mutableListOf()
                    hasMore = bean.total!!.toInt() > mOffset + bean.list.size

                    bean.list.forEach {
                        if (it.price_type?.toInt() == 1) {
                            list.add(ClassifyFreeBinder.Free(it))
                        } else {
                            list.add(ClassifyPayBinder.Pay(it))
                        }
                    }
                    emit(list)
                }
            } else {
                throw Error(bean.msg())
            }
        }
    }

    // type 0区域 1分类 2上映年代 3费用类型 4排序方式
    private fun switch(info: ConditionsBinder.Conditions, position: Int) {
        viewModelScope.launch(Dispatchers.IO) {
            val oldData = state.value.data.toMutableList()
            val index: Int = oldData.indexOf(info)
            if (index >= 0) {
                mArgs[info.key] = info.list[position].`val`
                oldData[index] = info.copy(index = position)
                setState { copy(data = oldData.toList()) }
                mOffset = 0
                getList().onStart {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
                }.catch {
                }.collect {
                    var size: Int = -1
                    val newData = state.value.data.toMutableList()
                    if (newData.isNotEmpty()) {
                        if (newData[0] is ConditionsBinder.Conditions) {
                            for (i in 0 until newData.size) {
                                if (newData[i] !is ConditionsBinder.Conditions) {
                                    size = i
                                    break
                                }
                            }
                            if (size == -1) {
                                size = newData.size
                            }
                        }
                    }
                    if (it.isNullOrEmpty()) {
                        setState {
                            copy(
                                data = dataTransfer(
                                    data.subList(0, size),
                                    mutableListOf(NoDataBinder.Data("暂无数据"))
                                ),
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                            )
                        }
                    } else {
                        setState {
                            copy(
                                data = dataTransfer(data.subList(0, size), it),
                                loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                            )
                        }
                    }
                }
            }
        }
    }

    private fun collect(data: Any, position: Int) {
        viewModelScope.launch {
            viewModelScope.launch {
                val map: MutableMap<String, Any> = mutableMapOf()
                map["type"] = 1
                map["belong_app_id"] = Constants.APP_ID
                var isCollect: Boolean = false
                var resourceId: String = ""
                when (data) {
                    is ClassifyFreeBinder.Free -> {
                        resourceId = data.info.id.toString()
                        isCollect = data.info.is_collect == 1
                    }
                    is ClassifyPayBinder.Pay -> {
                        resourceId = data.info.id.toString()
                        isCollect = data.info.is_collect == 1
                    }
                    else -> {
                        ToastUtils.show("未获取到正确的参数信息")
                        return@launch
                    }
                }
                map["resource_id"] = resourceId
                val ids = mutableListOf<String>()
                ids.add(resourceId)
                map["resource_id_list"] = ids
                val bean = if (!isCollect) {
                    ApiFactory.getInstance().create(CommonApiService::class.java).collect(map = map)
                } else {
                    ApiFactory.getInstance().create(CommonApiService::class.java)
                        .deleteCollect(map = map)
                }
                if (bean.success()) {
                    setState {
                        copy(
                            collectStatus = UpdateMoreCollectStatus(
                                -1,
                                position,
                                if (isCollect) {
                                    2
                                } else {
                                    1
                                }
                            )
                        )
                    }
                } else {
                    setEffect {
                        ClassifyUiEffect.Toast(bean.msg() ?: "收藏失败")
                    }
                }
            }
        }
    }
}
