package com.at.video.home.ui.home

import android.app.Application
import android.util.Log
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.network.BaseBean
import com.at.common.status.PageStatus
import com.at.module_export_main.IMainService
import com.at.video.home.entity.*
import com.at.video.home.http.ApiService
import com.at.video.home.service.HeartbeatService
import com.melancholy.network.ApiFactory
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.entity.AdvType
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.getFileSize
import com.video.module_widget.imageUrl
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @Create_time: 2022/4/27 12:31
 * @Description:
 */

data class HomeUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val tabs: TabsInfo? = null,
    val notices: List<NoticeBean>? = null
) : IUiState

sealed class HomeUiEvent : IUiEvent {
    object InitLoadData : HomeUiEvent()
    data class CheckUpdate(val versionCode: Long) : HomeUiEvent()
}

sealed class HomeUiEffect : IUiEffect {
    data class Notice(val notices: List<NoticeBean>) : HomeUiEffect()
    data class DialogAd(val list: List<AdvertisingBean>) : HomeUiEffect()
    data class UpdateDialog(val appVersionInfo: AppVersionInfo) : HomeUiEffect()
    object CheckUpdate : HomeUiEffect()
}

@HiltViewModel
class HomeViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<HomeUiState, HomeUiEvent, HomeUiEffect>(application) {

    override fun providerInitialState(): HomeUiState {
        return HomeUiState()
    }

    override fun handleEvent(event: HomeUiEvent) {
        when (event) {
            is HomeUiEvent.InitLoadData -> initData()
            is HomeUiEvent.CheckUpdate -> getAppVersionInfo(event.versionCode)
        }
    }

    private fun initData() {
        viewModelScope.launch(Dispatchers.IO) {
            combine(
                listOf(
                    getAd(AdvType.CarouselAdv.ordinal + 1),
                    getAd(AdvType.BannerAdv.ordinal + 1),
                    getTab()
                )
            ) {
                val carouselAdv = it[0]
                val bannerAdv = it[1]
                val tabs = it[2] as NavigationBean
                val tabList: MutableList<TabInfo> = mutableListOf()
                tabList.add(TabInfo("recommend", "推荐", 100))
                tabList.add(TabInfo("created_at", "最新", 101))
                tabList.add(TabInfo("popularity", "最热", 101))
                tabs.category_list?.list?.let { it1 ->
                    it1.forEach { it2 ->
                        tabList.add(TabInfo(it2.id!!, it2.name!!, 1))
                    }
                }
                tabList.add(TabInfo("person", "影人", 102))
                tabs.album_list?.list?.let { it1 ->
                    it1.forEach { it2 ->
                        tabList.add(TabInfo(it2.id!!, it2.name!!, 103))
                    }
                }
                val info = TabsInfo(
                    tabList,
                    carouselAdv as List<AdvertisingBean>?,
                    bannerAdv as List<AdvertisingBean>?
                )
                HeartbeatService.start()
                setState { copy(pageStatus = PageStatus.Success, tabs = info) }
            }.onStart {
                setState { copy(pageStatus = PageStatus.Loading) }
            }.catch { }.collect()
        }
        getScrollNotice()
        getDialogNoticeAndAd()
    }

    /**
     * 获取弹窗公告和广告
     */
    private fun getDialogNoticeAndAd() {
        viewModelScope.launch {
            combine(
                listOf(
                    getNotice(2, Int.MAX_VALUE),
                    getAd(AdvType.PopAdv.ordinal + 1)
                )
            ) { datas ->
                if (!datas.isNullOrEmpty() && datas.size > 1 && datas[1] != null) {
                    val ads: List<AdvertisingBean> = datas[1] as List<AdvertisingBean>
                    if (ads.isNotEmpty()) {
                        setEffect { HomeUiEffect.DialogAd(ads!!) }
                    }
                }
                val notices: List<NoticeBean>? = datas[0] as List<NoticeBean>?
                if (!notices.isNullOrEmpty()) {
                    setEffect { HomeUiEffect.Notice(notices) }
                }
                setEffect { HomeUiEffect.CheckUpdate }
            }.catch {
                setEffect { HomeUiEffect.CheckUpdate }
            }.collect()
        }
    }

    private fun getAd(type: Int): Flow<List<AdvertisingBean>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["type"] = type
            map["library_id"] = GlobalConsts.LIBRARY_ID
            val bean =
                ApiFactory.getInstance().create(com.video.module_widget.api.ApiService::class.java)
                    .getAdvertisingList(map)
            if (!bean.list.isNullOrEmpty()) {
                emit(bean.list!!)
            } else {
                emit(null)
            }
        }
    }

    /**
     * 获取顶部tab
     */
    private fun getTab(): Flow<NavigationBean> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            emit(ApiFactory.getInstance().create(ApiService::class.java).getTabs(map))
        }
    }

    /**
     * 获取专辑tab
     */
    private fun getAlbumTab(): Flow<BaseBean<List<TabInfo>>> {
        return flow {
            emit(ApiFactory.getInstance().create(ApiService::class.java).getAlbumTab())
        }
    }

    /**
     * 获取滚动广告
     */
    private fun getScrollNotice() {
        viewModelScope.launch {
            getNotice(1, 10).catch {
            }.collect {
                if (!it.isNullOrEmpty()) {
                    setState { copy(notices = it) }
                }
            }
        }
    }

    private fun getNotice(type: Int, limit: Int): Flow<List<NoticeBean>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["style"] = type
            map["source"] = 2
            map["limit"] = limit
            map["offset"] = 0
            map["belong_webmaster_id"] = GlobalConsts.WEB_MASTER_ID
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getNotice(map)
            if (bean.success()) {
                emit(bean.list)
            } else {
                throw Error(bean.message)
            }
        }
    }

    private fun getAppVersionInfo(appCode: Long) {
        val serverCode = IMainService.getMainService().getSysConfig().getAppCode()

        if (serverCode > appCode) {
            val downloadUrl = IMainService.getMainService().getSysConfig().getAppAddress()
            val minoUrl = IMainService.getMainService().getSysConfig().getAppAddress()
            val appContent = IMainService.getMainService().getSysConfig().getAppContent()
            val appName = IMainService.getMainService().getSysConfig().getAppName()
            val forceUpdate = IMainService.getMainService().getSysConfig().isForceUpgrade()
            val versionName = IMainService.getMainService().getSysConfig().getVersionName()
            viewModelScope.launch(Dispatchers.IO) {
                var res = downloadUrl.imageUrl()
                    ?.let {
                        Log.e("Update","$it")
                        ApiFactory.getInstance().create(ApiService::class.java).download(it) }
                var len = res?.contentLength()
                if (len != -1L) {
                    setEffect {
                        HomeUiEffect.UpdateDialog(
                            AppVersionInfo(
                                appName = appName,
                                updateLog = appContent,
                                downUrl = downloadUrl,
                                size = getFileSize(len!!),
                                code = serverCode,
                                forciblyUpdate = forceUpdate,
                                packageName = versionName
                            )
                        )
                    }
                } else {
                    res = ApiFactory.getInstance().create(ApiService::class.java).download(minoUrl)
                    len = res.contentLength()
                    if (len != -1L) {
                        setEffect {
                            HomeUiEffect.UpdateDialog(
                                AppVersionInfo(
                                    appName = appName,
                                    updateLog = appContent,
                                    downUrl = minoUrl,
                                    size = getFileSize(len),
                                    forciblyUpdate = forceUpdate,
                                    packageName = versionName
                                )
                            )
                        }
                    }
                }
            }
        }

//        val appId = Constants.APP_ID
//        viewModelScope.launch {
//            val res = ApiFactory.getInstance().create(ApiService::class.java).getVersionInfo(appId)
//            if(res.success()){
//                if(res.data !== null){
//                    setEffect { HomeUiEffect.UpdateDialog(res.data!!)}
//                }
//            }
//        }
    }
}
