package com.at.video.home.ui.home.recommend

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.home.entity.CategoryInfo
import com.at.video.home.entity.VideoBean
import com.at.video.home.http.ApiService
import com.at.video.home.ui.home.binder.CommonFreeBinder
import com.at.video.home.ui.home.binder.CommonPayBinder
import com.at.video.home.ui.home.binder.FunctionBinder
import com.at.video.home.ui.home.binder.VideoDataBinder
import com.at.video.home.ui.home.recommend.binder.FooterBinder
import com.melancholy.network.ApiFactory
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.NoDataBinder
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.*
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.StyleUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @Create_time: 2022/4/27 16:39
 * @Description:
 */

data class RecommendUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf()
) : IUiState

sealed class RecommendUiEvent : IUiEvent {
    data class HeaderData(
        val id: String,
        val carouselAdv: List<AdvertisingBean>?,
        val bannerAdv: List<AdvertisingBean>?,
//        val banAd: List<AdvertisingInfo>?,
        val style: HomeStyle?
    ) : RecommendUiEvent()

    object InitLoadData : RecommendUiEvent()
    object RefreshData : RecommendUiEvent()
    object LoadDataMore : RecommendUiEvent()
    data class Switch(val data: VideoDataBinder.Data, val position: Int) : RecommendUiEvent()
}

sealed class RecommendUiEffect : IUiEffect {
    data class NoChangeDataEffect(val msg: String) : RecommendUiEffect()
    data class ChangeDataEffect(val data: VideoDataBinder.Data, val pos: Int) : RecommendUiEffect()
}

@HiltViewModel
class RecommendViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<RecommendUiState, RecommendUiEvent, RecommendUiEffect>(application) {
    private lateinit var mId: String
    private var carouselAdv: List<AdvertisingBean>? = null
    private var bannerAdv: List<AdvertisingBean>? = null

    //    private var mBanAd: List<AdvertisingInfo>? = null
    private var style: HomeStyle? = null

    override fun providerInitialState(): RecommendUiState {
        return RecommendUiState()
    }

    override fun handleEvent(event: RecommendUiEvent) {
        when (event) {
            is RecommendUiEvent.HeaderData -> {
                mId = event.id
                carouselAdv = event.carouselAdv
                bannerAdv = event.bannerAdv
                style = event.style
            }
            is RecommendUiEvent.InitLoadData -> {
                initLoadData()
            }
            is RecommendUiEvent.RefreshData -> initLoadData(true)
            is RecommendUiEvent.LoadDataMore -> {}
            is RecommendUiEvent.Switch -> getChangeData(event.data, event.position)
        }
    }

    private fun getLimit(key: String): Int {
        style?.let {
            val homeContentStyle = StyleUtils.getHomeContentStyle(it, key)
            if (homeContentStyle != null) {
                val type = homeContentStyle.config?.layout?.style ?: 1

                return StyleUtils.getGridCountByType(type)
            }
        }
        return 12
    }

    /**
     * 初始化加载数据
     */
    private fun initLoadData(isRefresh: Boolean = false) {
        viewModelScope.launch(Dispatchers.IO) {
            if (isRefresh) {
                setState {
                    copy(
                        refreshStatus = RefreshStatus.RefreshLoading,
                        loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                    )
                }
            } else {
                setState { copy(pageStatus = PageStatus.Loading) }
            }
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["site_id"] = GlobalConsts.SITE_ID
            map["latest_limit"] = getLimit(StyleConsts.DATA_NEWEST)
            map["hot_limit"] = getLimit(StyleConsts.DATA_HOTTEST)
            map["category_video_limit"] = getLimit(StyleConsts.DATA_CATEGORY)
            map["album_video_limit"] = getLimit(StyleConsts.DATA_ALBUM)
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .recommend(map)
            val datas: MutableList<Any> = mutableListOf()
            if (bean.success() || bean.code == 502) {
                val functionBinderData: FunctionBinder.FunctionInfo = FunctionBinder.FunctionInfo(
                    mutableListOf()
                )
                style?.apply {
                    content?.forEach {
                        when (it!!.type) {
                            StyleConsts.SWIPER_ADV -> {
                                val bannerAdvs = carouselAdv?.let { it1 ->
                                    DataUtils.filterAdvPosition(
                                        it1,
                                        AdvPosition.Recommend
                                    )
                                }
                                bannerAdvs?.let { it1 ->
                                    datas.add(BannerBinder.BannerInfo(it1))
                                }
                            }
                            StyleConsts.QUICK_ENTRY -> if (StyleUtils.homeMenuEnable) datas.add(
                                functionBinderData
                            )
                            StyleConsts.BANNER_ADV -> {
                                val bannerAdvs = bannerAdv?.let { it1 ->
                                    DataUtils.filterAdvPosition(it1, AdvPosition.Recommend)
                                }
                                bannerAdvs?.let { it1 ->
                                    val adBinderData: MutableList<Any> = mutableListOf()
                                    it1.forEach { it2 ->
                                        if (it2.image_type != 3) {
                                            adBinderData.add(AdImageBinder.Data(it2))
                                        } else {
                                            adBinderData.add(AdTextBinder.Data(it2))
                                        }
                                    }
                                    datas.addAll(adBinderData)
                                }
                            }
                            StyleConsts.DATA_NEWEST -> {
                                val style = it.config?.layout?.style
                                bean.latest_list?.list?.let { it1 ->
                                    addCategoryByLimit(
                                        datas,
                                        "最近更新",
                                        it1,
                                        style ?: 1,
                                        "created_at",
                                        1,
                                        it
                                    )
                                }
                            }
                            StyleConsts.DATA_HOTTEST -> {
                                val style = it.config?.layout?.style
                                bean.hot_list?.list?.let { it1 ->
                                    addCategoryByLimit(
                                        datas,
                                        "重磅最热",
                                        it1,
                                        style ?: 1,
                                        "popularity",
                                        2,
                                        it
                                    )
                                }
                            }
                            StyleConsts.DATA_CATEGORY -> {
                                val style = it.config?.layout?.style
                                bean.category_list?.forEach { it2 ->
                                    if (it2.list?.list?.isNotEmpty() == true) {
                                        addCategoryByLimit(
                                            datas,
                                            it2.category?.name!!,
                                            it2.list.list,
                                            style ?: 1,
                                            it2.category.id!!,
                                            3,
                                            it
                                        )
                                    }
                                }
                            }
                            StyleConsts.DATA_ALBUM -> {
                                val style = it.config?.layout?.style
                                bean.album_list?.forEach { it2 ->
                                    if (it2.list?.list?.isNotEmpty() == true) {
                                        addCategoryByLimit(
                                            datas,
                                            it2.album?.name!!,
                                            it2.list.list,
                                            style ?: 1,
                                            it2.album.id!!,
                                            4,
                                            it
                                        )
                                    }
                                }
                            }
                        }
                    }
                }
                if (bean.code == 502) {
                    datas.add(NoDataBinder.Data("暂无数据"))
                }
                if (isRefresh) {
                    setState {
                        copy(
                            refreshStatus = RefreshStatus.RefreshSuccess,
                            data = datas,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                        )
                    }
                } else {
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            data = datas,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                        )
                    }
                }
            } else {
                if (isRefresh) {
                    setState { copy(refreshStatus = RefreshStatus.RefreshFail(Error(bean.message))) }
                } else {
                    if (bean.code == 502) {
                        datas.add(NoDataBinder.Data("暂无数据"))
                    } else {
                        datas.add(NoDataBinder.Data(bean.message!!))
                    }
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            data = datas,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                        )
                    }
                }
            }
        }
    }

//    /**
//     * 添加分类
//     * @param datas 数据
//     * @param name 分类名称
//     * @param categories 分类数据
//     * @param type 类型
//     */
//    private fun addCategory(
//        datas: MutableList<Any>, name: String, categories: List<VideoBean>,
//        type: Int, id: String? = null,
//        dataType:Int,
//        style:Content
//    ) {
//        if(categories.isNotEmpty()){
//            val videoData = mutableListOf<Any>()
//            categories.forEach {
//                if (it.price_type?.toInt() == 1) {
//                    videoData.add(CommonFreeBinder.Free(it))
//                } else {
//                    videoData.add(CommonPayBinder.Pay(it))
//                }
//            }
//            val list: MutableList<List<Any>> = mutableListOf()
//            //判断可以换几次
//            val pageSize = StyleUtils.getGridCountByType(type)
//            val count = videoData.size / pageSize
//            for (i: Int in 0 until count) {
//                val start: Int = i * pageSize
//                list.add(videoData.subList(start, start + pageSize))
//            }
//            //最后一次不满pageSize
//            if ((count * pageSize) < videoData.size) {
//                list.add(videoData.subList(count * pageSize, videoData.size))
//            }
//            datas.add(VideoDataBinder.Data(list[0].toMutableList(), name, type, id,list,0,dataType,style))
//        }
//    }

    /**
     * 添加分类
     * @param datas 数据
     * @param name 分类名称
     * @param categories 分类数据
     * @param type 类型
     */
    private fun addCategoryByLimit(
        datas: MutableList<Any>,
        name: String,
        categories: List<VideoBean>,
        type: Int,
        id: String? = null,
        dataType: Int,
        style: Content
    ) {
        if (categories.isNotEmpty()) {
            val videoData = mutableListOf<Any>()
            categories.forEach {
                if (it.price_type?.toInt() == 1) {
                    videoData.add(CommonFreeBinder.Free(it))
                } else {
                    videoData.add(CommonPayBinder.Pay(it))
                }
            }
//            val list: MutableList<List<Any>> = mutableListOf()
//            //判断可以换几次
//            val pageSize = StyleUtils.getGridCountByType(type)
//            val count = videoData.size / pageSize
//            for (i: Int in 0 until count) {
//                val start: Int = i * pageSize
//                list.add(videoData.subList(start, start + pageSize))
//            }
//            //最后一次不满pageSize
//            if ((count * pageSize) < videoData.size) {
//                list.add(videoData.subList(count * pageSize, videoData.size))
//            }
            datas.add(VideoDataBinder.Data(videoData, name, type, id, 0, dataType, style))
        }
    }

    // 获取换一换数据
    @Synchronized
    private fun getChangeData(data: VideoDataBinder.Data, position: Int) {
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["site_id"] = GlobalConsts.SITE_ID
            map["order"] = "random"
            map["limit"] = StyleUtils.getGridCountByType(data.type)
            when (data.dataType) {
                1, 2 -> {
                    map["sort"] = data.id.toString()
                }
                3 -> {
                    map["category_id"] = data.id.toString()
                }
                else ->
                    map["album_id"] = data.id.toString()
            }

            val videos = ApiFactory.getInstance().create(ApiService::class.java).getFilmList(map)
            if (videos.success() && videos.list?.isNotEmpty() == true) {
                val videoData = mutableListOf<Any>()
                videos.list.forEach {
                    if (it.price_type?.toInt() == 1) {
                        videoData.add(CommonFreeBinder.Free(it))
                    } else {
                        videoData.add(CommonPayBinder.Pay(it))
                    }
                }
                setEffect {
                    RecommendUiEffect.ChangeDataEffect(
                        VideoDataBinder.Data(
                            videoData,
                            data.title,
                            data.type,
                            data.id,
                            data.curPos,
                            data.dataType
                        ),
                        position
                    )
                }
            } else {
                setEffect {
                    RecommendUiEffect.NoChangeDataEffect("没有更多数据啦!")
                }
            }
        }
    }

    /**
     *
     */
    @Synchronized
    private suspend fun changeLoadData(
        data: FooterBinder.Data,
        currentIndex: Int,
        offset: Int,
        type: Int,
        prevIndex: Int,
        position: Int,
        id: Int?
    ) {
        val map: MutableMap<String, Any> = mutableMapOf()
        if (id == null) return
        map["limit"] = 18
        map["offset"] = offset
        map["order"] = "desc"
        when (type) {
            0 -> map["sort"] = "updateDate"
            1 -> map["sort"] = "heat"
            2 -> {
                map["sort"] = "updateDate"
                map["categoryId"] = id!!
            }
            3 -> {
                map["sort"] = "updateDate"
                map["albumId"] = id!!
            }
        }
        val bean = ApiFactory.getInstance().create(ApiService::class.java).getChange(map)
        if (bean.success() && !bean.data.isNullOrEmpty()) {
            val categories = bean.data!!
            val list: MutableList<List<CategoryInfo>> = mutableListOf()
            list.addAll(data.data)
            val count = categories.size / 6
            for (i: Int in 0 until count) {
                val start: Int = i * 6
                list.add(categories.subList(start, start + 6))
            }
            if ((count * 6) < categories.size) {
                list.add(categories.subList(count * 6, categories.size))
            }
            change(list, currentIndex, prevIndex, position, type, offset + categories.size, id)
        } else {
            change(data.data, 0, prevIndex, position, type, offset, id)
        }
    }

    /**
     * 换一换
     * @param categories 换一换数据
     * @param currentIndex 要换的数据的index
     * @param prevPosition 正在使用的index
     * @param position 数据的位置
     * @param type 类型
     * @param offset 加载数据类型
     * @param id 查询id
     */
    @Synchronized
    private fun change(
        categories: List<List<CategoryInfo>>,
        currentIndex: Int,
        prevPosition: Int,
        position: Int,
        type: Int,
        offset: Int,
        id: Int?
    ) {
        val datas = state.value.data
        val startIndex: Int = position - prevPosition
        val newDatas: MutableList<Any> = mutableListOf()
        newDatas.addAll(datas.subList(0, startIndex))
        categories[currentIndex].forEach {
//            if(it.chargingCoin == 0) {
//                newDatas.add(CommonFreeBinder.Free(it))
//            } else {
//                newDatas.add(CommonPayBinder.Pay(it))
//            }
        }
        newDatas.add(FooterBinder.Data(categories, currentIndex, offset, type, id))
        if (position + 1 < datas.size) {
            newDatas.addAll(datas.subList(position + 1, datas.size))
        }
        setState { copy(data = newDatas) }
    }
}
