package com.at.video.home.ui.home.star

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.home.entity.Area
import com.at.video.home.entity.StarBean
import com.at.video.home.http.ApiService
import com.at.video.home.ui.home.binder.FunctionBinder
import com.at.video.home.ui.home.star.binder.StarBinder
import com.at.video.home.ui.home.star.binder.StarTypeBinder
import com.melancholy.network.ApiFactory
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.NoDataBinder
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvPosition
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.catch
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.onStart
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @Create_time: 2022/4/27 16:39
 * @Description:
 */

data class StarUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: MutableList<Any> = mutableListOf(),
    val types: MutableList<Area> = mutableListOf(),
    val starDatas: StarTypeBinder.StarType? = null,
    val area: String? = null
) : IUiState

sealed class StarUiEvent : IUiEvent {
    data class HeaderData(
        val id: String,
        val carouselAdv: List<AdvertisingBean>?,
        val bannerAdv: List<AdvertisingBean>?,
//        val banAd: List<AdvertisingInfo>?,
        val style: HomeStyle?
    ) : StarUiEvent()

    object InitLoadData : StarUiEvent()
    object RefreshData : StarUiEvent()
    object LoadDataMore : StarUiEvent()
    data class ChangeType(val area: Area, val position: Int = 0, val curPosition: Int = 0) :
        StarUiEvent()
}

sealed class StarUiEffect : IUiEffect

@HiltViewModel
class StarViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<StarUiState, StarUiEvent, StarUiEffect>(application) {
    private var mOffset: Int = 0
    private var mLimit: Int = 9
    private var area: Area? = null
    private lateinit var mId: String
    private var carouselAdv: List<AdvertisingBean>? = null
    private var bannerAdv: List<AdvertisingBean>? = null

    //    private var mBanAd: List<AdvertisingInfo>? = null
    private var position: Int = 0
    private var curPosition: Int = -1
    private var style: HomeStyle? = null
    private var hasMore = false
    override fun providerInitialState(): StarUiState {
        return StarUiState()
    }

    override fun handleEvent(event: StarUiEvent) {
        when (event) {
            is StarUiEvent.HeaderData -> {
                mId = event.id
                carouselAdv = event.carouselAdv
                bannerAdv = event.bannerAdv
//                mBanAd = event.banAd
                style = event.style
            }
            is StarUiEvent.InitLoadData -> {
                mOffset = 0
                initType()
                curPosition = -1
            }
            is StarUiEvent.RefreshData -> {
                if (area != null) {
                    curPosition = -1
                    hasMore = false
                    refreshData()
                } else {
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            refreshStatus = RefreshStatus.RefreshSuccess,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
            is StarUiEvent.LoadDataMore -> loadDataMore()
            is StarUiEvent.ChangeType -> {
                area = event.area
                position = event.position
                mOffset = 0
                curPosition = event.curPosition
                hasMore = false
                refreshData()
            }
        }
    }

    private fun initType() {
        mLimit = getLimit()
        viewModelScope.launch(Dispatchers.IO) {
            setState { copy(pageStatus = PageStatus.Loading) }
            val map: MutableMap<String, Any> = mutableMapOf()
            val res = ApiFactory.getInstance().create(ApiService::class.java).getStarType(map)
            if (res.success()) {
                if (res.list.isNullOrEmpty()) {
                   initData(null)
                } else {
                    setState { copy(types = res.list.toMutableList()) }
                    area = res.list[0]
                    refreshData()
                }
            } else {
                val datas = mutableListOf<Any>()
                if (res.code == 502) {
                    initData(null)
                } else {
                    datas.add(NoDataBinder.Data(res.message!!))
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            data = datas,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)
                        )
                    }
//                    setState { copy(pageStatus = PageStatus.Error(Throwable(res.message))) }
                }
            }
        }
    }

    private fun getLimit(): Int {
        var limit = 10
        style?.let {
            limit =
                StyleUtils.getHomeContentStyle(it, StyleConsts.PEOPLE_GRID)?.config?.layout?.limit!!
        }
        return limit
    }

    private fun initData(lists: MutableList<Any>?) {
        val datas: MutableList<Any> = mutableListOf()
        style?.apply {
            content?.forEach { it1 ->
                when (it1!!.type) {
                    StyleConsts.SWIPER_ADV -> {
                        val carourseAdvs =
                            JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.CAROUSEL_ADV)
                                ?.let { it2 ->
                                    DataUtils.filterAdvPosition(
                                        it2,
                                        AdvPosition.StarHome
                                    )
                                }

                        carourseAdvs?.let { advs ->
                            datas.add(BannerBinder.BannerInfo(advs))
                        }
                    }
                    StyleConsts.BANNER_ADV -> {
                        val bannerAdvs =
                            JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.BANNER_ADV)
                                ?.let { it2 ->
                                    DataUtils.filterAdvPosition(it2, AdvPosition.StarHome)
                                }
                        val adBinderData: MutableList<Any> = mutableListOf()
                        bannerAdvs?.let { advs ->
                            advs.forEach { item ->
                                run {
                                    if (item.image_type != 3) {
                                        adBinderData.add(AdImageBinder.Data(item))
                                    } else {
                                        adBinderData.add(AdTextBinder.Data(item))
                                    }
                                }
                            }
                            datas.addAll(adBinderData)
                        }
                    }
                    StyleConsts.QUICK_ENTRY -> if (StyleUtils.homeMenuEnable) datas.add(
                        FunctionBinder.FunctionInfo(
                            mutableListOf()
                        )
                    )
                    StyleConsts.DATA_STAR -> {
                        var stars = mutableListOf<Any>()
                        val style = it1.config?.layout?.style ?: 1
                        val color = it1.config?.color ?: "rgba(0,0,0,1)"
                        if (!lists.isNullOrEmpty()) {
                            lists.forEach { item ->
                                run {
                                    stars.add(
                                        StarBinder.Data(
                                            item as StarBean,
                                            type = style,
                                            titleColor = color
                                        )
                                    )
                                }
                            }
                        } else {
                            stars.add(NoDataBinder.Data("暂无数据"))
                        }
                        datas.add(
                            StarTypeBinder.StarType(
                                state.value.types,
                                stars,
                                position,
                                datas.size
                            )
                        )
                    }
                }
            }

            setState {
                copy(
                    pageStatus = PageStatus.Success,
                    refreshStatus = RefreshStatus.RefreshSuccess,
                    data = datas,
                    loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                )
            }
        }
    }

    private fun refreshData() {
        viewModelScope.launch(Dispatchers.IO) {
            getList().onStart {
                setState {
                    copy(
                        refreshStatus = RefreshStatus.RefreshLoading,
                        loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                    )
                }
            }.catch {
                setState { copy(pageStatus = PageStatus.Error(it)) }
            }.collect {
                initData(it as MutableList<Any>?)
            }
        }
    }

    private fun loadDataMore() {
        mOffset += mLimit
        viewModelScope.launch(Dispatchers.IO) {
            getList().onStart {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            }.catch {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
            }.collect {
                var preData = mutableListOf<Any>()
                var pos = 0
                var color = ""
                var type = 1
                state.value.data.forEach { it1 ->
                    run {
                        if (it1 is StarTypeBinder.StarType) {
                            preData.addAll(it1.starData)
                            pos = it1.curPosition
                        }
                    }
                }
                preData.addAll(it as MutableList<StarBean>)
                setState {
                    copy(
                        starDatas = StarTypeBinder.StarType(
                            state.value.types,
                            preData,
                            position,
                            pos
                        ),
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                    )
                }
            }
        }
    }

    /**
     * 获取数据
     */
    private fun getList(): Flow<List<Any>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["offset"] = mOffset
            map["limit"] = mLimit
            map["categ_id"] = area!!.id!!
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getStarList(map)
            if (bean.success() || bean.code == 502) {
                if (bean.list.isNullOrEmpty()) {
                    hasMore = false
                    emit(null)
                } else {
                    hasMore = bean.total!!.toInt() > mOffset + bean.list.size
                    emit(bean.list)
                }
            } else {
                throw Error(bean.msg())
            }
        }
    }
}
