package com.at.video.main.ui.launch

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.View
import androidx.lifecycle.lifecycleScope
import com.at.common.ImageConfig
import com.at.common.global.CacheHelper
import com.at.common.global.Constants
import com.at.common.global.Constants.APP_NAME
import com.at.common.load
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.showAndHideBar
import com.at.common.utils.LoadingUtil
import com.at.video.main.R
import com.at.video.main.databinding.MainActivityLauncherBinding
import com.at.video.main.service.AdService
import com.at.video.main.ui.splash.HintDialog
import com.melancholy.router.api.facade.Postcard
import com.melancholy.router.api.facade.callback.NavigationCallback
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.dialog.CommonTipDialog
import com.video.module_widget.dialog.MessageDialog
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.Splash
import com.video.module_widget.imageUrl
import com.video.module_widget.utils.GlideImageConfig
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.loadByGlide
import dagger.hilt.android.AndroidEntryPoint
import java.io.File
import java.lang.ref.WeakReference


/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/8/3 上午10:10
 * @UpdateUser: ly
 * @UpdateDate: 2023年01月11日14:39:06
 * @UpdateRemark: 更新说明：整理代码结构和移除多余代码
 * @Version: 1.0
 */
@AndroidEntryPoint
class LaunchActivity : BaseMviActivity<MainActivityLauncherBinding, LaunchViewModel>(),
    View.OnClickListener {

    /**
     * 默认广告地址
     */
    var defaultAdv: String = ""

    /**
     * 默认闪屏logo
     */
    var defaultLogo: String = ""

    override fun createViewBinding(): MainActivityLauncherBinding =
        MainActivityLauncherBinding.inflate(layoutInflater)

    override fun getViewModelClass(): Class<LaunchViewModel> = LaunchViewModel::class.java


    override fun onCreate(savedInstanceState: Bundle?) {
        /**
         * 解决启动页面重复启动
         */
        if (!isTaskRoot) {
            finish()
            return
        }
        if (intent.flags and Intent.FLAG_ACTIVITY_BROUGHT_TO_FRONT != 0) {
            finish()
            return
        }
        super.onCreate(savedInstanceState)
    }

    override fun immersiveBar() {
        showAndHideBar(false)
    }

    override fun initialize(savedInstanceState: Bundle?) {

        requireBinding().ctvTimer.setOnClickListener(this)

        requireBinding().apply {
            if (isFirst()) {
                firstOpenApp()
            } else {
                lastOpenApp()
            }
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(LaunchUiEvent.InitData)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is LaunchUiEffect.Jump -> jump()
                    is LaunchUiEffect.Loading -> LoadingUtil.showLoading(
                        this@LaunchActivity, it.content
                    )
                    is LaunchUiEffect.Dismiss -> LoadingUtil.dismiss()
                    is LaunchUiEffect.Finish -> {
                        HintDialog.showDialog(supportFragmentManager, ::callback)
                    }
                    is LaunchUiEffect.EmptyAdv -> {
                        loadDefaultPic()
                    }
                    is LaunchUiEffect.AppError -> {
                        CommonTipDialog.showDialog(supportFragmentManager,
                            it.message,
                            closeLabel = "退出应用",
                            object : CommonTipDialog.CloseCallback {
                                override fun close() {
                                    finish()
                                }

                            })
                    }
                    else -> {}
                }
            }
        }

        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(LaunchUiState::data) {
                if (it != null) {
                    requireBinding().apply {
                        ivAd.tag = it
                        if (it.show_time != 0) {
                            ctvTimer.visibility = View.VISIBLE
                            ctvTimer.setSumTime(it.show_time * 1000)
                            ctvTimer.startTimer()
                        } else {
                            jump()
                        }

                        loadSplashAds(it)
                    }
                }
            }
        }
    }

    /**
     * 加载闪屏页广告
     * @param advBean 广告信息
     */
    private fun loadSplashAds(advBean: AdvertisingBean) {
        requireBinding().apply {
            divLogo.visibility = View.GONE
            if (advBean.addr != null) {
                ivAd.load(
                    ImageConfig(
                        File(advBean.addr),
                        placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default,
                        errorId = com.video.module_widget.R.drawable.common_ic_ad_default
                    )
                )
                Log.e("IMG", "加载背景图1：${advBean.image_url.imageUrl()}")
            } else {
                ivAd.loadByGlide(
                    GlideImageConfig(
                        defaultAdv.imageUrl(),
                        placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default,
                        errorId = com.video.module_widget.R.drawable.common_ic_ad_default
                    ), WeakReference(this@LaunchActivity)
                )
                Log.e("IMG", "加载背景图2：${defaultAdv.imageUrl()}")
                jump()
            }
        }

    }


    /**
     * 获取配置回调方法
     * @param isFinish 是否完成
     */
    private fun callback(isFinish: Boolean) {
        if (isFinish) {
            finish()
        } else {
            getViewModel().sendEvent(LaunchUiEvent.InitData)
        }
    }

    /**
     * 第一打开app 操作
     */
    private fun firstOpenApp() {
        requireBinding().apply {
            ivAd.visibility = View.GONE
            divLogo.loadByGlide(
                GlideImageConfig(
                    R.mipmap.ic_launcher,
                    placeholderId = com.video.module_widget.R.drawable.common_ic_image_altas_default,
                    errorId = com.video.module_widget.R.drawable.common_ic_image_altas_default
                ), WeakReference(this@LaunchActivity)
            )
            //                Log.e("APp", APP_NAME)
            val templateWebViewContent =
                "<html><head></head><body><p style=\"display:inline-block;text-align-last:center;text-align:center;font-size:16px;;width:100%\">${APP_NAME}</p></body></html>"
            //                         tvStart.text =  Html.fromHtml(templateWebViewContent)
            tvStart.isVerticalScrollBarEnabled = false
            tvStart.setBackgroundColor(0)
            tvStart.background.alpha = 0
            tvStart.loadData(
                templateWebViewContent, "text/html", "utf-8"
            )
        }
        CacheHelper.getInstance().putBoolean("isFirst", false)
    }

    /**
     * 再次打开app 操作
     */
    private fun lastOpenApp() {
        requireBinding().apply {
            var splash = JsonUtils.getStyleBean<Splash>(StyleConsts.LAUNCH)
            if (splash == null) {
                val jsonStr = JsonUtils.getDataFromAssetFile(application, "data.json")
                if (jsonStr != null) {
                    val template = jsonStr.getJSONObject("template")
                    val launch = template.getString(StyleConsts.LAUNCH)
                    splash = JsonUtils.getObjectData<Splash>(launch)
                }
            }
            splash?.config?.apply {
                text?.let { lt ->
                    val templateWebViewContent =
                        "<html><head></head><body><p style=\"display:inline-block;text-align-last:${lt.align};text-align:${lt.align};font-size:${lt.size}px;color:${lt.color};width:100%\">${lt.content}</p></body></html>"
                    tvStart.isVerticalScrollBarEnabled = false
                    tvStart.setBackgroundColor(0)
                    tvStart.background.alpha = 0
                    tvStart.loadData(
                        templateWebViewContent, "text/html", "utf-8"
                    )
                }
                logo?.apply {
                    if (mode == 1) {
                        when (style) {
                            1 -> {
                                divLogo.setImageResource(com.video.module_widget.R.drawable.vector_drawable_logo_1)
                            }
                            2 -> {
                                divLogo.setImageResource(com.video.module_widget.R.drawable.vector_drawable_logo_2)
                            }
                            3 -> {
                                divLogo.setImageResource(com.video.module_widget.R.drawable.vector_drawable_logo_3)
                            }

                        }
                    } else {
                        defaultLogo = image ?: ""
                    }
                }
                bg?.let {
                    defaultAdv = if (it.mode == 1) {
                        it.upload!!
                    } else {
                        it.input!!
                    }
                }
            }
        }

    }


    /**
     * 没有广告的时候，加载默认图
     */
    private fun loadDefaultPic() {
        requireBinding().apply {
            if (defaultAdv.isNotEmpty()) {
                ivAd.loadByGlide(
                    GlideImageConfig(
                        defaultAdv.imageUrl(),
                        placeholderId = com.video.module_widget.R.drawable.common_ic_ad_default,
                        errorId = com.video.module_widget.R.drawable.common_ic_ad_default
                    ), WeakReference(this@LaunchActivity)
                )
                Log.e("IMG", "加载背景图3：${defaultAdv.imageUrl()}")
            } else {
                ivAd.visibility = View.GONE
            }
            if (defaultLogo.isNotEmpty()) {
                divLogo.loadByGlide(
                    GlideImageConfig(
                        defaultLogo.imageUrl(),
                        placeholderId = com.video.module_widget.R.drawable.common_ic_image_altas_default,
                        errorId = com.video.module_widget.R.drawable.common_ic_image_altas_default
                    ), WeakReference(this@LaunchActivity)
                )
                defaultLogo.let { it1 -> Log.e("ICON1", it1.imageUrl().toString()) }
            }
            // 没有广告，默认执行倒计时，但是不展示跳过按钮
            ctvTimer.visibility = View.GONE
            ctvTimer.setSumTime(2 * 1000)
            ctvTimer.startTimer()
        }
    }

    /**
     * 跳转主页方法
     * @param jumpAddr 跳转地址（进入首页以后）
     */
    @JvmOverloads
    private fun jump(jumpAddr: String? = null) {
        Router.getInstance().build("/main/MainActivity")
            .navigation(this, callback = object : NavigationCallback {
                override fun onArrival(postcard: Postcard) {
                    if (jumpAddr != null) {
                        Router.getInstance().build("/webview/WebViewActivity")
                            .withString(Constants.KEY_DATA, jumpAddr).navigation(
                                this@LaunchActivity,
                                callback = object : NavigationCallback {
                                    override fun onArrival(postcard: Postcard) {
                                        window.setBackgroundDrawable(null)
                                        startService(Intent(application, AdService::class.java))
                                        finish()
                                    }

                                    override fun onFound(postcard: Postcard) {
                                    }

                                    override fun onInterrupt(postcard: Postcard) {
                                    }

                                    override fun onLost(postcard: Postcard) {
                                    }
                                })
                    } else {
                        startService(Intent(application, AdService::class.java))
                        finish()
                    }
                }

                override fun onFound(postcard: Postcard) {

                }

                override fun onInterrupt(postcard: Postcard) {

                }

                override fun onLost(postcard: Postcard) {

                }

            })
    }

    /**
     * 是否第一次打开app
     */
    private fun isFirst(): Boolean {
        return CacheHelper.getInstance().getBoolean("isFirst", true)
    }


    override fun onClick(view: View) {
        if (view.id == R.id.ctv_timer) {
            jump()
        } else if (view.id == R.id.iv_ad) {
            val ad: AdvertisingBean? = requireBinding().ivAd.tag as? AdvertisingBean
            if (ad != null) {
                requireBinding().ctvTimer.stopTimer()
                jump(ad.jump_addr)
            }
        }
    }
}