package com.at.video.main.ui.launch

import android.app.Application
import android.content.Context
import android.util.Log
import androidx.lifecycle.viewModelScope
import com.at.common.global.CacheHelper
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.PageStatus
import com.at.video.export.mine.IMineService
import com.at.video.main.ConfigManager
import com.at.video.main.entity.*
import com.at.video.main.http.ApiService
import com.melancholy.network.ApiFactory
import com.melancholy.utils.AppUtils
import com.video.module_widget.api.CommonApiService
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.exception.ExceptionHandler
import com.video.module_widget.fileUrl
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.PhoneUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.TrackersUtil
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import org.json.JSONObject
import java.io.*
import javax.inject.Inject

/**
 *
 * @Description: LaunchViewModel
 * @CreateDate: 2022/8/3 上午10:11
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/8/3 上午10:11
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

data class LaunchUiState(
    val pageStatus: PageStatus = PageStatus.Empty,
    val data: AdvertisingBean? = null
) : IUiState

sealed class LaunchUiEvent : IUiEvent {
    object InitData : LaunchUiEvent()
}

sealed class LaunchUiEffect : IUiEffect {
    data class Loading(val content: String = "") : LaunchUiEffect()
    object Dismiss : LaunchUiEffect()
    object Jump : LaunchUiEffect()
    object Finish : LaunchUiEffect()
    object EmptyAdv : LaunchUiEffect()
    data class Toast(val msg: String) : LaunchUiEffect()
    data class AppError(val message: String) : LaunchUiEffect()
}

@HiltViewModel
class LaunchViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<LaunchUiState, LaunchUiEvent, LaunchUiEffect>(application) {

    private var webmasterId = ""
    private var templateId = ""

    override fun providerInitialState(): LaunchUiState {
        return LaunchUiState()
    }

    override fun handleEvent(event: LaunchUiEvent) {
        when (event) {
            is LaunchUiEvent.InitData -> initData()
        }
    }

    private fun initData() {
//        Log.e("DeviceInfo", "initData: ${PhoneUtils.toPrintDeviceInfo()}")
        viewModelScope.launch(Dispatchers.IO) {
            getDomains().onStart {
                setEffect {
                    LaunchUiEffect.Loading("获取配置信息中...")
                }
            }.catch { e ->
//                Log.e("error:domians-》", "${e.message}")
                setEffect { LaunchUiEffect.Dismiss }
                setEffect { LaunchUiEffect.Finish }
            }.flatMapConcat {
                Constants.BASE_URL = "${it.cms_proxy?.main_domain}/"
                ApiFactory.getInstance().setBaseUrl(Constants.BASE_URL)
                GlobalConsts.DATA_REPORT = it.data_report?.main_domain.toString()
                CacheHelper.getInstance().putString(
                    GlobalConsts.CMS_ALTERNATE_DOMAINS,
                    it.cms_proxy?.alternate_domains?.joinToString(";")
                )
                CacheHelper.getInstance().putString(
                    GlobalConsts.ROOT_ALTERNATE_DOMAINS,
                    it.config_distribute?.alternate_domains?.joinToString(";")
                )
                CacheHelper.getInstance().putString(
                    GlobalConsts.DATA_REPORT_ALTERNATE_DOMAINS,
                    it.data_report?.alternate_domains?.joinToString(";")
                )
//                Log.e("config", "initData: 开始获取应用信息")
                getAppInfo()
            }.collect {
//                Log.e("config", "initData: 应用信息获取成功")
                if (it != null && it.application !== null) {
                    webmasterId = it.application.webmaster_id.toString()
                    GlobalConsts.WEB_MASTER_ID = webmasterId
                    templateId = it.application.template_id.toString()
                    GlobalConsts.LIBRARY_ID = it.application.advert_id.toString()
                    GlobalConsts.CMS_URL = "${it.application.cms_address}/"
                    GlobalConsts.SHARE_URL = it.application.share_address
                    dataReport()
                    reportLog()
                    combine(
                        listOf(
                            getTemplate(),
                            getSplashAd(),
                            getAgreements(),
                            getEmailConfig()
                        )
                    ) { it1 ->
//                        Log.e("config", "initData: 其他配置信息")

                        val agreements = it1[2] as List<Agreement>
                        var userProctol = ""
                        var privatePolice = ""
                        if (agreements.isNotEmpty()) {
                            for (i in agreements.indices) {
                                val item = agreements[i]
                                if (item.agreement_type == 1) {
                                    userProctol = item.content
                                } else {
                                    privatePolice = item.content
                                }
                            }
                        }
                        ConfigManager.instance.saveConfig(
                            AppResInfo(
                                userProctol,
                                privatePolice,
                                it.application_version?.android?.version_code ?: 0,
                                it.application_version?.android?.version_content ?: "",
                                it.application_version?.android?.pkg_address ?: "",
                                it.application.advert_id ?: "",
                                it.application.content?.video ?: "",
                                it.application.content?.image ?: "",
                                it.application.content?.article ?: "",
                                it1[3] as Boolean,
                                it.application_version?.android?.minio_pkg_address?.fileUrl() ?: "",
                                it.application.name ?: "",
                                it.application_version?.android?.upgrade ?: 3,
                                it.application_version?.android?.version_name ?: ""
                            )
                        )
//                        Log.e("config", "initData: 保存config")
                        val adv = it1[1] as AdvertisingBean?
//                    HeartbeatService.start()
                        if (adv == null) {
                            setEffect { LaunchUiEffect.EmptyAdv }
                        } else {
                            setState { copy(data = adv) }
                        }
                        setEffect { LaunchUiEffect.Dismiss }
                    }.onStart {}.catch {
//                        loadDataFromAsset()
//                        Log.e("config", "initData: 获取异常")
                        setEffect { LaunchUiEffect.Dismiss }
                        setEffect { LaunchUiEffect.Finish }
                    }.collect()
                } else {
                    setEffect { LaunchUiEffect.Dismiss }
//                    setEffect { LaunchUiEffect.Finish }
                }
            }
        }
    }

    private fun getEmailConfig(): Flow<Boolean> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["belong_app_id"] = Constants.APP_ID
            map["belong_webmaster_id"] = GlobalConsts.WEB_MASTER_ID
            map["not_exist_error"] = 0
            val res =
                ApiFactory.getInstance().create(ApiService::class.java).isEmailConfig(map = map)
//            Log.e("getEmailConfig",res.toString())

            emit(res.success())
        }
    }

    /**
     * 检测服务器地址
     */
    private fun checkServerUrl() {
        viewModelScope.launch(Dispatchers.IO) {
//            val startTime = System.currentTimeMillis()
            setEffect { LaunchUiEffect.Loading("检测服务器中...") }

            var baseUrls = Constants.BASE_URLS
            if (baseUrls.size == 1) { // 默认只有一条配置时，不需要检测地址
                initData()
            } else {
                val deffreds = baseUrls.map {
                    val url = if (it.endsWith("/")) it else "$it/"
                    async {
                        testUrl(url)
                    }
                }

                fun cancelAll() = deffreds.forEach { it.cancel() }
                val urls = arrayListOf<String>()
                for (def in deffreds) {
                    val res = def.await()
                    if (res.isNotEmpty()) {
                        urls.add(res)
//                            Log.e("request","time:${System.currentTimeMillis() - startTime},res:${res}")
//                    cancelAll()
                    }
                }
//                    Log.e("request","replace url:${urls[0]}")

                if (urls.isNotEmpty()) {
                    CacheHelper.getInstance().putString(
                        AppUtils.getPackageName(getApplication()),
                        urls.joinToString(",")
                    )
                    Constants.BASE_URL = urls[0]
                    ApiFactory.getInstance().setBaseUrl(urls[0])
                    initData()
                }
            }
        }
    }

    private suspend fun testUrl(url: String): String {
//        Log.e("request","url:${url}")
        try {
            val res = ApiFactory.getInstance().create(ApiService::class.java)
                .ping("${url}ping").execute()
            val body = res.body()
            val contentLength = body?.msg
            if (body != null && body.success()) {
                return url
            } else {
                return ""
            }
        } catch (e: Exception) {
            return ""
        }
    }

    /**
     * 获取配置信息
     */
    @Suppress("UNCHECKED_CAST")
//    private fun getConfig(): Flow<ConfigInfo> {
//        return combine(listOf(getSettings(), getAgreements())) {
//            ConfigManager.instance.saveConfig(it[0] as SettingsInfo, it[1] as AgreementInfo)
//            ConfigManager.instance.getSysConfig()
//        }.catch {
//            loadDataFromAsset()
//            if (ConfigManager.instance.hasConfig()) {
//                emit(ConfigManager.instance.getSysConfig())
//            } else {
//                throw Error()
//            }
//        }
//    }

    /**
     * 获取相关配置文件
     */
    private fun getSettings(): Flow<SettingsInfo> {
        return flow {
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getSettings()
            if (bean.success() && bean.data != null) {
                Constants.SERVICE_URL = bean.data!!.customerServiceAddr
                try {
                    getStyle("http://192.168.2.6/schema/")
                } catch (e: Exception) {
                    throw Error(e.message)
                }
                emit(bean.data!!)
            } else {
                throw Error(bean.msg)
            }
        }
    }

    /**
     * 获取协议相关信息
     */
    private fun getAgreements(): Flow<List<Agreement>> {
        return flow {
            val map = mutableMapOf<String, Any>()
//            map["belong_app_id"] = Constants.APP_ID
            map["belong_webmaster_id"] = webmasterId
            val bean =
                ApiFactory.getInstance().create(ApiService::class.java).getAgreement(map = map)
//            Log.e("getAgreements",bean.toString())
            if (bean.success() && bean.agreements != null) {
                emit(bean.agreements)
            } else {
                throw Error(bean.message)
            }
        }
    }

    /**
     * 获取域名
     */
    private fun getDomains(): Flow<Domain> {
        return flow {
            val map = mutableMapOf<String, Any>()
            map["types"] = arrayListOf(1, 2, 3)
            Log.e("BaseUrl", "getDomains: ${Constants.BASE_URL}")
            val url = CacheHelper.getInstance().getString(Constants.KEY_DATA_3, "")
            try {
                val res = ApiFactory.getInstance().create(ApiService::class.java)
                    .getDomains(url = "${url}v1/config/domain", map = map)
//            Log.e("getDomains",res.toString())
                if (res.success() && res.domain != null) {
                    emit(res.domain)
                } else {
                    throw Error(res.message)
                }
            }catch (e:Exception){
                Log.e("error:domians-》", "${e.message}")
                throw Error(e.message)
            }
        }
    }

    /**
     * 获取应用信息
     */
    private fun getAppInfo(): Flow<AppInfo?> {
        return flow {
//            val map = mutableMapOf<String, Any>()
//            map["belong_app_id"] = Constants.APP_ID
//            map["version"] = ""
            try {
                val res = ApiFactory.getInstance().create(ApiService::class.java).getAppInfo()
//                Log.e("APPINFO",res.toString())
                if (res.success()) {
                    emit(res)
                } else {
                    emit(null)
                    if (res.code == 403) {
                        setEffect { LaunchUiEffect.AppError(res.message!!) }
                    } else {
                        setEffect { LaunchUiEffect.Dismiss }
                        setEffect { LaunchUiEffect.Finish }
                    }
                }
            } catch (e: Exception) {
                Log.e("appinfo", "getAppInfo: ${e.message}")
                setEffect { LaunchUiEffect.Dismiss }
                setEffect { LaunchUiEffect.Finish }
            }
        }
    }

    /**
     * 数据上报
     */
    private fun dataReport() {
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["startup_record"] = DataUtils.dataReport(0).startup_record!!
            try {
                val res = ApiFactory.getInstance().create(CommonApiService::class.java)
                    .dataReport(GlobalConsts.DATA_REPORT + "/v1/basic_record/create", map)
                if (res.success()) {
                    Log.d("DATA_REPORT", "上报成功")
                } else {
                    Log.d("DATA_REPORT", "上报失败")
                }
            } catch (e: Exception) {
                Log.e("Exception", e.message.toString())
            }
        }
    }

    /**
     * 上报错误日志，会自动检查是否上传完成
     */
    private fun reportLog() {
        try {
            ExceptionHandler.uploadExceptionAgain(getApplication<Application>().applicationContext)
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    /**
     * 获取显示的广告
     */
    private fun getSplashAd(): Flow<AdvertisingBean?> {
        return flow {
            try {
                val json: String = CacheHelper.getInstance().getString("ads", null)!!
                val advs = JsonUtils.getListData<AdvertisingBean>(json)
                var info: AdvertisingBean? = null
                if (advs!!.size > 1) {
                    val num = (Math.random() * advs.size).toInt()
                    info = advs[num]
//                    Log.e("adv", "当前显示广告:${num}")
//                    Log.e("size", "广告数量:${advs.size}")
                } else {
                    info = advs[0]
                }
                emit(info)
            } catch (e: Exception) {
                emit(null)
            }
        }
    }

    private suspend fun getStyle(url: String) {
        try {
            val path = CacheHelper.getInstance().getString("theme_style", "")
            val res = ApiFactory.getInstance().create(ApiService::class.java).download(url)
            val inputStream = res.byteStream()
            val fos = FileOutputStream(path)
            var bis: BufferedInputStream? = null
            val length: Int
            try {
                var len = 0
                var buffer = ByteArray(2048 * 2)
                while (inputStream.read(buffer).also { len = it } != -1) {
                    fos.write(buffer, 0, len)
                }
                bis = BufferedInputStream(inputStream)
                length = bis.available()
                val data = ByteArray(length)
                bis.read(data)
                if (data.isNotEmpty()) {
                    parseData(JSONObject(String(data)))
                } else {
                    throw Error("NoDataError")
                }
            } catch (e: Exception) {
//                Log.e("Launch", "saveError")
                throw Error("saveError")
            } finally {
                fos.close()
                inputStream.close()
                bis?.close()
            }
        } catch (e: Exception) {
//            Log.e("Launch", "fileError")
            throw Error("fileError")
        }
    }

    private fun loadDataFromAsset() {
        val jsonStr = getAssertsFile(getApplication(), "data.json")?.let { String(it) }
        if (!jsonStr.isNullOrEmpty()) {
            val json = JSONObject(jsonStr)
            parseData(json)
//            Log.e("APP", "加载本地模板")
        }
    }

    private fun parseData(json: JSONObject) {
        try {
            val template: JSONObject = json.get("template") as JSONObject
            // 全局样式
            CacheHelper.getInstance()
                .putString(StyleConsts.LAUNCH, template.getString(StyleConsts.LAUNCH))
            CacheHelper.getInstance()
                .putString(StyleConsts.GLOBAL, json.getString(StyleConsts.GLOBAL))
            CacheHelper.getInstance()
                .putString(StyleConsts.HOME, template.getString(StyleConsts.HOME))
            CacheHelper.getInstance().putString(
                StyleConsts.HOME_RECOMMEND,
                template.getString(
                    StyleConsts.HOME_RECOMMEND
                )
            )
            CacheHelper.getInstance()
                .putString(StyleConsts.HOME_NEW, template.getString(StyleConsts.HOME_NEW))
            CacheHelper.getInstance()
                .putString(StyleConsts.HOME_HOST, template.getString(StyleConsts.HOME_HOST))
            CacheHelper.getInstance().putString(
                StyleConsts.HOME_CATEGORY,
                template.getString(
                    StyleConsts.HOME_CATEGORY
                )
            )
            CacheHelper.getInstance()
                .putString(StyleConsts.HOME_ALBUM, template.getString(StyleConsts.HOME_ALBUM))
            CacheHelper.getInstance()
                .putString(StyleConsts.HOME_PEOPLE, template.getString(StyleConsts.HOME_PEOPLE))
            CacheHelper.getInstance()
                .putString(StyleConsts.VIDEO_FILTER, template.getString(StyleConsts.VIDEO_FILTER))
            CacheHelper.getInstance()
                .putString(StyleConsts.VIDEO_MORE, template.getString(StyleConsts.VIDEO_MORE))
            CacheHelper.getInstance()
                .putString(StyleConsts.ARTICLE_HOME, template.getString(StyleConsts.ARTICLE_HOME))
            CacheHelper.getInstance().putString(
                StyleConsts.ARTICLE_CATEGORY,
                template.getString(
                    StyleConsts.ARTICLE_CATEGORY
                )
            )
            CacheHelper.getInstance().putString(
                StyleConsts.ARTICLE_DETAIL,
                template.getString(
                    StyleConsts.ARTICLE_DETAIL
                )
            )
            CacheHelper.getInstance()
                .putString(StyleConsts.ARTICLE_READ, template.getString(StyleConsts.ARTICLE_READ))
            CacheHelper.getInstance()
                .putString(StyleConsts.ATLAS_HOME, template.getString(StyleConsts.ATLAS_HOME))
            CacheHelper.getInstance().putString(
                StyleConsts.ATLAS_CATEGORY,
                template.getString(
                    StyleConsts.ATLAS_CATEGORY
                )
            )
            CacheHelper.getInstance()
                .putString(StyleConsts.ATLAS_DETAIL, template.getString(StyleConsts.ATLAS_DETAIL))
            CacheHelper.getInstance()
                .putString(StyleConsts.ATLAS_MORE, template.getString(StyleConsts.ATLAS_MORE))
            CacheHelper.getInstance()
                .putString(StyleConsts.VIDEO_PLAYER, template.getString(StyleConsts.VIDEO_PLAYER))
            CacheHelper.getInstance().putString(
                StyleConsts.VIDEO_SYNOPSIS,
                template.getString(
                    StyleConsts.VIDEO_SYNOPSIS
                )
            )
            CacheHelper.getInstance()
                .putString(StyleConsts.ACTOR_DETAILS, template.getString(StyleConsts.ACTOR_DETAILS))
            CacheHelper.getInstance().putString(
                StyleConsts.ACTOR_DETAIL_MORE,
                template.getString(
                    StyleConsts.ACTOR_DETAIL_MORE
                )
            )

            StyleUtils.initGlobalConst()
        } catch (e: Exception) {
//            Log.e("APP", "解析异常：" + e.message)
            e.printStackTrace()
        }
    }

    private fun getTemplate(): Flow<TemplateBean> {
        return flow {
            val map = mutableMapOf<String, Any>()
            map["id"] = templateId
//            Log.e("APP", "获取模板前")

            val res = ApiFactory.getInstance().create(ApiService::class.java).getTemplate(map)
//            Log.e("APP", "成功获取模板")
            if (res.success()) {
                val json = JSONObject(res.data?.replace("\\", "") ?: "")
                parseData(json)
//                Log.e("APP", "获取模板")
            } else {
                loadDataFromAsset()
            }
            emit(res)
        }
    }

    private fun getAssertsFile(context: Context, fileName: String?): ByteArray? {
        var inputStream: InputStream? = null
        val assetManager = context.assets
        try {
            inputStream = assetManager.open(fileName!!)
            var bis: BufferedInputStream? = null
            val length: Int
            try {
                bis = BufferedInputStream(inputStream)
                length = bis.available()
                val data = ByteArray(length)
                bis.read(data)
                return data
            } catch (e: IOException) {
            } finally {
                if (bis != null) {
                    try {
                        bis.close()
                    } catch (e: Exception) {
                    }
                }
            }
            return null
        } catch (e: IOException) {
            e.printStackTrace()
        }
        return null
    }
}
