package com.at.video.main.ui.main

import android.Manifest
import android.graphics.Color
import android.os.Build
import android.os.Bundle
import android.view.Gravity
import android.view.ViewGroup.LayoutParams.MATCH_PARENT
import android.view.ViewGroup.LayoutParams.WRAP_CONTENT
import android.widget.FrameLayout
import android.widget.RadioGroup
import androidx.annotation.RequiresApi
import androidx.fragment.app.Fragment
import androidx.lifecycle.lifecycleScope
import com.at.common.immersiveStatusBar
import com.at.common.mvi.BaseMviActivity
import com.at.common.setLightStatusBar
import com.at.common.utils.ToastUtils
import com.at.video.main.R
import com.at.video.main.databinding.MainActivityMainBinding
import com.hjq.permissions.OnPermissionCallback
import com.hjq.permissions.XXPermissions
import com.jeremyliao.liveeventbus.LiveEventBus
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.LoginSuccessEvent
import com.video.module_widget.entity.home.HomeTopBarConfig
import com.video.module_widget.provider.IPlayerService
import com.video.module_widget.utils.LocationManagerUtils
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint
import kotlinx.coroutines.flow.collect

/**
 * @Create_time: 2022/4/27 11:59
 * @Description: 主页面
 */

@AndroidEntryPoint
@Route(path = "/main/MainActivity")
class MainActivity :
    BaseMviActivity<MainActivityMainBinding, MainViewModel>(),
    RadioGroup.OnCheckedChangeListener {
    private lateinit var mPermissions: XXPermissions
    private var mSelectedIndex: Int = -1
    private val mPaths: MutableList<String> = mutableListOf()

    override fun createViewBinding(): MainActivityMainBinding {
        return MainActivityMainBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<MainViewModel> {
        return MainViewModel::class.java
    }

    override fun immersiveBar() {
        immersiveStatusBar()
        setLightStatusBar(true)
    }

    @RequiresApi(Build.VERSION_CODES.M)
    override fun initialize(savedInstanceState: Bundle?) {
        requireBinding().rgBottom.setOnCheckedChangeListener(this)
        mSelectedIndex = savedInstanceState?.getInt("index") ?: -1
        mPermissions = XXPermissions.with(this)

        initPaths()
        if (savedInstanceState == null) {
            showFragment(0)
        }
        initRadioGroup(R.id.rb_home)
        parseStyle()
        mPermissions.permission(
            Manifest.permission.ACCESS_FINE_LOCATION,
            Manifest.permission.ACCESS_COARSE_LOCATION
        ).request(object :
            OnPermissionCallback {
            override fun onGranted(permissions: MutableList<String>?, all: Boolean) {
                if (all) {
                    LocationManagerUtils.getNowLocation()
                } else {
                    ToastUtils.show("定位权限获取失败")
                }
            }

            override fun onDenied(permissions: MutableList<String>?, never: Boolean) {
                super.onDenied(permissions, never)
            }
        })

        // 订阅登录成功消息
        LiveEventBus.get<LoginSuccessEvent>("login_success").observe(this) {
            getViewModel().sendEvent(MainUiEvent.GetTrackInfo)
        }
        // 提前做种
        IPlayerService.getInstance().initTorrentEngine()
    }

    override fun loadData() {
        getViewModel().sendEvent(MainUiEvent.InitData)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is MainUiEffect.BottomAd -> if (home.bottomFloatAdv?.show == true) {
                        createBottomAd(it.info)
                    }
                    else -> {}
                }
            }
        }
    }

    private lateinit var home: HomeTopBarConfig

    private fun parseStyle() {
        home = StyleUtils.getStyleBean(StyleConsts.HOME)!!
    }

    /**
     * 创建底部广告
     */
    private fun createBottomAd(info: AdvertisingBean) {
        val params: FrameLayout.LayoutParams = FrameLayout.LayoutParams(MATCH_PARENT, WRAP_CONTENT)
        params.gravity = Gravity.BOTTOM
        params.bottomMargin = resources.getDimensionPixelSize(com.at.common.R.dimen.dp_54)
        if (info.image_type == 3) {
            val adView = com.at.video.main.widget.BottomTxtAdView(this)
            adView.layoutParams = params
            adView.setData(info, home.bottomFloatAdv)
            requireBinding().flRootContent.addView(adView)
        } else {
            val addView = com.at.video.main.widget.BottomAdView(this)
            addView.layoutParams = params
            addView.setData(info, home.bottomFloatAdv)
            requireBinding().flRootContent.addView(addView)
        }
    }

    /**
     * 初始化Fragment
     */
    private fun initPaths() {
        mPaths.add("/home/HomeFragment")
//        mPaths.add("/member/MemberFragment")
        mPaths.add("/game/GameFragment")
        mPaths.add("/mine/MineFragment")
    }

    /**
     * 显示某一个Fragment
     */
    private fun showFragment(index: Int) {
        if (mSelectedIndex != index) {
            val transaction = supportFragmentManager.beginTransaction()
            if (mSelectedIndex >= 0) {
                val path: String = mPaths[mSelectedIndex]
                val prevFragment: Fragment? = supportFragmentManager.findFragmentByTag(path)
                if (prevFragment != null) {
                    transaction.hide(prevFragment)
                }
            }
            val path: String = mPaths[index]
            val currentFragment: Fragment = supportFragmentManager.findFragmentByTag(path)
                ?: Router.getInstance().build(path).navigation() as Fragment
            if (!currentFragment.isAdded) {
                // 添加Fragment到Activity并设置Tag，方便重启之后找
                transaction.add(R.id.fl_content, currentFragment, path)
            }
            transaction.show(currentFragment)
            transaction.commitNowAllowingStateLoss()
            mSelectedIndex = index
        }
    }

    @RequiresApi(Build.VERSION_CODES.M)
    override fun onCheckedChanged(p0: RadioGroup, checkId: Int) {
        initRadioGroup(checkId)
        when (checkId) {
            R.id.rb_home -> showFragment(0)
//            R.id.rb_member -> showFragment(1)
            R.id.rb_game -> showFragment(1)
            R.id.rb_mine -> showFragment(2)
        }
    }

    @RequiresApi(Build.VERSION_CODES.M)
    private fun initRadioGroup(checkId: Int) {
        requireBinding().apply {
//            rbHome.typeface = Typeface.DEFAULT
//            rbHome.textSize= DensityUtils.px2sp(this@MainActivity, resources.getDimension(com.at.common.R.dimen.sp_12).toInt())
            rbHome.setTextColor(resources.getColor(R.color.color_FFB1B1B1))
            rbHome.compoundDrawableTintList = StyleUtils.createColorStateList(
                resources.getColor(R.color.color_FFB1B1B1),
                StyleUtils.getPrimaryColor().toArgb()
            )
//            rbMember.typeface = Typeface.DEFAULT
//            rbMember.textSize= DensityUtils.px2sp(this@MainActivity, resources.getDimension(com.at.common.R.dimen.sp_12).toInt())
//            rbMember.setTextColor(resources.getColor(R.color.color_FFB1B1B1))
//            rbMember.compoundDrawableTintList = StyleUtils.createColorStateList(resources.getColor(R.color.color_FFB1B1B1),StyleUtils.getPrimaryColor().toArgb())
//            rbGame.typeface = Typeface.DEFAULT
//            rbGame.textSize= DensityUtils.px2sp(this@MainActivity, resources.getDimension(com.at.common.R.dimen.sp_12).toInt())
            rbGame.setTextColor(resources.getColor(R.color.color_FFB1B1B1))
            rbGame.compoundDrawableTintList = StyleUtils.createColorStateList(
                resources.getColor(R.color.color_FFB1B1B1),
                StyleUtils.getPrimaryColor().toArgb()
            )

//            rbMine.typeface = Typeface.DEFAULT
//            rbMine.textSize= DensityUtils.px2sp(this@MainActivity, resources.getDimension(com.at.common.R.dimen.sp_12).toInt())
            rbMine.setTextColor(resources.getColor(R.color.color_FFB1B1B1))
            rbMine.compoundDrawableTintList = StyleUtils.createColorStateList(
                resources.getColor(R.color.color_FFB1B1B1),
                StyleUtils.getPrimaryColor().toArgb()
            )
            flRootContent.setBackgroundColor(Color.WHITE)
            when (checkId) {
                R.id.rb_home -> {
//                    rbHome.typeface = Typeface.DEFAULT_BOLD
                    rbHome.setTextColor(StyleUtils.getPrimaryColor().toArgb())
//                    rbHome.textSize= DensityUtils.px2sp(this@MainActivity, resources.getDimension(com.at.common.R.dimen.sp_14).toInt())
                }
//                R.id.rb_member -> {
//                    flRootContent.setBackgroundColor(Color.parseColor("#222222"))
//                    rbMember.setTextColor(StyleUtils.getPrimaryColor().toArgb())
// //                    rbMember.typeface = Typeface.DEFAULT_BOLD
// //                    rbMember.textSize= DensityUtils.px2sp(this@MainActivity, resources.getDimension(com.at.common.R.dimen.sp_14).toInt())
//                }
                R.id.rb_game -> {
                    rbGame.setTextColor(StyleUtils.getPrimaryColor().toArgb())
//                    rbGame.typeface = Typeface.DEFAULT_BOLD
//                    rbGame.textSize= DensityUtils.px2sp(this@MainActivity, resources.getDimension(com.at.common.R.dimen.sp_14).toInt())
                }
                R.id.rb_mine -> {
                    flRootContent.setBackgroundColor(Color.parseColor("#F8F8F8"))
                    rbMine.setTextColor(StyleUtils.getPrimaryColor().toArgb())
//                    rbMine.typeface = Typeface.DEFAULT_BOLD
//                    rbMine.textSize= DensityUtils.px2sp(this@MainActivity, resources.getDimension(com.at.common.R.dimen.sp_14).toInt())
                }
            }
        }
    }

    override fun onSaveInstanceState(outState: Bundle) {
        outState.putInt("index", mSelectedIndex)
        super.onSaveInstanceState(outState)
    }
}
