package com.at.video.main.ui.splash

import android.app.Application
import android.content.Context
import android.util.Log
import androidx.lifecycle.viewModelScope
import com.at.common.global.CacheHelper
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.utils.JsonUtils
import com.at.video.main.ConfigManager
import com.at.video.main.entity.AdvertisingInfo
import com.at.video.main.entity.AgreementInfo
import com.at.video.main.entity.ConfigInfo
import com.at.video.main.entity.SettingsInfo
import com.at.video.main.http.ApiService
import com.melancholy.network.ApiFactory
import com.melancholy.utils.AppUtils
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.utils.StyleUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.async
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import org.json.JSONObject
import java.io.BufferedInputStream
import java.io.FileOutputStream
import java.io.IOException
import java.io.InputStream

/**
 * @Create_time: 2022/5/30 19:06
 * @Description:
 */

data class SplashUiState(
    val data: AdvertisingInfo? = null
) : IUiState

sealed class SplashUiEvent : IUiEvent {
    object InitData : SplashUiEvent()
}

sealed class SplashUiEffect : IUiEffect {
    data class Loading(val content: String = "") : SplashUiEffect()
    object Dismiss : SplashUiEffect()
    object Jump : SplashUiEffect()
    object Finish : SplashUiEffect()
    data class Toast(val msg: String) : SplashUiEffect()
}

class SplashViewModel(application: Application) : BaseAndroidViewModel<SplashUiState, SplashUiEvent,
    SplashUiEffect>(application) {

    override fun providerInitialState(): SplashUiState {
        return SplashUiState()
    }

    override fun handleEvent(event: SplashUiEvent) {
        when (event) {
            is SplashUiEvent.InitData -> loadDataFromAsset()
        }
    }

    private fun initData() {
        viewModelScope.launch(Dispatchers.IO) {
            getConfig().onStart {
                setEffect { SplashUiEffect.Loading("获取配置信息中...") }
            }.catch {
                setEffect { SplashUiEffect.Dismiss }
                setEffect { SplashUiEffect.Finish }
            }.flatMapConcat {
                getSplashAd()
            }.collect {
                setEffect { SplashUiEffect.Dismiss }
                setState { copy(data = it) }
                if (it == null) {
                    setEffect { SplashUiEffect.Jump }
                }
            }
        }
    }

    /**
     * 检测服务器地址
     */
    private fun checkServerUrl() {
        viewModelScope.launch(Dispatchers.IO) {
//            val startTime = System.currentTimeMillis()
            setEffect { SplashUiEffect.Loading("检测服务器中...") }

            var baseUrls = Constants.BASE_URLS
            if (baseUrls.size == 1) { // 默认只有一条配置时，不需要检测地址
                initData()
            } else {
                val deffreds = baseUrls.map {
                    val url = if (it.endsWith("/")) it else "$it/"
                    async {
                        testUrl(url)
                    }
                }
                fun cancelAll() = deffreds.forEach { it.cancel() }
                val urls = arrayListOf<String>()
                for (def in deffreds) {
                    val res = def.await()
                    if (res.isNotEmpty()) {
                        urls.add(res)
//                            Log.e("request","time:${System.currentTimeMillis() - startTime},res:${res}")
//                    cancelAll()
                    }
                }
//                    Log.e("request","replace url:${urls[0]}")

                if (urls.isNotEmpty()) {
                    CacheHelper.getInstance().putString(AppUtils.getPackageName(getApplication()), urls.joinToString(","))
                    Constants.BASE_URL = urls[0]
                    ApiFactory.getInstance().setBaseUrl(urls[0] + Constants.URL_PATH_SUFFIX)
                    initData()
                }
            }
        }
    }

    private suspend fun testUrl(url: String): String {
//        Log.e("request","url:${url}")
        try {
            val res = ApiFactory.getInstance().create(ApiService::class.java).ping("${url}${Constants.URL_PATH_SUFFIX}ping").execute()
            val body = res.body()
            val contentLength = body?.msg
            if (body != null && body.success()) {
                return url
            } else {
                return ""
            }
        } catch (e: Exception) {
            return ""
        }
    }

    /**
     * 获取配置信息
     */
    @Suppress("UNCHECKED_CAST")
    private fun getConfig(): Flow<ConfigInfo> {
        return combine(listOf(getSettings(), getAgreements())) {
//            ConfigManager.instance.saveConfig(it[0] as SettingsInfo, it[1] as AgreementInfo)
            ConfigManager.instance.getSysConfig()
        }.catch {
            loadDataFromAsset()
            if (ConfigManager.instance.hasConfig()) {
                emit(ConfigManager.instance.getSysConfig())
            } else {
                throw Error()
            }
        }
    }

    /**
     * 获取相关配置文件
     */
    private fun getSettings(): Flow<SettingsInfo> {
        return flow {
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getSettings()
            if (bean.success() && bean.data != null) {
                Constants.SERVICE_URL = bean.data!!.customerServiceAddr
                try {
                    getStyle("http://192.168.2.6/schema/")
                } catch (e: Exception) {
                    throw Error(e.message)
                }
                emit(bean.data!!)
            } else {
                throw Error(bean.msg)
            }
        }
    }

    /**
     * 获取协议相关信息
     */
    private fun getAgreements(): Flow<AgreementInfo> {
        return flow {
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getAgreements()
            if (bean.success() && bean.data != null) {
                emit(bean.data!!)
            } else {
                throw Error(bean.msg)
            }
        }
    }

    /**
     * 获取显示的广告
     */
    private fun getSplashAd(): Flow<AdvertisingInfo?> {
        return flow {
            val info: AdvertisingInfo? = try {
                val json: String = CacheHelper.getInstance().getString("ad", null)!!
                JsonUtils.fromJson(json)!!
            } catch (e: Exception) {
                null
            }
            emit(info)
        }
    }

    private suspend fun getStyle(url: String) {
        try {
            val path = CacheHelper.getInstance().getString("theme_style", "")
            val res = ApiFactory.getInstance().create(ApiService::class.java).download(url)
            val inputStream = res.byteStream()
            val fos = FileOutputStream(path)
            var bis: BufferedInputStream? = null
            val length: Int
            try {
                var len = 0
                var buffer = ByteArray(2048 * 2)
                while (inputStream.read(buffer).also { len = it } != -1) {
                    fos.write(buffer, 0, len)
                }
                bis = BufferedInputStream(inputStream)
                length = bis.available()
                val data = ByteArray(length)
                bis.read(data)
                if (data.isNotEmpty()) {
                    parseData(JSONObject(String(data)))
                } else {
                    throw Error("NoDataError")
                }
            } catch (e: Exception) {
                Log.e("Launch", "saveError")
                throw Error("saveError")
            } finally {
                fos.close()
                inputStream.close()
                bis?.close()
            }
        } catch (e: Exception) {
            Log.e("Launch", "fileError")
            throw Error("fileError")
        }
    }

    private fun loadDataFromAsset() {
        val jsonStr = getAssertsFile(getApplication(), "data.json")?.let { String(it) }
        if (!jsonStr.isNullOrEmpty()) {
            val json = JSONObject(jsonStr)
            parseData(json)
        }
    }

    private fun parseData(json: JSONObject) {
        val template: JSONObject = json.get("template") as JSONObject
        // 全局样式
        CacheHelper.getInstance().putString(StyleConsts.LAUNCH, template.getString(StyleConsts.LAUNCH))
        CacheHelper.getInstance().putString(StyleConsts.GLOBAL, json.getString(StyleConsts.GLOBAL))
        CacheHelper.getInstance().putString(StyleConsts.HOME, template.getString(StyleConsts.HOME))
        CacheHelper.getInstance().putString(StyleConsts.HOME_RECOMMEND, template.getString(StyleConsts.HOME_RECOMMEND))
        CacheHelper.getInstance().putString(StyleConsts.HOME_NEW, template.getString(StyleConsts.HOME_NEW))
        CacheHelper.getInstance().putString(StyleConsts.HOME_HOST, template.getString(StyleConsts.HOME_HOST))
        CacheHelper.getInstance().putString(StyleConsts.HOME_CATEGORY, template.getString(StyleConsts.HOME_CATEGORY))
        CacheHelper.getInstance().putString(StyleConsts.HOME_ALBUM, template.getString(StyleConsts.HOME_ALBUM))
        CacheHelper.getInstance().putString(StyleConsts.HOME_PEOPLE, template.getString(StyleConsts.HOME_PEOPLE))
        CacheHelper.getInstance().putString(StyleConsts.VIDEO_FILTER, template.getString(StyleConsts.VIDEO_FILTER))
        CacheHelper.getInstance().putString(StyleConsts.VIDEO_MORE, template.getString(StyleConsts.VIDEO_MORE))
        CacheHelper.getInstance().putString(StyleConsts.ARTICLE_HOME, template.getString(StyleConsts.ARTICLE_HOME))
        CacheHelper.getInstance().putString(StyleConsts.ARTICLE_CATEGORY, template.getString(StyleConsts.ARTICLE_CATEGORY))
        CacheHelper.getInstance().putString(StyleConsts.ARTICLE_DETAIL, template.getString(StyleConsts.ARTICLE_DETAIL))
        val read: JSONObject = template.getJSONObject(StyleConsts.ARTICLE_READ)
        CacheHelper.getInstance().putString(StyleConsts.ARTICLE_READ, read.getInt("style").toString())
        CacheHelper.getInstance().putString(StyleConsts.ATLAS_HOME, template.getString(StyleConsts.ATLAS_HOME))
        CacheHelper.getInstance().putString(StyleConsts.ATLAS_CATEGORY, template.getString(StyleConsts.ATLAS_CATEGORY))
        CacheHelper.getInstance().putString(StyleConsts.ATLAS_DETAIL, template.getString(StyleConsts.ATLAS_DETAIL))
        CacheHelper.getInstance().putString(StyleConsts.ATLAS_MORE, template.getString(StyleConsts.ATLAS_MORE))
        CacheHelper.getInstance().putString(StyleConsts.VIDEO_PLAYER, template.getString(StyleConsts.VIDEO_PLAYER))
        CacheHelper.getInstance().putString(StyleConsts.VIDEO_SYNOPSIS, template.getString(StyleConsts.VIDEO_SYNOPSIS))
        CacheHelper.getInstance().putString(StyleConsts.ACTOR_DETAILS, template.getString(StyleConsts.ACTOR_DETAILS))
        CacheHelper.getInstance().putString(StyleConsts.ACTOR_DETAIL_MORE, template.getString(StyleConsts.ACTOR_DETAIL_MORE))

        StyleUtils.initGlobalConst()
    }

    private fun getAssertsFile(context: Context, fileName: String?): ByteArray? {
        var inputStream: InputStream? = null
        val assetManager = context.assets
        try {
            inputStream = assetManager.open(fileName!!)
            var bis: BufferedInputStream? = null
            val length: Int
            try {
                bis = BufferedInputStream(inputStream)
                length = bis.available()
                val data = ByteArray(length)
                bis.read(data)
                return data
            } catch (e: IOException) {
            } finally {
                if (bis != null) {
                    try {
                        bis.close()
                    } catch (e: Exception) {
                    }
                }
            }
            return null
        } catch (e: IOException) {
            e.printStackTrace()
        }
        return null
    }
}
