package com.at.video.member.ui.detail

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.member.http.ApiService
import com.melancholy.network.ApiFactory
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

/**
 * @Create_time: 2022/5/5 17:36
 * @Description:
 */

data class DetailUiState(
    val pageStatus: PageStatus = PageStatus.Empty,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf()
) : IUiState

sealed class DetailUiEvent : IUiEvent {
    object InitLoadData : DetailUiEvent()
    object RefreshData : DetailUiEvent()
    object LoadMoreData : DetailUiEvent()
}

sealed class DetailUiEffect : IUiEffect

class DetailViewModel(application: Application) : BaseAndroidViewModel<DetailUiState, DetailUiEvent,
    DetailUiEffect>(application) {
    private var mLimit = 9
    private var mOffset = 0
    override fun providerInitialState(): DetailUiState {
        return DetailUiState()
    }

    override fun handleEvent(event: DetailUiEvent) {
        when (event) {
            is DetailUiEvent.InitLoadData -> {
                initData()
            }
            is DetailUiEvent.RefreshData -> { refreshData() }
            is DetailUiEvent.LoadMoreData -> loadMoreData()
        }
    }

    private fun loadMoreData() {
        viewModelScope.launch(Dispatchers.IO) {
            setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            val map = mutableMapOf<String, Any>()
            map["offset"] = mOffset
            map["limit"] = mLimit
            val info =
                ApiFactory.getInstance().create(ApiService::class.java).getPurchasesDetails(map)
            if (info.success()) {
                val list = info.data?.list
                if (list.isNullOrEmpty()) {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    if (list.size < mLimit) {
                        setState { copy(data = (data + list), loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                    } else {
                        mOffset += mLimit
                        setState { copy(data = (data + list), loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(true)) }
                    }
                }
            } else {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(Error(info.msg))) }
            }
        }
    }

    private fun refreshData() {
        viewModelScope.launch(Dispatchers.IO) {
            setState { copy(refreshStatus = RefreshStatus.RefreshLoading, loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
            mOffset = 0
            val map = mutableMapOf<String, Any>()
            map["offset"] = mOffset
            map["limit"] = mLimit
            val info =
                ApiFactory.getInstance().create(ApiService::class.java).getPurchasesDetails(map)
            if (info.success()) {
                val list = info.data?.list
                if (list.isNullOrEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty, refreshStatus = RefreshStatus.RefreshSuccess, loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    val newList: MutableList<Any> = mutableListOf()
                    newList.add(0)
                    newList.addAll(list)
                    if (list.size < mLimit) {
                        setState { copy(refreshStatus = RefreshStatus.RefreshSuccess, data = newList, loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                    } else {
                        mOffset += mLimit
                        setState { copy(refreshStatus = RefreshStatus.RefreshSuccess, data = newList, loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(true)) }
                    }
                }
            } else {
                setState { copy(refreshStatus = RefreshStatus.RefreshFail(Error(info.msg))) }
            }
        }
    }

    private fun initData() {
        viewModelScope.launch(Dispatchers.IO) {
            setState { copy(pageStatus = PageStatus.Loading) }
            val map = mutableMapOf<String, Any>()
            map["offset"] = mOffset
            map["limit"] = mLimit
            val newList: MutableList<Any> = mutableListOf()
            newList.add(0)
            setState { copy(pageStatus = PageStatus.Success, data = newList, loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
//            val info =
//                ApiFactory.getInstance().create(ApiService::class.java).getPurchasesDetails(map)
//            if (info.success()){
//                val list = info.data?.list
//                if (list.isNullOrEmpty()){
//                    setState { copy(pageStatus=PageStatus.Empty) }
//                }else{
//                    val newList: MutableList<Any> = mutableListOf()
//                    newList.add(0)
//                    newList.addAll(list)
//                    if (list.size < mLimit){
//                        setState { copy(pageStatus=PageStatus.Success,data= newList, loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
//                    }else{
//                        mOffset+=mLimit
//                        setState { copy(pageStatus=PageStatus.Success, data = newList, loadMoreStatus =LoadMoreStatus.LoadMoreSuccess(true)) }
//                    }
//                }
//
//            }else{
//                setState { copy(pageStatus=PageStatus.Error(Error(info.msg))) }
//            }
        }
    }
}
