package com.at.video.mine.ui.collection

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.mine.entity.CollectBean
import com.at.video.mine.entity.CollectInfo
import com.at.video.mine.http.ApiService
import com.melancholy.network.ApiFactory
import com.video.module_widget.api.CommonApiService
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @description:
 * @author:  stone
 * @email: 可以编辑
 * @date :   2022/5/9 上午10:00
 */

data class MoviesUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<CollectBean> = listOf(),
    val isDelete: Boolean = false,
    val isSelectAll: Boolean = false
): IUiState

sealed class MoviesUiEvent: IUiEvent {
    data class HeadData(val type: Int): MoviesUiEvent()
    object InitLoadData: MoviesUiEvent()
    object RefreshData: MoviesUiEvent()
    object LoadDataMore: MoviesUiEvent()
    data class SelectOrUnselect(val info: CollectBean, val position: Int): MoviesUiEvent()
    object Cancel: MoviesUiEvent()
    object Delete: MoviesUiEvent()
    object SelectAll: MoviesUiEvent()
    object CancelAll: MoviesUiEvent()
}

sealed class MoviesUiEffect: IUiEffect {

    data class Loading(val content: String = ""): MoviesUiEffect()
    object Dismiss: MoviesUiEffect()
    data class Toast(val msg: String): MoviesUiEffect()
}

@HiltViewModel
class MoviesViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<MoviesUiState, MoviesUiEvent, MoviesUiEffect>(application) {
    private var mOffset: Int = 0
    private val mLimit: Int = 9
    private var mType: Int = 1
    private var mSelectList: MutableMap<Int, CollectBean> = mutableMapOf()

    override fun providerInitialState(): MoviesUiState {
        return MoviesUiState()
    }

    override fun handleEvent(event: MoviesUiEvent) {
        when(event) {
            is MoviesUiEvent.HeadData -> mType = event.type
            is MoviesUiEvent.InitLoadData -> initLoadData()
            is MoviesUiEvent.RefreshData -> refreshData()
            is MoviesUiEvent.LoadDataMore -> loadDataMore()
            is MoviesUiEvent.SelectOrUnselect -> selectOrUnselect(event.info, event.position)
            is MoviesUiEvent.Cancel -> cancel()
            is MoviesUiEvent.SelectAll -> selectAll()
            is MoviesUiEvent.Delete -> delete()
            is MoviesUiEvent.CancelAll -> cancelAll()
        }
    }

    private fun initLoadData() {
        viewModelScope.launch {
            getCollect().onStart {
                setState { copy(pageStatus = PageStatus.Loading) }
            }.catch {
                setState { copy(pageStatus = PageStatus.Error(it)) }
            }.collect {
                if(it.isNullOrEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty) }
                } else {
                    val hasMore: Boolean = it.size == mLimit
                    if(hasMore) {
                        mOffset += mLimit
                    }
                    setState { copy(pageStatus = PageStatus.Success, data = it,
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
                }
            }
        }
    }

    private fun refreshData() {
        viewModelScope.launch {
            mOffset = 0
            getCollect().onStart {
                setState { copy(refreshStatus = RefreshStatus.RefreshLoading,
                    loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
            }.catch {
                setState { copy(refreshStatus = RefreshStatus.RefreshFail(Error(it))) }
            }.collect {
                if(it.isNullOrEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty,
                        refreshStatus = RefreshStatus.RefreshSuccess) }
                } else {
                    val hasMore: Boolean = it.size == mLimit
                    if(hasMore) {
                        mOffset += mLimit
                    }
                    setState { copy(refreshStatus = RefreshStatus.RefreshSuccess, data = it,
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
                }
            }
        }
    }

    private fun loadDataMore() {
        viewModelScope.launch {
            getCollect().onStart {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            }.catch {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(Error(it))) }
            }.collect {
                if(it.isNullOrEmpty()) {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    val hasMore: Boolean = it.size == mLimit
                    if(hasMore) {
                        mOffset += mLimit
                    }
                    setState { copy(data = data + it,
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
                }
            }
        }
    }

    private fun getCollect(): Flow<List<CollectBean>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["type"] = mType
            map["offset"] = mOffset
            map["limit"] = mLimit
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .collectList(map = map)
            if(bean.success()) {
                if(bean.list.isNullOrEmpty()) {
                    emit(null)
                } else {
                    emit(bean.list)
                }
            } else {
                throw Error(bean.message)
            }
        }
    }

    private fun selectOrUnselect(info: CollectBean, position: Int) {
        viewModelScope.launch(Dispatchers.Default) {
            val newInfo = if(info.is_selected) {
                mSelectList.remove(position)
                info.copy(is_selected = false)
            } else {
                val new = info.copy(is_selected = true)
                mSelectList[position] = new
                new
            }
            val newData: MutableList<CollectBean> = state.value.data.toMutableList()
            newData[position] = newInfo
            setState { copy(data = newData.toList()) }
            setState { copy(data = newData.toList(), isSelectAll = newData.size == mSelectList.size,
                isDelete = mSelectList.isNotEmpty()) }
        }
    }

    private fun cancel() {
        mSelectList.clear()

    }

    private fun selectAll() {
        viewModelScope.launch(Dispatchers.Default) {
            val newData: MutableList<CollectBean> = state.value.data.toMutableList()
            if(newData.size == 0) {
                return@launch
            }
            if(newData.size == mSelectList.size) {
                val iterator: MutableIterator<MutableMap.MutableEntry<Int, CollectBean>> =
                    mSelectList.iterator()
                while (iterator.hasNext()) {
                    val value: MutableMap.MutableEntry<Int, CollectBean> = iterator.next()
                    newData[value.key] = value.value.copy(is_selected = false)
                    iterator.remove()
                }
            } else {
                newData.forEachIndexed { index, collectInfo ->
                    if(!mSelectList.containsKey(index)) {
                        val newInfo: CollectBean = collectInfo.copy(is_selected = true)
                        mSelectList[index] = newInfo
                        newData[index] = newInfo
                    }
                }
            }
            setState { copy(data = newData.toList(), isSelectAll = newData.size == mSelectList.size,
                isDelete = mSelectList.isNotEmpty()) }
        }
    }

    private fun delete() {
        viewModelScope.launch(Dispatchers.IO) {
            setEffect { MoviesUiEffect.Loading("删除中") }
            val ids: MutableList<String> = mutableListOf()
            mSelectList.forEach {
                ids.add(it.value.resource_id.toString())
            }
            val map: MutableMap<String, Any> = mutableMapOf()
            map["resource_id_list"] = ids
//            map["operateType"] = "collection"
            map["type"] = mType
            val bean = ApiFactory.getInstance().create(CommonApiService::class.java)
                .deleteCollect(map)
            if(bean.success()) {
                if(mSelectList.size == state.value.data.size) {
                    mSelectList.clear()
                    setState { copy(pageStatus = PageStatus.Empty, data = listOf(),
                        isSelectAll = false, isDelete = false) }
                } else {
                    val newData: MutableList<CollectBean> = state.value.data.toMutableList()
                    val iterator: MutableIterator<MutableMap.MutableEntry<Int, CollectBean>> =
                        mSelectList.iterator()
                    while (iterator.hasNext()) {
                        val value: MutableMap.MutableEntry<Int, CollectBean> = iterator.next()
                        newData.remove(value.value)
                        iterator.remove()
                    }
                    setState { copy(data = newData.toList(), isSelectAll = false, isDelete = false) }
                }
                setEffect { MoviesUiEffect.Toast("删除成功") }
            } else {
                setEffect { MoviesUiEffect.Toast(bean.message.toString()) }
            }
            setEffect { MoviesUiEffect.Dismiss }
        }
    }

    private fun cancelAll() {
        viewModelScope.launch(Dispatchers.Default) {
            val newData: MutableList<CollectBean> = state.value.data.toMutableList()
            val iterator: MutableIterator<MutableMap.MutableEntry<Int, CollectBean>> =
                mSelectList.iterator()
            while (iterator.hasNext()) {
                val value: MutableMap.MutableEntry<Int, CollectBean> = iterator.next()
                newData[value.key] = value.value.copy(is_selected = false)
                iterator.remove()
            }
            setState { copy(data = newData.toList(), isSelectAll = false, isDelete = false) }
        }
    }
}