package com.at.video.mine.ui.email.replace

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.module_export_main.IMainService
import com.at.video.export.mine.IMineService
import com.at.video.mine.http.ApiService
import com.at.video.mine.manager.UserManager
import com.at.video.mine.ui.userinfo.UserInfoUiEffect
import com.melancholy.network.ApiFactory
import kotlinx.coroutines.launch

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/5/6 下午1:51
 */
data class ReplaceEmailUiState(
    val email: String = "",
    val code: String = ""
) : IUiState

sealed class ReplaceEmailUiEvent : IUiEvent {
    data class Email(val email: String) : ReplaceEmailUiEvent()
    data class OldEmail(val oldEmail: String) : ReplaceEmailUiEvent()
    data class Code(val code: String) : ReplaceEmailUiEvent()
    object GetCode : ReplaceEmailUiEvent()
    data class Confirm(val oldCode: String?) : ReplaceEmailUiEvent()
}

sealed class ReplaceEmailUiEffect : IUiEffect {
    data class Loading(val content: String = "") : ReplaceEmailUiEffect()
    object Dismiss : ReplaceEmailUiEffect()
    data class Toast(val msg: String) : ReplaceEmailUiEffect()
    object CodeSuccess : ReplaceEmailUiEffect()
}


class ReplaceEmailViewModel(application: Application) : BaseAndroidViewModel<ReplaceEmailUiState,
        ReplaceEmailUiEvent, ReplaceEmailUiEffect>(application) {

    private var oldEmail: String? = null
    override fun providerInitialState(): ReplaceEmailUiState {
        return ReplaceEmailUiState()
    }

    override fun handleEvent(event: ReplaceEmailUiEvent) {
        when (event) {
            is ReplaceEmailUiEvent.Email -> setState { copy(email = event.email) }
            is ReplaceEmailUiEvent.Code -> setState { copy(code = event.code) }
            is ReplaceEmailUiEvent.GetCode -> getCode()
            is ReplaceEmailUiEvent.Confirm -> emailChange(event.oldCode)
            is ReplaceEmailUiEvent.OldEmail -> oldEmail = event.oldEmail
        }
    }

    /**
     * 获取验证码
     */
    private fun getCode() {
        viewModelScope.launch {
            if (!oldEmail.isNullOrEmpty()&&state.value.email==oldEmail){
                setEffect { ReplaceEmailUiEffect.Toast( "新旧邮箱不能相同") }
                return@launch
            }
            val map: MutableMap<String, Any> = mutableMapOf()
            map["email"] = state.value.email
            map["type"] = 4
            map["way"] = 1
            map["belong_app_id"] = Constants.APP_ID
            setEffect {
                ReplaceEmailUiEffect.Loading("获取验证码中")
            }
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .sendCode(map = map)
            setEffect { ReplaceEmailUiEffect.Dismiss }
            if (info.success()) {
                setEffect { ReplaceEmailUiEffect.Toast("获取验证码成功") }
                setEffect { ReplaceEmailUiEffect.CodeSuccess }
            } else {
                setEffect { ReplaceEmailUiEffect.Toast(info.message ?: "获取验证码失败") }
            }
        }
    }

    /**
     * 邮箱变更
     */
    private fun emailChange(oldCode: String?) {
        viewModelScope.launch {
            if (!oldEmail.isNullOrEmpty()&&state.value.email==oldEmail){
                setEffect { ReplaceEmailUiEffect.Toast( "新旧邮箱不能相同") }
                return@launch
            }
            val map: MutableMap<String, Any> = mutableMapOf()
            map["email"] = state.value.email
            val isExit =  IMainService.getMainService().getSysConfig().getIsConfigEmail()
            if(isExit){
                map["code"] = state.value.code
                if (!oldCode.isNullOrEmpty()) {
                    map["oldCode"] = oldCode
                }
            }
            setEffect { ReplaceEmailUiEffect.Loading("修改中") }
//            val id: String = IMineService.getMineService().getUser().value?.getUserId() ?: ""
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .modifyUser( map = map)
            if (info.success()) {
                UserManager.instance.updateUserEmail(state.value.email)
                setEffect { ReplaceEmailUiEffect.Toast("修改成功") }
            } else {
                setEffect { ReplaceEmailUiEffect.Toast(info.message ?: "") }
            }
            setEffect { ReplaceEmailUiEffect.Dismiss }
        }
    }
}