package com.at.video.mine.ui.history

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.mine.entity.ChapterInfo
import com.at.video.mine.entity.HistoryBean
import com.at.video.mine.entity.HistoryList
import com.at.video.mine.http.ApiService
import com.melancholy.network.ApiFactory
import com.video.module_widget.api.CommonApiService
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @description:
 * @author:  stone
 * @email: 可以编辑
 * @date :   2022/5/9 上午10:00
 */
data class MoviesUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<HistoryBean> = listOf(),
    val isDelete: Boolean = false,
    val isSelectAll: Boolean = false
) : IUiState

sealed class MoviesUiEvent : IUiEvent {
    data class HeadData(val type: Int) : MoviesUiEvent()
    object InitLoadData : MoviesUiEvent()
    object RefreshData : MoviesUiEvent()
    object LoadDataMore : MoviesUiEvent()
    data class SelectOrUnselect(val info: HistoryBean, val position: Int) : MoviesUiEvent()
    object Cancel : MoviesUiEvent()
    object Delete : MoviesUiEvent()
    object SelectAll : MoviesUiEvent()
    object CancelAll : MoviesUiEvent()
}

sealed class MoviesUiEffect : IUiEffect {
    data class Loading(val msg: String) : MoviesUiEffect()
    object Dismiss : MoviesUiEffect()
    data class Toast(val toast: String) : MoviesUiEffect()
}

@HiltViewModel
class MoviesViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<MoviesUiState, MoviesUiEvent, MoviesUiEffect>(application) {
    private var mOffset: Int = 0
    private val mLimit: Int = 9
    private var mType: Int = 1
    private var mSelectList: MutableMap<Int, HistoryBean> = mutableMapOf()

    override fun providerInitialState(): MoviesUiState {
        return MoviesUiState()
    }

    override fun handleEvent(event: MoviesUiEvent) {
        when (event) {
            is MoviesUiEvent.HeadData -> mType = event.type
            is MoviesUiEvent.InitLoadData -> initLoadData()
            is MoviesUiEvent.RefreshData -> refreshData()
            is MoviesUiEvent.LoadDataMore -> loadDataMore()
            is MoviesUiEvent.SelectOrUnselect -> selectOrUnselect(event.info, event.position)
            is MoviesUiEvent.Cancel -> cancel()
            is MoviesUiEvent.SelectAll -> selectAll()
            is MoviesUiEvent.Delete -> delete()
            is MoviesUiEvent.CancelAll -> cancelAll()
        }
    }

    private fun delete() {
        viewModelScope.launch(Dispatchers.IO) {
            setEffect {
                MoviesUiEffect.Loading("删除中...")
            }
            val ids = mutableListOf<String>()
            mSelectList.forEach {
                ids.add(it.value.resource_id!!)
            }
            val map = mutableMapOf<String, Any>()
            map["resource_id_list"] = ids
            map["type"] = mType
            val info = ApiFactory.getInstance().create(CommonApiService::class.java).deleteHis(map)
            setEffect {
                MoviesUiEffect.Dismiss
            }
            if (info.success()) {//删除成功 刷新数据
                if (mSelectList.size == state.value.data.size) {
                    mSelectList.clear()
                    setState {
                        copy(
                            pageStatus = PageStatus.Empty,
                            data = listOf(),
                            isSelectAll = false,
                            isDelete = false
                        )
                    }
                } else {
                    val newData: MutableList<HistoryBean> = state.value.data.toMutableList()
                    val iterator: MutableIterator<MutableMap.MutableEntry<Int, HistoryBean>> =
                        mSelectList.iterator()
                    while (iterator.hasNext()) {
                        val value: MutableMap.MutableEntry<Int, HistoryBean> = iterator.next()
                        newData.remove(value.value)
                        iterator.remove()
                    }
                    setState {
                        copy(
                            data = newData.toList(), isSelectAll = false, isDelete = false
                        )
                    }
                }
                setEffect { MoviesUiEffect.Toast("删除成功") }
            } else {
                setEffect {
                    MoviesUiEffect.Toast(info.message ?: "删除失败")
                }
            }
        }
    }

    private fun cancelAll() {
        viewModelScope.launch(Dispatchers.IO) {
            val newData: MutableList<HistoryBean> = state.value.data.toMutableList()
            val iterator: MutableIterator<MutableMap.MutableEntry<Int, HistoryBean>> =
                mSelectList.iterator()
            while (iterator.hasNext()) {
                val value: MutableMap.MutableEntry<Int, HistoryBean> = iterator.next()
                newData[value.key] = value.value.copy(is_selected = false)
                iterator.remove()
            }
            setState { copy(data = newData.toList(), isSelectAll = false, isDelete = false) }
        }
    }

    private fun initLoadData() {
        viewModelScope.launch {
            getHis().onStart {
                setState { copy(pageStatus = PageStatus.Loading) }
            }.catch {
                setState { copy(pageStatus = PageStatus.Error(it)) }
            }.collect {
                if (it == null || (it.day.isEmpty() && it.history.isEmpty())) {
                    setState { copy(pageStatus = PageStatus.Empty) }
                } else {
                    val day = it.day
//                    dealDay(day)
                    val list: MutableList<HistoryBean> = mutableListOf()
//                    list.addAll(day)
                    list.addAll(it.history)
                    val hasMore: Boolean = it.history.size == mLimit
                    if (hasMore) {
                        mOffset += mLimit
                    }
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            data = list,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    private fun dealDay(days: List<HistoryBean>) {
        days.forEach {
            it.hisTag = "今日"
        }
    }

    private fun refreshData() {
        viewModelScope.launch {
            mOffset = 0
            getHis().onStart {
                setState {
                    copy(
                        refreshStatus = RefreshStatus.RefreshLoading,
                        loadMoreStatus = LoadMoreStatus.LoadMoreIdle
                    )
                }
            }.catch {
                setState { copy(refreshStatus = RefreshStatus.RefreshFail(Error(it))) }
            }.collect {
                if (it == null || (it.day.isEmpty() && it.history.isEmpty())) {
                    setState {
                        copy(
                            pageStatus = PageStatus.Empty,
                            refreshStatus = RefreshStatus.RefreshSuccess
                        )
                    }
                } else {
                    val day = it.day
//                    dealDay(day)
                    val list: MutableList<HistoryBean> = mutableListOf()
//                    list.addAll(day)
                    list.addAll(it.history)

                    val hasMore: Boolean = it.history.size == mLimit
                    if (hasMore) {
                        mOffset += mLimit
                    }
                    setState {
                        copy(
                            pageStatus = PageStatus.Success,
                            data = list,
                            refreshStatus = RefreshStatus.RefreshSuccess,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    private fun loadDataMore() {
        viewModelScope.launch {
            getHis().onStart {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            }.catch {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreFail(Error(it))) }
            }.collect {
                if (it == null || it.history.isEmpty()) {
                    setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
                } else {
                    val hasMore: Boolean = it.history.size == mLimit
                    if (hasMore) {
                        mOffset += mLimit
                    }
                    setState {
                        copy(
                            data = data + it.history,
                            loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)
                        )
                    }
                }
            }
        }
    }

    private fun getHis(): Flow<HistoryList?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["type"] = mType
            map["offset"] = mOffset
            map["limit"] = mLimit
            val bean =
                ApiFactory.getInstance().create(ApiService::class.java).getWatchHisList(map = map)
            if (bean.success()) {
                emit(bean.list?.let { HistoryList(day = arrayListOf(), history = it) })
            } else {
                throw Error(bean.message)
            }
        }
    }

    @Synchronized
    private fun selectOrUnselect(info: HistoryBean, position: Int) {

        viewModelScope.launch(Dispatchers.IO) {
            val newInfo = if (info.is_selected) {
                removeInfo(info)
                info.copy(is_selected = false)
            } else {
                val new = info.copy(is_selected = true)
                mSelectList[position] = new
                new
            }
            val newData: MutableList<HistoryBean> = state.value.data.toMutableList()
            newData[position] = newInfo
            setState { copy(data = newData.toList()) }
            setState {
                copy(
                    data = newData.toList(),
                    isSelectAll = newData.size == mSelectList.size,
                    isDelete = mSelectList.isNotEmpty()
                )
            }
        }
    }

    @Synchronized
    private fun removeInfo(info: HistoryBean) {
        for (i in 0..mSelectList.size) {
            if (mSelectList[i]?.resource_id == info.resource_id) {
                mSelectList.remove(i)
                break
            }
        }
    }

    private fun cancel() {
        mSelectList.clear()
        // notifyData()
    }

    private fun selectAll() {
        viewModelScope.launch {
            val newData: MutableList<HistoryBean> = state.value.data.toMutableList()
            if (newData.size == 0) {
                return@launch
            }
            if (newData.size == mSelectList.size) {
                val iterator: MutableIterator<MutableMap.MutableEntry<Int, HistoryBean>> =
                    mSelectList.iterator()
                while (iterator.hasNext()) {
                    val value: MutableMap.MutableEntry<Int, HistoryBean> = iterator.next()
                    newData[value.key] = value.value.copy(is_selected = false)
                    iterator.remove()
                }
            } else {
                newData.forEachIndexed { index, chapterInfo ->
                    if (!mSelectList.containsKey(index)) {
                        val newInfo: HistoryBean = chapterInfo.copy(is_selected = true)
                        mSelectList[index] = newInfo
                        newData[index] = newInfo
                    }
                }
            }
            setState {
                copy(
                    data = newData.toList(),
                    isSelectAll = newData.size == mSelectList.size,
                    isDelete = mSelectList.isNotEmpty()
                )
            }
        }
    }
}