package com.at.video.mine.ui.history.movies

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.fragment.app.viewModels
import androidx.lifecycle.LifecycleOwner
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviFragment
import com.at.common.mvi.collectState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.mine.R
import com.at.video.mine.databinding.MineFragmentHistoryBinding
import com.at.video.mine.entity.HistoryBean
import com.at.video.mine.ui.history.*
import com.chad.library.adapter.base.BaseQuickAdapter
import com.chad.library.adapter.base.listener.OnItemChildClickListener
import com.chad.library.adapter.base.listener.OnItemClickListener
import com.kennyc.view.MultiStateView
import com.melancholy.router.annotation.annotation.Route
import com.scwang.smart.refresh.layout.SmartRefreshLayout
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import dagger.hilt.android.AndroidEntryPoint

/**
 * @description:
 * @author:  stone
 * @email: 小说
 * @date :   2022/5/9 上午11:00
 */
@AndroidEntryPoint
@Route(path = "/mine/history/MoviesFragment")
class MoviesFragment : BaseMviFragment<MineFragmentHistoryBinding, MoviesViewModel>(),
    OnRefreshLoadMoreListener, OnItemClickListener, OnItemChildClickListener, View.OnClickListener {
    private val mAdapter: MoviesAdapter = MoviesAdapter()
    private val mActivityViewModel by viewModels<HistoryViewModel>(ownerProducer = { requireActivity() })

    override fun createViewBinding(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): MineFragmentHistoryBinding {
        return MineFragmentHistoryBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<MoviesViewModel> {
        return MoviesViewModel::class.java
    }

    override fun getViewLifecycleOwner(): LifecycleOwner = requireActivity()

    override fun getViewModelKey(): String {
        return requireArguments().getInt(Constants.KEY_DATA).toString()
    }

    override fun initialize(view: View, savedInstanceState: Bundle?) {
        val type: Int = requireArguments().getInt(Constants.KEY_DATA)
        getViewModel().sendEvent(MoviesUiEvent.HeadData(type))
        val srlRefresh: SmartRefreshLayout = requireViewById(com.at.common.R.id.srl_refresh)
        srlRefresh.setOnRefreshLoadMoreListener(this)
        srlRefresh.setEnableRefresh(false)
        srlRefresh.setEnableLoadMore(false)
        val rvList: RecyclerView = requireViewById(com.at.common.R.id.rv_list)
        rvList.layoutManager = LinearLayoutManager(requireContext())
        rvList.setHasFixedSize(true)
        rvList.itemAnimator = null
        rvList.adapter = mAdapter
        mAdapter.setOnItemClickListener(this)
        mAdapter.setOnItemChildClickListener(this)
        requireBinding().apply {
            tvAllSelect.setOnClickListener(this@MoviesFragment)
            tvDelete.setOnClickListener(this@MoviesFragment)
        }
    }

    override fun observer() {
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoviesUiState::pageStatus) {
                val srlRefresh: SmartRefreshLayout = requireViewById(com.at.common.R.id.srl_refresh)
                requireBinding().apply {
                    when (it) {
                        is PageStatus.Loading -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.LOADING
                        }
                        is PageStatus.Error -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.getView(MultiStateView.ViewState.ERROR)!!
                                .findViewById<TextView>(com.at.common.R.id.tv_hint).text =
                                it.error.message
                            msvView.viewState = MultiStateView.ViewState.ERROR
                        }
                        is PageStatus.Empty -> {
                            srlRefresh.setEnableRefresh(false)
                            srlRefresh.setEnableLoadMore(false)
                            msvView.viewState = MultiStateView.ViewState.EMPTY
                        }
                        is PageStatus.Success -> {
                            srlRefresh.setEnableRefresh(true)
                            srlRefresh.setEnableLoadMore(true)
                            msvView.viewState = MultiStateView.ViewState.CONTENT
                        }
                    }
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoviesUiState::refreshStatus) {
                val srlRefresh: SmartRefreshLayout = requireViewById(com.at.common.R.id.srl_refresh)
                when (it) {
                    is RefreshStatus.RefreshSuccess -> {
                        srlRefresh.finishRefresh()
                    }
                    is RefreshStatus.RefreshLoading -> {
                        if (!srlRefresh.isRefreshing) {
                            srlRefresh.autoRefreshAnimationOnly()
                        }
                    }
                    is RefreshStatus.RefreshFail -> {
                        srlRefresh.finishRefresh(false)
                    }
                    else -> {}
                }
            }
        }
        viewLifecycleOwner.lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoviesUiState::loadMoreStatus) {
                val srlRefresh: SmartRefreshLayout = requireViewById(com.at.common.R.id.srl_refresh)
                when (it) {
                    is LoadMoreStatus.LoadMoreSuccess -> {
                        if (it.hasMore) {
                            srlRefresh.finishLoadMore()
                        } else {
                            srlRefresh.finishLoadMoreWithNoMoreData()
                        }
                    }
                    is LoadMoreStatus.LoadMoreLoading -> {
                        if (!srlRefresh.isLoading) {
                            srlRefresh.autoLoadMoreAnimationOnly()
                        }
                    }
                    is LoadMoreStatus.LoadMoreFail -> {
                        srlRefresh.finishLoadMore(false)
                    }
                    else -> {}
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoviesUiState::data) {
                mAdapter.setNewInstance(it.toMutableList())
            }
        }

        lifecycleScope.launchWhenResumed {
            mActivityViewModel.state.collectState(HistoryUiState::isEdit) {
                mAdapter.setEdit(it)
                requireBinding().flBottom.visibility = if (it) View.VISIBLE else View.GONE
                if (!it) {
                    getViewModel().sendEvent(MoviesUiEvent.CancelAll)
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoviesUiState::isSelectAll) {
                if (it) {
                    requireBinding().tvAllSelect.text = "取消全选"
                } else {
                    requireBinding().tvAllSelect.text = "全选"
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(MoviesUiState::isDelete) {
                requireBinding().tvDelete.isEnabled = it
                if (it) {
                    requireBinding().tvDelete.setTextColor(StyleUtils.getPrimaryColor().toArgb())
                } else {
                    requireBinding().tvDelete.setTextColor(resources.getColor(R.color.color_FFB4B8BF))
                }
            }
        }
    }

    override fun loadData() {

    }

    override fun onResume() {
        super.onResume()
        getViewModel().sendEvent(MoviesUiEvent.InitLoadData)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MoviesUiEvent.RefreshData)
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        getViewModel().sendEvent(MoviesUiEvent.LoadDataMore)
    }

    override fun onItemClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {

    }

    override fun onItemChildClick(adapter: BaseQuickAdapter<*, *>, view: View, position: Int) {
        val info: HistoryBean = adapter.getItem(position) as HistoryBean
        getViewModel().sendEvent(MoviesUiEvent.SelectOrUnselect(info, position))
    }

    override fun onClick(view: View) {
        when (view.id) {
            R.id.tv_all_select -> getViewModel().sendEvent(MoviesUiEvent.SelectAll)
            R.id.tv_delete -> getViewModel().sendEvent(MoviesUiEvent.Delete)
        }
    }
}