package com.at.video.mine.ui.login

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.video.mine.http.ApiService
import com.at.video.mine.manager.UserManager
import com.jeremyliao.liveeventbus.LiveEventBus
import com.melancholy.network.ApiFactory
import com.video.module_widget.entity.LoginSuccessEvent
import com.video.module_widget.utils.PhoneUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/28 上午10:28
 */

data class LoginUiState(
    val isVisible: Boolean = false,
    val userName: String = "",
    val password: String = "",
    val isCheck: Boolean = true
) : IUiState

sealed class LoginUiEvent : IUiEvent {
    object Login : LoginUiEvent()
    object PasswordVisible : LoginUiEvent()
    data class Account(val account: String) : LoginUiEvent()
    data class Password(val password: String) : LoginUiEvent()
    object Agreement : LoginUiEvent()
}

sealed class LoginUiEffect : IUiEffect {
    data class Loading(val content: String = "") : LoginUiEffect()
    object Dismiss : LoginUiEffect()
    data class Toast(val msg: String) : LoginUiEffect()
    data class AccountError(val msg: String) : LoginUiEffect()
    object LoginSuccess : LoginUiEffect()
}

@HiltViewModel
class LoginViewModel @Inject constructor(application: Application) : BaseAndroidViewModel<
        LoginUiState, LoginUiEvent, LoginUiEffect>(application) {

    override fun providerInitialState(): LoginUiState {
        return LoginUiState()
    }

    override fun handleEvent(event: LoginUiEvent) {
        when (event) {
            is LoginUiEvent.PasswordVisible -> setState { copy(isVisible = !isVisible) }
            is LoginUiEvent.Account -> setState { copy(userName = event.account) }
            is LoginUiEvent.Password -> setState { copy(password = event.password) }
            is LoginUiEvent.Agreement -> setState { copy(isCheck = !isCheck) }
            is LoginUiEvent.Login -> login()
        }
    }

    private fun login() {
        viewModelScope.launch {
            if (state.value.isCheck) {
                val map: MutableMap<String, Any> = mutableMapOf()
                map["username"] = state.value.userName
                map["password"] = state.value.password
                map["belong_app_id"] = Constants.APP_ID
                map["login_source"] = 2
                map["login_type"] = 1
                map["login_system_version"] = "Android${PhoneUtils.getDeviceAndroidVersion()}"
                map["login_device_name"] =
                    "${PhoneUtils.getDeviceBrand()} ${PhoneUtils.getDeviceModel()}"
                setEffect {
                    LoginUiEffect.Loading("加载数据中")
                }
                val info = ApiFactory.getInstance().create(ApiService::class.java)
                    .login(map = map)
                setEffect { LoginUiEffect.Dismiss }
                if (info.success()) {
                    UserManager.instance.saveInfo(info)
                    LiveEventBus.get<LoginSuccessEvent>("login_success").post(LoginSuccessEvent())
                    setEffect { LoginUiEffect.LoginSuccess }
                } else {
                    if (info.code == 403 && (info.reason == "App" || info.reason == "Content")) {
                        setEffect { LoginUiEffect.AccountError(info.message ?: "该账号已被限制登录") }
                    } else {
                        setEffect { LoginUiEffect.Toast(info.message ?: "登录失败,请稍后重试") }
                    }
                }
            } else {
                setEffect { LoginUiEffect.Toast("请阅读并同意协议") }
            }
        }
    }
}
