package com.at.video.mine.ui.password.back

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.video.mine.http.ApiService
import com.melancholy.network.ApiFactory
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/28 下午4:35
 */

data class BackPasswordUiState(
    val account: String = "",
    val email: String = "",
    val code: String = ""
): IUiState

sealed class BackPasswordUiEvent: IUiEvent {
    data class Account(val account: String): BackPasswordUiEvent()
    data class Email(val email: String): BackPasswordUiEvent()
    data class Code(val code: String): BackPasswordUiEvent()
    object GetCode: BackPasswordUiEvent()
    object Validation: BackPasswordUiEvent()
}

sealed class BackPasswordUiEffect: IUiEffect {
    data class Loading(val content: String = ""): BackPasswordUiEffect()
    object Dismiss: BackPasswordUiEffect()
    data class Toast(val msg: String): BackPasswordUiEffect()
    object CodeSuccess: BackPasswordUiEffect()
    data class ValidationSuccess(val email: String, val code: String): BackPasswordUiEffect()
}

@HiltViewModel
class BackPasswordViewModel @Inject constructor(application: Application): BaseAndroidViewModel<
        BackPasswordUiState, BackPasswordUiEvent, BackPasswordUiEffect>(application) {
    private var mUserId: Long = 0

    override fun providerInitialState(): BackPasswordUiState {
        return BackPasswordUiState()
    }

    override fun handleEvent(event: BackPasswordUiEvent) {
        when(event) {
            is BackPasswordUiEvent.Account -> setState { copy(account = event.account) }
            is BackPasswordUiEvent.Email -> setState { copy(email = event.email) }
            is BackPasswordUiEvent.Code -> setState { copy(code = event.code) }
            is BackPasswordUiEvent.GetCode -> getCode()
            is BackPasswordUiEvent.Validation -> validation()
        }
    }

    /**
     * 获取验证码
     */
    private fun getCode() {
        viewModelScope.launch(Dispatchers.IO) {
            setEffect { BackPasswordUiEffect.Loading("获取验证码中") }
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["username"] = state.value.account
            map["email"] = state.value.email
            map["type"] = 3
            map["way"] = 1
            map["belong_app_id"] = Constants.APP_ID
            val data = ApiFactory.getInstance().create(ApiService::class.java).sendCode(map = map)
            setEffect { BackPasswordUiEffect.Dismiss }
            if(data.success() ) {
                setEffect { BackPasswordUiEffect.Toast("获取验证码成功") }
                setEffect { BackPasswordUiEffect.CodeSuccess }
            } else {
                setEffect { BackPasswordUiEffect.Toast(data.message!!) }
            }
        }
    }

    /**
     * 验证邮箱
     */
    private fun validation() {
        viewModelScope.launch(Dispatchers.IO) {
            setEffect { BackPasswordUiEffect.Loading("验证邮箱中") }
            val map: MutableMap<String, Any> = mutableMapOf()
            map["email"] = state.value.email
            map["code"] = state.value.code
            map["type"] = 3
            map["way"] = 1
            map["belong_app_id"] = Constants.APP_ID
            val data = ApiFactory.getInstance().create(ApiService::class.java).validation(map = map)
            setEffect { BackPasswordUiEffect.Dismiss }
            if(data.success()) {
                setEffect { BackPasswordUiEffect.Toast("获取验证码成功") }
                setEffect { BackPasswordUiEffect.ValidationSuccess(state.value.email, state.value.code) }
            } else {
                setEffect { BackPasswordUiEffect.Toast(data.message!!) }
            }
        }
    }
}