package com.at.video.mine.ui.password.reset

import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.text.method.HideReturnsTransformationMethod
import android.text.method.PasswordTransformationMethod
import android.view.View
import androidx.lifecycle.lifecycleScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivityResetPasswordBinding
import com.at.video.mine.manager.UserManager
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.video.module_widget.utils.StyleUtils
import com.video.module_widget.utils.toArgb
import kotlinx.coroutines.flow.collect

/**
 * @author:  stone
 * @email:
 * @date :   2022/4/29 下午1:42
 * @description: 修改密码
 */
@Route(path = "/mine/resetPasswordActivity")
class ResetPasswordActivity : BaseMviActivity<MineActivityResetPasswordBinding,
        ResetPasswordViewModel>(), View.OnClickListener {

    override fun createViewBinding(): MineActivityResetPasswordBinding {
        return MineActivityResetPasswordBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<ResetPasswordViewModel> {
        return ResetPasswordViewModel::class.java
    }

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            addBackButton(Color.BLACK)
            setTitle("设置新密码")
        }
        requireBinding().apply {
            cetNewOne.addTextChangedListener(NewOneTextWatcher())
            ivNewOneVisible.setOnClickListener(this@ResetPasswordActivity)
            cetNewTwo.addTextChangedListener(NewTwoTextWatcher())
            ivNewTwoVisible.setOnClickListener(this@ResetPasswordActivity)
            sbConfirm.setOnClickListener(this@ResetPasswordActivity)
            sbConfirm.apply {
                setBackgroundColor(StyleUtils.getPrimaryColor().toArgb())
                setShapeSolidDisabledColor(StyleUtils.getPrimaryColor().toArgb(0.2))
                setShapeRadius(resources.getDimensionPixelOffset(com.at.common.R.dimen.dp_4))
                reBuilder()
            }
        }
    }

    override fun loadData() {

    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is ResetPasswordUiEffect.Loading -> {
                        LoadingUtil.showLoading(this@ResetPasswordActivity, it.content)
                    }
                    is ResetPasswordUiEffect.Dismiss -> LoadingUtil.dismiss()
                    is ResetPasswordUiEffect.Toast -> ToastUtils.show(it.msg)
                    is ResetPasswordUiEffect.CheckPwdFail -> {
                        ToastUtils.show(it.msg.msg())
                    }
                    is ResetPasswordUiEffect.CheckPwdSuccess -> {
                        val email: String = intent.getStringExtra(Constants.KEY_DATA)!!
                        val code: String = intent.getStringExtra(Constants.KEY_DATA_1)!!
                        getViewModel().sendEvent(ResetPasswordUiEvent.Confirm(email, code))
                    }
                    is ResetPasswordUiEffect.Success -> {
                        Router.getInstance().build("/mine/LoginActivity").navigation()
                        finish()
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ResetPasswordUiState::isNewOneVisible) {
                requireBinding().apply {
                    cetNewOne.transformationMethod = if (it) {
                        HideReturnsTransformationMethod.getInstance()
                    } else {
                        PasswordTransformationMethod.getInstance()
                    }
                    ivNewOneVisible.setImageResource(
                        if (it) R.drawable.mine_ic_password_visible
                        else R.drawable.mine_ic_password_invisible
                    )
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ResetPasswordUiState::newOnePassword) {
                requireBinding().apply {
                    if (cetNewOne.text.toString() != it) {
                        cetNewOne.setText(it)
                    }
                    sbConfirm.isEnabled = it.isNotEmpty() && cetNewTwo.text.toString().isNotEmpty()
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ResetPasswordUiState::isNewTwoVisible) {
                requireBinding().apply {
                    cetNewTwo.transformationMethod = if (it) {
                        HideReturnsTransformationMethod.getInstance()
                    } else {
                        PasswordTransformationMethod.getInstance()
                    }
                    ivNewTwoVisible.setImageResource(
                        if (it) R.drawable.mine_ic_password_visible
                        else R.drawable.mine_ic_password_invisible
                    )
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(ResetPasswordUiState::newTwoPassword) {
                requireBinding().apply {
                    if (cetNewTwo.text.toString() != it) {
                        cetNewTwo.setText(it)
                    }
                    sbConfirm.isEnabled = it.isNotEmpty() && cetNewOne.text.toString().isNotEmpty()
                }
            }
        }
    }

    override fun onClick(view: View) {
        when (view.id) {
            //显示或隐藏原密码
            R.id.iv_new_one_visible -> getViewModel().sendEvent(ResetPasswordUiEvent.NewOneVisible)
            //显示或隐藏原密码
            R.id.iv_new_two_visible -> getViewModel().sendEvent(ResetPasswordUiEvent.NewTwoVisible)
            //确认
            R.id.sb_confirm -> {
                getViewModel().sendEvent(ResetPasswordUiEvent.CheckPasswordVaild(getViewModel().state.value.newOnePassword))
            }
        }
    }

    private inner class NewOneTextWatcher : TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(ResetPasswordUiEvent.NewOnePassword(p0.toString()))
        }
    }

    private inner class NewTwoTextWatcher : TextWatcher {
        override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {

        }

        override fun afterTextChanged(p0: Editable) {
            getViewModel().sendEvent(ResetPasswordUiEvent.NewTwoPassword(p0.toString()))
        }
    }
}