package com.at.video.mine.ui.phone

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.video.mine.entity.CountryCode
import com.at.video.mine.http.ApiService
import com.at.video.mine.manager.UserManager
import com.melancholy.network.ApiFactory
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/6/17 下午4:05
 */
data class PhoneUiState(val phone: String = "", val countryCodes: List<CountryCode> = arrayListOf()) : IUiState
sealed class PhoneUiEvent : IUiEvent {
    data class Phone(val phone: String) : PhoneUiEvent()
    object Confirm : PhoneUiEvent()
    object LoadCountCode : PhoneUiEvent()
    data class PhoneCode(val phoneCode: String) : PhoneUiEvent()
}

sealed class PhoneUiEffect : IUiEffect {
    data class Loading(val content: String = "") : PhoneUiEffect()
    object Dismiss : PhoneUiEffect()
    data class Toast(val msg: String) : PhoneUiEffect()
}

@HiltViewModel
class ModifyPhoneViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<PhoneUiState, PhoneUiEvent, PhoneUiEffect>(
        application
    ) {
    var phoneCode = ""
    override fun handleEvent(event: PhoneUiEvent) {
        when (event) {
            is PhoneUiEvent.Phone -> setState { copy(phone = event.phone) }
            is PhoneUiEvent.Confirm -> savePhone()
            is PhoneUiEvent.LoadCountCode -> loadCountryCode()
            is PhoneUiEvent.PhoneCode -> phoneCode = event.phoneCode
        }
    }

    // 修改手机号
    private fun savePhone() {
        viewModelScope.launch(Dispatchers.IO) {
            val map: MutableMap<String, Any> = mutableMapOf()
            val phone = state.value.phone
            map["phone_number"] = phone
            map["mobile_country_code"] = phoneCode
            setEffect { PhoneUiEffect.Loading("修改中") }
//            val id: Long = IMineService.getMineService().getUser().value?.getUserId() ?: 0
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .modifyUser(map = map)
            if (info.success()) {
                UserManager.instance.updateUserPhone(phone, phoneCode)
                setEffect { PhoneUiEffect.Toast("修改成功") }
            } else {
                setEffect { PhoneUiEffect.Toast(info.message ?: "") }
            }
            setEffect { PhoneUiEffect.Dismiss }
        }
    }

    private fun loadCountryCode() {
        viewModelScope.launch(Dispatchers.IO) {
            val res = ApiFactory.getInstance().create(ApiService::class.java)
                .getCountryCode()
            if (res.success() && !res.list.isNullOrEmpty()) {
                setState { copy(countryCodes = res.list) }
            } else {
                setEffect { PhoneUiEffect.Toast("获取国家码失败") }
            }
        }
    }

    override fun providerInitialState(): PhoneUiState {
        return PhoneUiState()
    }
}
