package com.at.video.mine.ui.register

import android.app.Application
import android.graphics.Bitmap
import android.net.Uri
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.video.mine.http.ApiService
import com.at.video.mine.ui.mine.utils.saveToAlbum
import com.melancholy.network.ApiFactory
import com.video.module_widget.entity.BaseResponse
import com.video.module_widget.utils.PhoneUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/28 上午10:28
 */

data class RegisterUiState(
    val data: String,
    val isVisible: Boolean = false,
    val userName: String = "",
    val password: String = "",
    val isCheck: Boolean = true
) : IUiState

sealed class RegisterUiEvent : IUiEvent {
    object Register : RegisterUiEvent()
    object RandomAccount : RegisterUiEvent()
    object PasswordVisible : RegisterUiEvent()
    object Agreement : RegisterUiEvent()
    data class Account(val account: String) : RegisterUiEvent()
    data class Password(val password: String) : RegisterUiEvent()
    data class Picture(val bitmap: Bitmap) : RegisterUiEvent()
    object CheckAccount : RegisterUiEvent()
    object CheckPwd : RegisterUiEvent()
}

sealed class RegisterUiEffect : IUiEffect {
    data class Loading(val content: String = "") : RegisterUiEffect()
    object Dismiss : RegisterUiEffect()
    object Register : RegisterUiEffect()
    data class Toast(val msg: String) : RegisterUiEffect()
    data class CheckSuccess(val isAccount: Boolean) : RegisterUiEffect()
    data class CheckFail(val msg: BaseResponse) : RegisterUiEffect()
}

@HiltViewModel
class RegisterViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<RegisterUiState, RegisterUiEvent, RegisterUiEffect>(application) {
    var isExit = false

    init {
        sendEvent(RegisterUiEvent.RandomAccount)
    }

    override fun providerInitialState(): RegisterUiState {
        return RegisterUiState("模块数据")
    }

    override fun handleEvent(event: RegisterUiEvent) {
        when (event) {
            is RegisterUiEvent.RandomAccount -> randomAccount()
            is RegisterUiEvent.Register -> register()
            is RegisterUiEvent.PasswordVisible -> setState { copy(isVisible = !isVisible) }
            is RegisterUiEvent.Account -> setState { copy(userName = event.account) }
            is RegisterUiEvent.Password -> setState { copy(password = event.password) }
            is RegisterUiEvent.Agreement -> setState { copy(isCheck = !isCheck) }
            is RegisterUiEvent.Picture -> savePicture(event.bitmap)
            is RegisterUiEvent.CheckAccount -> checkData()
            is RegisterUiEvent.CheckPwd -> checkData(false)
        }
    }

    private fun randomAccount() {
        viewModelScope.launch(Dispatchers.IO) {
            setEffect { RegisterUiEffect.Loading("加载数据中") }
//            getEmailConfig()
            val data = ApiFactory.getInstance().create(ApiService::class.java).getUserAccount()
            if (data.success()) {
                setState { copy(userName = data.username, password = data.password) }
            }
            setEffect { RegisterUiEffect.Dismiss }
        }
    }

//    private fun getEmailConfig(){
//        viewModelScope.launch {
//            val map: MutableMap<String, Any> = mutableMapOf()
//            map["belong_app_id"] = Constants.APP_ID
//            map["belong_webmaster_id"] = GlobalConsts.WEB_MASTER_ID
//            map["not_exist_error"] = 0
//            val res = ApiFactory.getInstance().create(ApiService::class.java).isEmailConfig(map = map)
//            isExit = res.success()
//        }
//    }

    private fun register() {
        viewModelScope.launch(Dispatchers.IO) {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["username"] = state.value.userName
            map["password"] = state.value.password
//                map["passwordRepeat"] = state.value.password
            map["belong_app_id"] = Constants.APP_ID
            map["registered_source"] = 2
            map["login_system_version"] = "Android${PhoneUtils.getDeviceAndroidVersion()}"
            map["login_device_name"] = "${PhoneUtils.getDeviceBrand()} ${PhoneUtils.getDeviceModel()}"
//                val regex = Regex("^[0-9A-Za-z]{5,12}\$")
//                val matches = regex.matches(state.value.userName)
//                if(!matches){
//                    setEffect { RegisterUiEffect.Toast("用户名长度为5-12位，由数字、字母组成，不区分大小写") }
//                    return@launch
//                }
//                if(state.value.password.length < 6){
//                    setEffect { RegisterUiEffect.Toast("密码长度为6-12位字符，非空格") }
//                    return@launch
//                }
            setEffect { RegisterUiEffect.Loading("加载数据中") }
            val b = ApiFactory.getInstance().create(ApiService::class.java).register(map = map)
            setEffect { RegisterUiEffect.Dismiss }
            if (b.success()) {
                setEffect { RegisterUiEffect.Register }
            } else {
                setEffect { RegisterUiEffect.CheckFail(b) }
                setEffect { RegisterUiEffect.Toast(b.message ?: "注册失败,请稍后重试") }
            }
        }
    }

    private fun savePicture(bitmap: Bitmap) {
        viewModelScope.launch(Dispatchers.IO) {
            val uri: Uri? = bitmap.saveToAlbum(
                getApplication(),
                "${System.currentTimeMillis()}.png"
            )
            bitmap.recycle()
            if (uri == null) {
                setEffect { RegisterUiEffect.Toast("注册成功！账号密码保存至相册失败") }
            } else {
                setEffect { RegisterUiEffect.Toast("注册成功！账号密码已保存相册") }
            }
        }
    }

    private fun checkData(isAccount: Boolean = true) {
        viewModelScope.launch(Dispatchers.IO) {
            val map: MutableMap<String, Any> = mutableMapOf()
            if (isAccount) {
                map["username"] = state.value.userName
            } else {
                map["password"] = state.value.password
            }
            val res =
                ApiFactory.getInstance().create(ApiService::class.java).checkInputData(map = map)
            if (res.success()) {
                setEffect { RegisterUiEffect.CheckSuccess(isAccount) }
            } else {
                setEffect { RegisterUiEffect.CheckFail(res) }
            }
        }
    }
}
