package com.at.video.mine.ui.settings

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState

import com.at.video.mine.entity.AppVersionInfo
import com.at.video.mine.http.ApiService
import com.melancholy.network.ApiFactory
import com.video.module_widget.getFileSize
import com.video.module_widget.imageUrl
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.io.File
import java.io.FileInputStream
import java.text.DecimalFormat
import javax.inject.Inject

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/29 上午10:35
 */
data class SettingUiState(
    val cacheSize: String = "0B"
): IUiState
sealed class SettingUiEvent: IUiEvent{
    object InitData: SettingUiEvent()
    object DeleteCache: SettingUiEvent()
    data class CheckUpdate(val versionCode: Long): SettingUiEvent()
}
sealed class SettingUiEffect: IUiEffect {
    data class UpdateDialog(val appVersionInfo: AppVersionInfo): SettingUiEffect()
    data class Toast(val msg:String):SettingUiEffect()
}
@HiltViewModel
class SettingViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<SettingUiState, SettingUiEvent, SettingUiEffect>(application) {

    override fun providerInitialState(): SettingUiState {
        return SettingUiState()
    }

    override fun handleEvent(event: SettingUiEvent) {
        when(event){
            is SettingUiEvent.InitData -> getCacheSize()
            is SettingUiEvent.DeleteCache -> deleteCache()
            is SettingUiEvent.CheckUpdate -> getAppVersionInfo(event.versionCode)
        }
    }

    private fun getCacheSize() {
        viewModelScope.launch(Dispatchers.IO) {
            val application: Application = getApplication<Application>()
            var size: Long = getAllFileSize(application.cacheDir)
            application.externalCacheDirs?.forEach {
                size += getAllFileSize(it)
            }
            setState { copy(cacheSize = formatFileSize(size)) }
        }
    }

    private fun deleteCache() {
        viewModelScope.launch(Dispatchers.IO) {
            val application: Application = getApplication<Application>()
            deleteFile(application.cacheDir)
            application.externalCacheDirs?.forEach {
                deleteFile(it)
            }
            setState { copy(cacheSize = "0B") }
        }
    }

    private fun getAppVersionInfo(appCode: Long){
        viewModelScope.launch {
            val map = mutableMapOf<String, Any>()
          try {
              val res = ApiFactory.getInstance().create(ApiService::class.java).getAppInfo(map = map)
              if(res.success()){
                  if(res.application_version !== null && res.application !=null){
                      val downloadUrl = res.application_version.android?.pkg_address
                      val minoUrl = res.application_version.android?.minio_pkg_address
                      val appContent = res.application_version.android?.version_content
                      val appName = res.application.name
                      val forceUpdate = res.application_version.android?.upgrade == 1
                      val serverCode = res.application_version.android?.version_code
                      val versionName = res.application_version.android?.version_name
                      var res = downloadUrl.imageUrl()
                          ?.let { ApiFactory.getInstance().create(ApiService::class.java).download(it) }
                      var len = res?.contentLength()
                      if(len != -1L){
                          setEffect { SettingUiEffect.UpdateDialog(AppVersionInfo(
                              appName = appName,
                              updateLog = appContent,
                              downUrl = downloadUrl,
                              size = getFileSize(len!!),
                              code = serverCode!!,
                              forciblyUpdate = forceUpdate,
                              packageName = versionName
                          ))}
                      }else {
                          res = ApiFactory.getInstance().create(ApiService::class.java).download(minoUrl!!)
                          len = res.contentLength()
                          if(len != -1L){
                              setEffect { SettingUiEffect.UpdateDialog(AppVersionInfo(
                                  appName = appName,
                                  updateLog = appContent,
                                  downUrl = minoUrl,
                                  size = getFileSize(len),
                                  forciblyUpdate = forceUpdate,
                                  packageName = versionName
                              ))}
                          }else{
                              setEffect { SettingUiEffect.Toast("服务器异常,稍后再试！") }
                          }
                      }

                  }
              }else {
                  setEffect { SettingUiEffect.UpdateDialog(AppVersionInfo())}
              }
          }catch (e:Exception){
              setEffect { SettingUiEffect.UpdateDialog(AppVersionInfo())}
          }
        }
    }



    companion object {
        /**
         * 获取所有文件的大小
         * @param file
         * @return
         */
        @JvmStatic
        fun getAllFileSize(file: File): Long {
            return if (file.isDirectory) {
                val list: Array<File>? = file.listFiles()
                if (list.isNullOrEmpty()) {
                    0
                } else {
                    var size = 0L
                    list.forEach {
                        size += getAllFileSize(it)
                    }
                    size
                }
            } else {
                getSingleFileSize(file)
            }
        }

        /**
         * 获取单个文件的大小
         * @param file
         * @return
         */
        @JvmStatic
        fun getSingleFileSize(file: File): Long {
            return if (file.exists() && file.isFile) {
                var fis: FileInputStream? = null
                return try {
                    fis = FileInputStream(file)
                    fis.available().toLong()
                } catch (e: Exception) {
                    0L
                } finally {
                    if (fis != null) {
                        try {
                            fis.close()
                        } catch (e: Exception) {
                        }
                    }
                }
            } else {
                0L
            }
        }

        /**
         * 转换文件大小
         * @param size 文件大小
         * @param pattern 小数点格式
         * @return
         */
        @JvmStatic
        @JvmOverloads
        fun formatFileSize(size: Long, pattern: String = "0.00"): String {
            val df = DecimalFormat(pattern)
            return if (size <= 0) {
                "${df.format(0.0)}B"
            } else if (size < 1024) {
                "${df.format(size.toDouble())}B"
            } else if (size < 1048576) {
                "${df.format(size / 1024.0)}KB"
            } else if (size < 1073741824) {
                "${df.format(size / 1048576.0)}MB"
            } else {
                "${df.format(size / 1073741824.0)}GB"
            }
        }

        @JvmStatic
        fun deleteFile(file: File) {
            if(file.isDirectory) {
                file.listFiles()?.forEach {
                    deleteFile(it)
                }
            }
            try {
                file.delete()
            } catch (e: Exception) {
                //ignore
            }
        }
    }
}