package com.at.video.mine.ui.settings

import android.annotation.SuppressLint
import android.graphics.Color
import android.os.Bundle
import android.view.View
import androidx.lifecycle.lifecycleScope
import com.at.common.mvi.BaseMviActivity
import com.at.common.mvi.collectState
import com.at.common.setLightStatusBar
import com.at.common.utils.LoadingUtil
import com.at.common.utils.ToastUtils
import com.at.video.export.mine.IMineService
import com.at.video.mine.R
import com.at.video.mine.databinding.MineActivitySettingBinding
import com.at.video.mine.manager.UserManager
import com.at.video.update.dialog.UpdateDialog
import com.melancholy.router.annotation.annotation.Route
import com.melancholy.router.api.launcher.Router
import com.melancholy.utils.AppUtils
import com.video.module_widget.imageUrl
import dagger.hilt.android.AndroidEntryPoint

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/29 上午10:33
 */
@AndroidEntryPoint
@Route(path = "/mine/SettingActivity")
class SettingsActivity :
    BaseMviActivity<MineActivitySettingBinding, SettingViewModel>(),
    View.OnClickListener {

    override fun createViewBinding(): MineActivitySettingBinding {
        return MineActivitySettingBinding.inflate(layoutInflater)
    }

    override fun getViewModelClass(): Class<SettingViewModel> = SettingViewModel::class.java

    override fun immersiveBar() {
        super.immersiveBar()
        setLightStatusBar(true)
    }

    @SuppressLint("SetTextI18n")
    override fun initialize(savedInstanceState: Bundle?) {
        requireTitleBar().apply {
            setBackgroundColor(Color.WHITE)
            addBackButton(Color.BLACK)
            setTitle("设置")
        }
        requireBinding().run {
            btnAccountDataLl.setOnClickListener(this@SettingsActivity)
            tvPwdLabel.setOnClickListener(this@SettingsActivity)
            btnClearCache.setOnClickListener(this@SettingsActivity)
            btnUpdate.setOnClickListener(this@SettingsActivity)
            btnLogout.setOnClickListener(this@SettingsActivity)
            tvUpdateContent.text = "当前版本v${AppUtils.getVersionName(this@SettingsActivity)}"
        }
    }

    override fun loadData() {
        getViewModel().sendEvent(SettingUiEvent.InitData)
    }

    override fun observer() {
        lifecycleScope.launchWhenResumed {
            IMineService.getMineService().getUser().collect {
                requireBinding().btnLogout.visibility = if (it == null) View.INVISIBLE else View.VISIBLE
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().state.collectState(SettingUiState::cacheSize) {
                requireBinding().tvCacheContent.text = it
            }
        }
        lifecycleScope.launchWhenResumed {
            IMineService.getMineService().getUser().collect {
                if (it != null) {
                    requireBinding().apply {
                        tvNickname.text = it.getUserNickname()
                    }
                }
            }
        }
        lifecycleScope.launchWhenResumed {
            getViewModel().effect.collect {
                when (it) {
                    is SettingUiEffect.UpdateDialog -> {
                        LoadingUtil.dismiss()
                        if (it.appVersionInfo.code > AppUtils.getVersionCode(this@SettingsActivity)) {
                            it.appVersionInfo?.let { it1 ->
                                val title = "${it.appVersionInfo.appName}(最新版:${it1.packageName})"
                                UpdateDialog.showDialog(
                                    supportFragmentManager,
                                    it1.downUrl!!.imageUrl()!!,
                                    it1.size!!,
                                    "是否更新到最新版$title?",
                                    it1.updateLog!!,
                                    it1.forciblyUpdate
                                )
                            }
                        } else {
                            ToastUtils.show("当前已是最新版本!")
                        }
                    }
                    is SettingUiEffect.Toast -> ToastUtils.show(it.msg)
                }
            }
        }
    }

    override fun onClick(p0: View) {
        when (p0.id) {
            // 个人资料
            R.id.btn_account_data_ll -> {
                Router.getInstance().build("/mine/UserInfoActivity").navigation()
            }
            // 修改密码
            R.id.tv_pwd_label -> {
                Router.getInstance().build("/mine/ModifyPasswordActivity").navigation()
            }
            // 清除缓存
            R.id.btn_clear_cache -> getViewModel().sendEvent(SettingUiEvent.DeleteCache)
            // 更新
            R.id.btn_update -> {
                LoadingUtil.showLoading(
                    this@SettingsActivity,
                    "检测更新中..."
                )
                getViewModel().sendEvent(SettingUiEvent.CheckUpdate(AppUtils.getVersionCode(this)))
            }
            // 登出
            R.id.btn_logout -> {
                UserManager.instance.clearInfo()
                ToastUtils.show("退出成功")
                finish()
            }
        }
    }
}
