package com.at.video.mine.ui.userinfo

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.network.BaseBean
import com.at.video.export.mine.IMineService
import com.at.video.mine.entity.AreaInfo
import com.at.video.mine.entity.City
import com.at.video.mine.entity.CityInfo
import com.at.video.mine.http.ApiService
import com.at.video.mine.manager.UserManager
import com.melancholy.network.ApiFactory
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.io.BufferedReader
import java.io.File
import java.io.InputStreamReader
import javax.inject.Inject

/**
 * @description:
 * @author:  stone
 * @email:
 * @date :   2022/4/29 下午3:16
 */

data class UserInfoUiState(
    val data: String
): IUiState

sealed class UserInfoUiEvent: IUiEvent {
    object Province: UserInfoUiEvent()
    object Location: UserInfoUiEvent()
    data class Avatar(val file: File): UserInfoUiEvent()
    data class Nickname(val name: String): UserInfoUiEvent()
    data class Gender(val gender: Int): UserInfoUiEvent()
    data class Birthday(val birthday: Long): UserInfoUiEvent()
    data class Address(val addressId: String): UserInfoUiEvent()
}

sealed class UserInfoUiEffect: IUiEffect {
    data class Loading(val content: String = ""): UserInfoUiEffect()
    object Dismiss: UserInfoUiEffect()
    data class Toast(val msg: String): UserInfoUiEffect()
    data class Location(
        val provinces: MutableList<City>,
        val cities: MutableList<List<City>>
    ): UserInfoUiEffect()
}

@HiltViewModel
class UserInfoViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<UserInfoUiState, UserInfoUiEvent, UserInfoUiEffect>(application) {
    private var mArea: CityInfo? = null
    private val mProvinces: MutableList<City> = mutableListOf()
    private val mCities: MutableList<List<City>> = mutableListOf()

    override fun providerInitialState(): UserInfoUiState {
        return UserInfoUiState("data")
    }

    override fun handleEvent(event: UserInfoUiEvent) {
        when(event) {
            is UserInfoUiEvent.Province -> {
                getUserInfo()
                getProvinces()
            }
            is UserInfoUiEvent.Location -> {
                var info = getArea()
                if(info == null || !info.success()) {
                    setEffect { UserInfoUiEffect.Loading("加载数据中") }
                    getProvinces()
                    info = getArea()
                    if(info == null || !info.success()) {
                        setEffect { UserInfoUiEffect.Toast("加载数据失败") }
                    } else if(info.success() && info.list.isNullOrEmpty()) {
                        setEffect { UserInfoUiEffect.Toast("没有数据") }
                    } else {
                        setEffect { UserInfoUiEffect.Location(mProvinces, mCities) }
                    }
                    setEffect { UserInfoUiEffect.Dismiss }
                } else if(info.success() && info.list.isNullOrEmpty()) {
                    setEffect { UserInfoUiEffect.Toast("没有数据") }
                } else {
                    setEffect { UserInfoUiEffect.Location(mProvinces, mCities) }
                }
            }
            is UserInfoUiEvent.Avatar -> changeAvatar(event.file)
            is UserInfoUiEvent.Nickname -> changeNickname(event.name)
            is UserInfoUiEvent.Gender -> changeGender(event.gender)
            is UserInfoUiEvent.Birthday -> changeBirthday(event.birthday)
            is UserInfoUiEvent.Address -> changeAddress(event.addressId)
        }
    }

    @Synchronized
    private fun getArea(): CityInfo? = mArea


    private fun getUserInfo(){
        viewModelScope.launch(Dispatchers.IO) {
            val data = ApiFactory.getInstance().create(ApiService::class.java)
                .userinfo()
            if(data.success()){
                UserManager.instance.updateUser(data)
            }
        }
    }

    /**
     * 获取省份信息
     */
    @Synchronized
    private fun getProvinces() {
        viewModelScope.launch(Dispatchers.IO) {
            val data = ApiFactory.getInstance().create(ApiService::class.java)
                .getAddressArea()
            if(data.success() && !data.list.isNullOrEmpty()) {
                data.list.forEach {
                    mProvinces.add(it)
                    mCities.add(it.districts ?: emptyList())
                }
            }
            mArea = data
        }
    }

    /**
     * 修改头像
     */
    private fun changeAvatar(file: File) {
        viewModelScope.launch(Dispatchers.IO) {
            val part = MultipartBody.Part.createFormData("file", file.name,
                RequestBody.create(null, file))
            val avatar = ApiFactory.getInstance().create(ApiService::class.java)
                .updateFile(part = part)
            if(avatar.success() && avatar.address != null) {
                val map: MutableMap<String, Any> = mutableMapOf()
                map["avatar"] = avatar.address!!
//                val id: Long = IMineService.getMineService().getUser().value?.getUserId() ?: 0
                val info = ApiFactory.getInstance().create(ApiService::class.java)
                    .modifyUser( map = map)
                if (info.success()) {
                    UserManager.instance.updateUserAvatar(avatar.address!!)
                    setEffect { UserInfoUiEffect.Toast("修改成功") }
                } else {
                    setEffect { UserInfoUiEffect.Toast(info.message ?: "") }
                }
            } else {
                setEffect { UserInfoUiEffect.Toast(avatar.message ?: "") }
            }
            setEffect { UserInfoUiEffect.Dismiss }
        }
    }

    /**
     * 修改昵称
     */
    private fun changeNickname(nickname: String) {
        viewModelScope.launch(Dispatchers.IO) {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["nickname"] = nickname
            setEffect { UserInfoUiEffect.Loading("修改中") }
//            val id: Long = IMineService.getMineService().getUser().value?.getUserId() ?: 0
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .modifyUser( map = map)
            if (info.success()) {
                UserManager.instance.updateUserNickName(nickname)
                setEffect { UserInfoUiEffect.Toast("修改成功") }
            } else {
                setEffect { UserInfoUiEffect.Toast(info.message ?: "") }
            }
            setEffect { UserInfoUiEffect.Dismiss }
        }
    }

    /**
     * 修改性别
     */
    private fun changeGender(gender: Int) {
        viewModelScope.launch(Dispatchers.IO) {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["gender"] = gender
            setEffect { UserInfoUiEffect.Loading("修改中") }
//            val id: Long = IMineService.getMineService().getUser().value?.getUserId() ?: 0
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .modifyUser( map = map)
            if (info.success()) {
                UserManager.instance.updateUserGender(gender)
                setEffect { UserInfoUiEffect.Toast("修改成功") }
            } else {
                setEffect { UserInfoUiEffect.Toast(info.message ?: "") }
            }
            setEffect { UserInfoUiEffect.Dismiss }
        }
    }

    /**
     * 修改生日
     */
    private fun changeBirthday(birthday: Long) {
        viewModelScope.launch(Dispatchers.IO) {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["birthday"] = birthday
            setEffect { UserInfoUiEffect.Loading("修改中") }
//            val id: Long = IMineService.getMineService().getUser().value?.getUserId() ?: 0
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .modifyUser(map = map)
            if (info.success() ) {
                UserManager.instance.updateUserBirthday(birthday.toString())
                setEffect { UserInfoUiEffect.Toast("修改成功") }
            } else {
                setEffect { UserInfoUiEffect.Toast(info.message ?: "") }
            }
            setEffect { UserInfoUiEffect.Dismiss }
        }
    }

    /**
     * 修改所在地
     */
    private fun changeAddress(addressId: String) {
        viewModelScope.launch(Dispatchers.IO) {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["location"] = addressId
            setEffect { UserInfoUiEffect.Loading("修改中") }
//            val id: Long = IMineService.getMineService().getUser().value?.getUserId() ?: 0
            val info = ApiFactory.getInstance().create(ApiService::class.java)
                .modifyUser(map = map)
            if (info.success()) {
                UserManager.instance.updateUserAddr(addressId)
                setEffect { UserInfoUiEffect.Toast("修改成功") }
            } else {
                setEffect { UserInfoUiEffect.Toast(info.message ?: "") }
            }
            setEffect { UserInfoUiEffect.Dismiss }
        }
    }
}