/*
 * Copyright (C) 2016, 2017 Yaroslav Pronin <proninyaroslav@mail.ru>
 *
 * This file is part of LibreTorrent.
 *
 * LibreTorrent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * LibreTorrent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with LibreTorrent.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.at.video.player.libtorrent;

/*
 * This class is designed for seeding, downloading and management of torrents.
 */

import android.os.Handler;
import android.os.HandlerThread;
import android.util.Log;

import com.at.video.player.libtorrent.listener.DHTStatsAlertListener;
import com.frostwire.jlibtorrent.SessionManager;
import com.frostwire.jlibtorrent.SessionParams;
import com.frostwire.jlibtorrent.SettingsPack;
import com.frostwire.jlibtorrent.swig.settings_pack;

import java.util.Locale;
import java.util.concurrent.CountDownLatch;

public class TorrentEngine {
    private final String LIBTORRENT_THREAD_NAME = "TORRENTSTREAM_LIBTORRENT";
    private static TorrentEngine mTorrentEngine;
    private static SessionManager torrentSession;
    private HandlerThread libTorrentThread;
    private Handler libTorrentHandler;
    private CountDownLatch initialisingLatch;
    private TorrentOptions torrentOptions;
    private Boolean initialising = false, initialised = false;
    private Integer dhtNodes = 0;

    private TorrentEngine() {
        torrentOptions = new TorrentOptions.Builder().removeFilesAfterStop(false).autoDownload(true).build();
        initialise();
    }

    public static TorrentEngine getInstance() {
        if (mTorrentEngine == null) {
            mTorrentEngine = new TorrentEngine();
        }
        return mTorrentEngine;
    }

    public SessionManager getTorrentSession() {
        return torrentSession;
    }

    private void initialise() {
        if (libTorrentThread != null && torrentSession != null) {
            resumeSession();
        } else {
            if ((initialising || initialised) && libTorrentThread != null) {
                libTorrentThread.interrupt();
            }
            initialising = true;
            initialised = false;
            initialisingLatch = new CountDownLatch(1);

            libTorrentThread = new HandlerThread(LIBTORRENT_THREAD_NAME);
            libTorrentThread.start();
            libTorrentThread.setName("libTorrentThread");
            libTorrentHandler = new Handler(libTorrentThread.getLooper());
            libTorrentHandler.post(new Runnable() {
                @Override
                public void run() {
                    torrentSession = new SessionManager();
                    setOptions(torrentOptions);
                    torrentSession.addListener(dhtStatsAlertListener);
                    torrentSession.start();
                    initialising = false;
                    initialised = true;
                    initialisingLatch.countDown();
                }
            });
        }
    }

    private SettingsPack settingsPack;

    public SettingsPack getSettingsPack() {
        return settingsPack;
    }

    public void setSettingsPack(SettingsPack settingsPack) {
        this.settingsPack = settingsPack;
    }

    public void setOptions(TorrentOptions options) {
        torrentOptions = options;
        settingsPack = new SettingsPack()
                .anonymousMode(torrentOptions.anonymousMode)
                .connectionsLimit(torrentOptions.maxConnections)
                .downloadRateLimit(torrentOptions.maxDownloadSpeed)
                .uploadRateLimit(torrentOptions.maxUploadSpeed)
                .activeDhtLimit(torrentOptions.maxDht)
                .enableDht(false)
                // .broadcastLSD(false)
                .activeDownloads(5);
        settingsPack.setBoolean(settings_pack.bool_types.enable_incoming_utp.swigValue(), true);
        settingsPack.setBoolean(settings_pack.bool_types.enable_outgoing_utp.swigValue(), true);
        settingsPack.setBoolean(settings_pack.bool_types.enable_incoming_tcp.swigValue(), true);
        settingsPack.setBoolean(settings_pack.bool_types.enable_outgoing_tcp.swigValue(), true);
        settingsPack.setBoolean(settings_pack.bool_types.announce_to_all_trackers.swigValue(), true);
        settingsPack.setBoolean(settings_pack.bool_types.enable_upnp.swigValue(), false);
        settingsPack.setBoolean(settings_pack.bool_types.enable_natpmp.swigValue(), false);
//        settingsPack.setString(settings_pack.string_types.dht_bootstrap_nodes.swigValue(), dhtBootstrapNodes());
        settingsPack.setInteger(settings_pack.int_types.min_reconnect_time.swigValue(), 0);
        //重新发送
        settingsPack.setInteger(settings_pack.int_types.utp_num_resends.swigValue(), 100);
        settingsPack.setInteger(settings_pack.int_types.auto_manage_startup.swigValue(), 0);
        //混合模式算法 tcp，udp
        settingsPack.setInteger(settings_pack.int_types.mixed_mode_algorithm.swigValue(), settings_pack.bandwidth_mixed_algo_t.prefer_tcp.swigValue());
        //同时请求的资源大小，影响播放时候的定位下载
        settingsPack.setInteger(settings_pack.int_types.urlseed_max_request_bytes.swigValue(), 6 * 1024 * 1024);
        //最大网络种子链接
        settingsPack.setInteger(settings_pack.int_types.max_web_seed_connections.swigValue(), 1000);

        if (torrentOptions.listeningPort != -1) {
            String ifStr = String.format(Locale.ENGLISH, "%s:%d", "0.0.0.0", torrentOptions.listeningPort);
            settingsPack.setString(settings_pack.string_types.listen_interfaces.swigValue(), ifStr);
        }

        if (torrentOptions.proxyHost != null) {
            settingsPack.setString(settings_pack.string_types.proxy_hostname.swigValue(), torrentOptions.proxyHost);
            if (torrentOptions.proxyUsername != null) {
                settingsPack.setString(settings_pack.string_types.proxy_username.swigValue(), torrentOptions.proxyUsername);
                if (torrentOptions.proxyPassword != null) {
                    settingsPack.setString(settings_pack.string_types.proxy_password.swigValue(), torrentOptions.proxyPassword);
                }
            }
        }

        if (torrentOptions.peerFingerprint != null) {
            settingsPack.setString(settings_pack.string_types.peer_fingerprint.swigValue(), torrentOptions.peerFingerprint);
        }

        if (!torrentSession.isRunning()) {
            SessionParams sessionParams = new SessionParams(settingsPack);
            torrentSession.start(sessionParams);
        } else {
            torrentSession.applySettings(settingsPack);
        }
    }

    /**
     * 设置最大web种子链接数
     *
     * @param count
     */
    public void setMaxWebSeedConnections(int count) {
        if (settingsPack != null && count > 0) {
            Log.i("settingsPack", "setMaxWebSeedConnections: settingsPack不为null");
            //最大网络种子链接
            settingsPack.setInteger(settings_pack.int_types.max_web_seed_connections.swigValue(), count);
        } else {
            Log.i("settingsPack", "setMaxWebSeedConnections: settingsPack为null");
        }
        if (!torrentSession.isRunning()) {
            SessionParams sessionParams = new SessionParams(settingsPack);
            torrentSession.start(sessionParams);
        } else {
            torrentSession.applySettings(settingsPack);
        }
    }


    /**
     * Resume TorrentSession
     */
    public void resumeSession() {
        if (libTorrentThread != null && torrentSession != null) {
            libTorrentHandler.removeCallbacksAndMessages(null);

            //resume torrent session if needed
            if (torrentSession.isPaused()) {
                libTorrentHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        torrentSession.resume();
                    }
                });
            }

            if (!torrentSession.isDhtRunning()) {
                libTorrentHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        torrentSession.startDht();
                    }
                });
            }
        }
    }

    /**
     * Pause TorrentSession
     */
    public void pauseSession() {
        libTorrentHandler.post(new Runnable() {
            @Override
            public void run() {
                torrentSession.pause();
            }
        });
    }

    private final DHTStatsAlertListener dhtStatsAlertListener = new DHTStatsAlertListener() {
        @Override
        public void stats(int totalDhtNodes) {
            dhtNodes = totalDhtNodes;
        }
    };

    public TorrentOptions getOptions() {
        return torrentOptions;
    }

    public Integer getTotalDhtNodes() {
        return dhtNodes;
    }

    private static String dhtBootstrapNodes() {
        StringBuilder sb = new StringBuilder();
        sb.append("dht.libtorrent.org:25401").append(",");
        sb.append("router.bittorrent.com:6881").append(",");
        sb.append("dht.transmissionbt.com:6881").append(",");

        /* For IPv6 DHT */
        sb.append("outer.silotis.us:6881");

        return sb.toString();
    }
}
