package com.at.video.player.ui.detail

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.player.entity.StarDetailBean
import com.at.video.player.entity.StarWorks
import com.at.video.player.entity.TabInfo
import com.at.video.player.http.ApiService
import com.melancholy.network.ApiFactory
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.NoDataBinder
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvPosition
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 *
 * @Description: DetailViewModel
 * @CreateDate: 2022/6/13 上午10:11
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/13 上午10:11
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

data class DetailUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val data: List<Any> = listOf(),
    val isExtend:Boolean = false
): IUiState

sealed class DetailUiEvent: IUiEvent {
    data class HeaderData(val id: String,val style:HomeStyle?): DetailUiEvent()
    object InitLoadData: DetailUiEvent()
    object RefreshData: DetailUiEvent()
    object ExtensionOrShrink: DetailUiEvent()
}

sealed class DetailUiEffect: IUiEffect {

}

@HiltViewModel
class DetailViewModel @Inject constructor(application: Application): BaseAndroidViewModel<
        DetailUiState, DetailUiEvent, DetailUiEffect>(application) {
    private var mId: String = ""
    private var style:HomeStyle? = null
    var tabs:ArrayList<TabInfo> = ArrayList()

    override fun providerInitialState(): DetailUiState {
        return DetailUiState()
    }

    override fun handleEvent(event: DetailUiEvent) {
        when(event){
            is DetailUiEvent.HeaderData ->{
                mId = event.id
                style = event.style
            }
            is DetailUiEvent.InitLoadData -> initLoadData()
            is DetailUiEvent.RefreshData -> refreshData()
            is DetailUiEvent.ExtensionOrShrink -> {
//                val newData: MutableList<Any> = state.value.data.toMutableList()
//                val item: StartDetailInfo = newData[0] as StartDetailInfo
//                newData[0] = item.copy(isExtend = !item.isExtend)
                setState { copy(isExtend = !state.value.isExtend) }
            }
        }
    }
    private fun initLoadData() {
        if(mId == "0" || mId.isEmpty()){
            val newData: MutableList<Any> = mutableListOf()
            newData.add(NoDataBinder.Data("没有相关介绍"))
            setState { copy(pageStatus = PageStatus.Success, data = newData) }
            return
        }
        viewModelScope.launch {
            combine(listOf(
                getActorDetail(),
                getActorWorks()
            )){
                val newData: MutableList<Any> = mutableListOf()
                newData.add(it[0])
                style?.let { it1->
                    it1.content?.forEach { item ->
                        when(item!!.type){
                            StyleConsts.SWIPER_ADV -> {
                                val carourseAdvs = JsonUtils.getAdvListData<AdvertisingBean>(
                                    GlobalConsts.CAROUSEL_ADV)
                                val bannerAdvs = carourseAdvs?.let { it2 ->
                                    DataUtils.filterAdvPosition(
                                        it2, AdvPosition.Player)
                                }
                                bannerAdvs?.let { it2->
                                    newData.add( BannerBinder.BannerInfo(it2))
                                }
                            }
                            StyleConsts.BANNER_ADV -> {
                                val bannerAdv = JsonUtils.getAdvListData<AdvertisingBean>(
                                    GlobalConsts.BANNER_ADV)
                                val bannerAdvs = bannerAdv?.let { it2 ->
                                    DataUtils.filterAdvPosition(it2,AdvPosition.StarDetail)
                                }
                                bannerAdvs?.let { it3->
                                    val adBinderData: MutableList<Any> = mutableListOf()
                                    it3.forEach { it2 ->
                                        if (it2.image_type != 3) {
                                            adBinderData.add(AdImageBinder.Data(it2))
                                        } else {
                                            adBinderData.add(AdTextBinder.Data(it2))
                                        }
                                    }
                                    newData.addAll(adBinderData)
                                }
                            }
//                            StyleConsts.SYNOPSIS ->  newData.add(it[0])
                            StyleConsts.WORKS -> {
                                val works = it[1] as StarWorks
                                works.list.forEach { it1 ->
                                    if(!it1.video_list.isNullOrEmpty()){
                                        tabs.add(TabInfo(it1.category.id!!,it1.category.name!!))
                                    }
                                }
                                if(tabs.isNotEmpty()){
                                    newData.addAll(works.list)
                                }else{
                                    newData.add(NoDataBinder.Data(""))
                                }
                            }
                        }
                    }
                }
                setState { copy(pageStatus = PageStatus.Success, data = newData) }
            }.onStart {
                setState { copy(pageStatus = PageStatus.Loading) }
            }.catch {
                setState { copy(pageStatus = PageStatus.Error(it)) }
            }.collect()
        }
    }

    private fun refreshData() {
//        viewModelScope.launch {
//            getActorDetail().onStart {
//                setState { copy(refreshStatus = RefreshStatus.RefreshLoading) }
//            }.catch {
//                setState { copy(refreshStatus = RefreshStatus.RefreshFail(it)) }
//            }.collect {
//                setState { copy(refreshStatus = RefreshStatus.RefreshSuccess, data = it) }
//            }
//        }
    }

    private fun getActorDetail(): Flow<StarDetailBean> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["id"] = mId
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getStarDetail(map)
            if(bean.success()) {
                emit(bean)
            } else {
                throw Error(bean.message)
            }
        }
    }

    private var offset = 0
    private fun getActorWorks(): Flow<StarWorks> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["person_id"] = mId
            map["limit"] = StyleUtils.getGridCountByType(3)
            map["offset"] = offset
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getStartWork(map)
            if(bean.success()) {
                emit(bean)
            } else {
                throw Error(bean.message)
            }
        }
    }
}