package com.at.video.player.ui.player

import android.app.Application
import android.util.Log
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.export.mine.IMineService
import com.at.video.player.entity.*
import com.at.video.player.http.ApiService
import com.at.video.player.ui.player.binder.*
import com.melancholy.network.ApiFactory
import com.video.module_widget.api.CommonApiService
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvPosition
import com.video.module_widget.entity.AdvType
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.DataUtils
import com.video.module_widget.utils.JsonUtils
import com.video.module_widget.utils.StyleUtils
import com.youth.banner.util.LogUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

data class PlayerUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val data: List<Any> = listOf(),
    val info: VideoBean? = null,
    val watchHistory: WatchHistorys? = null,
    val level: LevelBinder.LevelData? = null,
    val router: RouteBinder.RouteData? = null,
    val levelPosition: Int = 0,
    val likeLsCollectStatus: UpdatePlayerCollectStatus = UpdatePlayerCollectStatus()
) : IUiState

sealed class PlayerUiEvent : IUiEvent {
    data class HeaderData(val id: String, val style: HomeStyle?) : PlayerUiEvent()
    object InitData : PlayerUiEvent()
    object RefreshData : PlayerUiEvent()

    // parentPos 父级位置 收藏或者点赞 type 1为点赞 2为收藏 id为空，代表使用的当前视频
    data class CollectOrLike(val parentPos: Int = -1, val type: Int, val position: Int? = null) :
        PlayerUiEvent()

    // 开始播放
    object Play : PlayerUiEvent()

    // 暂停广告
    object PauseAd : PlayerUiEvent()
    data class SwitchRouter(val router: RouteBinder.RouteData, val position: Int) : PlayerUiEvent()
    data class SwitchLevel(val level: LevelBinder.LevelData, val position: Int) : PlayerUiEvent()
    data class UpdatePlayTime(val index: Int, val position: Float, val totalTime: String) :
        PlayerUiEvent()

    object Share : PlayerUiEvent()
    object PayForPlay : PlayerUiEvent()
    object ReBackPlayRouter : PlayerUiEvent()
    class Switch(val position: Int, val data: VideoListBinder.Data) : PlayerUiEvent()
}

sealed class PlayerUiEffect : IUiEffect {
    data class Toast(val msg: String) : PlayerUiEffect()
    data class Loading(val content: String = "") : PlayerUiEffect()
    object Dismiss : PlayerUiEffect()
    data class PlayBeforeAd(val info: AdvertisingBean) : PlayerUiEffect()
    data class PlayPauseAd(val info: AdvertisingBean) : PlayerUiEffect()
    data class PayForPlay(val info: VideoBean) : PlayerUiEffect()
    data class PayStatus(val result: Boolean) : PlayerUiEffect()
    data class Play(
        val info: VideoBean,
        val router: RouteBinder.RouteData,
        val level: LevelBinder.LevelData
    ) : PlayerUiEffect()

    object PlayLoading : PlayerUiEffect()
    object VideoInfoFail : PlayerUiEffect()
    data class ChangeDataEffect(val data: VideoListBinder.Data, val pos: Int) : PlayerUiEffect()
}

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/6/10 上午9:28
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/10 上午9:28
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@HiltViewModel
class PlayerViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<PlayerUiState, PlayerUiEvent, PlayerUiEffect>(application) {
    private var mId: String = ""
    private var style: HomeStyle? = null
    override fun providerInitialState(): PlayerUiState = PlayerUiState()

    override fun handleEvent(event: PlayerUiEvent) {
        when (event) {
            is PlayerUiEvent.HeaderData -> {
                mId = event.id
                style = event.style
            }
            is PlayerUiEvent.InitData -> {
                initLoadData()
            }
            is PlayerUiEvent.RefreshData -> initLoadData()
            is PlayerUiEvent.CollectOrLike -> collectOrLike(
                event.parentPos,
                event.type,
                event.position
            )
            is PlayerUiEvent.Play -> {
                val data: VideoBean = state.value.info!!
                if (state.value.watchHistory == null) {
                    updatePlayCount(0, 0f, "")
                }
                setEffect { PlayerUiEffect.Play(data, state.value.router!!, state.value.level!!) }
            }
            is PlayerUiEvent.PauseAd -> getPauseAd()
            is PlayerUiEvent.SwitchRouter -> {
                val oldData: List<Any> = state.value.data
                val index = oldData.indexOf(event.router)
                if (index != -1) {
                    val levelDatas: MutableList<VideoUrl>? =
                        event.router.list[event.position].urls?.toMutableList()
                    var newLevel: Int = state.value.level?.index ?: 0
                    if (newLevel + 1 > levelDatas?.size ?: 0) {
                        newLevel = 0
                    }
                    val url = levelDatas?.get(newLevel)?.url
                    checkVideoUrl(url ?: "", isRouter = true, newLevel, event.router, null)
                    val newData: MutableList<Any> = oldData.toMutableList()
                    newData[index] = event.router.copy(index = event.position)
                    newData[index + 1]
                    val newLevelData = LevelBinder.LevelData(
                        levelDatas ?: mutableListOf(),
                        newLevel,
                        ( newData[index + 1] as LevelBinder.LevelData).style
                    )
                    newData[index + 1] = newLevelData
                    setState {
                        copy(
                            data = newData,
                            level = newLevelData,
                            router = event.router.copy(index = event.position)
                        )
                    }
                    getPlayerBefore()
                }
            }
            is PlayerUiEvent.SwitchLevel -> {
                val oldData: List<Any> = state.value.data
                val index = oldData.indexOf(event.level)
                if (index != -1) {
                    val newData: MutableList<Any> = oldData.toMutableList()
                    val url = "${event.level!!.list[event.position].url}"
                    checkVideoUrl(url, false, event.position, null, event.level)
                    Log.e("URL", url)
                    val realPos: Int = if (newData[0] is SynopsisBinder.SynopsisData) 0 else 1
                    newData[index] = event.level.copy(index = event.position)
                    setState {
                        copy(
                            data = newData,
                            level = event.level.copy(index = event.position),
                            watchHistory = state.value.watchHistory?.copy(progress = "0")
                        )
                    }
                    updatePlayCount(event.position, 0f, "")
                    getPlayerBefore()
                }
            }
            is PlayerUiEvent.UpdatePlayTime -> updatePlayCount(
                event.index,
                event.position,
                event.totalTime
            )
            is PlayerUiEvent.Share -> {
                val application: Application = getApplication()
//                ClipboardUtils.copyText(
//                    application, "${
//                        IMainService.getMainService().getSysConfig()
//                            .getConfigShareAddr()
//                    }/play/${state.value.info?.id}"
//                )
                setEffect { PlayerUiEffect.Toast("已添加到剪切板") }
            }
            is PlayerUiEvent.PayForPlay -> buyByGold()
            is PlayerUiEvent.ReBackPlayRouter -> { // 切换异常，恢复之前的播放
            }
            is PlayerUiEvent.Switch -> {
                switch(event.data, event.position)
            }
            else -> {}
        }
    }

    private fun checkVideoUrl(
        url: String,
        isRouter: Boolean = true,
        position: Int,
        router: RouteBinder.RouteData?,
        level: LevelBinder.LevelData?
    ) {
        val oldData: List<Any> = state.value.data
        val newData: MutableList<Any> = oldData.toMutableList()
        setEffect { PlayerUiEffect.PlayLoading }
        if (isRouter) {
            val index = oldData.indexOf(router!!)
            newData[index] = router!!.copy(index = position)
            setState { copy(data = newData, router = router.copy(index = position)) }
        } else {
            val index = oldData.indexOf(level!!)
            newData[index] = level.copy(index = position)
            setState { copy(data = newData, level = level.copy(index = position)) }
            updatePlayCount(position, 0f, "")
        }

//        viewModelScope.launch {
//            try {
//                val res = ApiFactory.getInstance().create(ApiService::class.java).download(url)
//                val len = res.contentLength()
//                if (len != -1L) {
//                    val oldData: List<Any> = state.value.data
//                    val newData: MutableList<Any> = oldData.toMutableList()
//                    setEffect { PlayerUiEffect.PlayLoading }
//                    if (isRouter) {
//                        val index = oldData.indexOf(router!!)
//                        newData[index] = router!!.copy(index = position)
//                        setState { copy(data = newData, router = router.copy(index = position)) }
//                    } else {
//                        val index = oldData.indexOf(level!!)
//                        newData[index] = level.copy(index = position)
//                        setState { copy(data = newData, level = level.copy(index = position)) }
//                        updatePlayCount(position, 0f)
//                    }
//                    getPlayerBefore()
//                } else {
//                    setEffect { PlayerUiEffect.Toast("线路异常") }
//                }
//            } catch (e: Exception) {
//                setEffect { PlayerUiEffect.Toast("线路异常") }
//            }
//
//        }
    }

    private var listDatas = mutableListOf<Any>()

    /**
     * 初始化加载数据
     */
    private fun initLoadData(isRefresh: Boolean = false) {
        listDatas.clear()
        viewModelScope.launch {
            combine(
                listOf(
                    getAd(AdvType.BeforePlayAdv.ordinal + 1, 1),
                    getVideoInformation(),
                    getAd(AdvType.ColumnAdv.ordinal + 1, 10)
                )
            ) {
                val list: MutableList<Any> = mutableListOf()
                val ad = it[0]

                if (it[1]?.size!! <= 2) {
                    setState { copy(pageStatus = PageStatus.Error(Throwable("资源不存在"))) }
                    return@combine
                }

                val levelData = (it[1] as List<Any>)[2] as LevelBinder.LevelData
                val routerData = (it[1] as List<Any>)[1] as RouteBinder.RouteData
                var levelIndex = 1
                if (state.value.watchHistory != null) {
                    val routerIndex = state.value.watchHistory!!.play_index?.toInt() ?: 1
                    levelIndex = state.value.watchHistory!!.episode?.toInt() ?: 1
                    val routerIndexNew: Int = routerIndex - 1
                    routerData.index = if (routerIndexNew < 0) {
                        0
                    } else {
                        routerIndexNew
                    }
                    if (routerIndex < routerData.list.size) {
                        levelData.list = routerData.list[routerData.index].urls?.toMutableList()
                            ?: mutableListOf()
                    }
                    if ((levelIndex - 1) < levelData.list.size) {
                        levelData.index = levelIndex - 1
                    }
                    Log.e("Level", "handleEvent: 播放线路:$routerIndex , 播放集数: $levelIndex")
                }
                val info = ((it[1] as List<Any>)[0] as SynopsisBinder.SynopsisData).data

//                if (levelData.list.size <= 1) {
//                    list.add((it[1] as List<Any>)[0])
//                    list.add((it[1] as List<Any>)[1])
//                } else {
//                    list.addAll(it[1] as List<Any>)
//                }

                if (info.price_type?.toInt() == 2) {
                    setEffect { PlayerUiEffect.PayForPlay(info) }
                } else {
//                    val showAdv = style?.playerBeforeAdv?.show ?: false // 动态模板不控制播放前广告
                    if (ad.isNullOrEmpty()) {
                        // 没有广告,直接播放
//                        if (info.watchHistory == null) {
//                            updatePlayCount(0, 0f)
//                        }
                        setEffect { PlayerUiEffect.Play(info, routerData, levelData) }
                    } else {
                        setEffect { PlayerUiEffect.PlayBeforeAd(ad[0] as AdvertisingBean) }
                    }
                }

//                it[2]?.forEach { ad ->
//                    if ((ad as AdvertisingInfo).type == 1) {
//                        newData.add(AdImageBinder.Data(ad))
//                    } else {
//                        newData.add(AdTextBinder.Data(ad))
//                    }
//                }
                list.add((it[1] as List<Any>)[0])
                list.add(routerData)

//                if(routerData.list.size > 1){
//                    list.add(routerData)
//                }
                style?.let { it1 ->
                    it1.content?.forEach { item ->
                        when (item!!.type) {
                            StyleConsts.SWIPER_ADV -> {
                                val carourseAdvs =
                                    JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.CAROUSEL_ADV)
//                                carourseAdvs?.let { advs->
//                                    list.add( BannerBinder.BannerInfo(advs))
//                                }
                                val bannerAdvs = carourseAdvs?.let { it2 ->
                                    DataUtils.filterAdvPosition(
                                        it2,
                                        AdvPosition.Player
                                    )
                                }
                                bannerAdvs?.let { it2 ->
                                    list.add(BannerBinder.BannerInfo(it2))
                                }
                            }
                            StyleConsts.BANNER_ADV -> {
                                val bannerAdv =
                                    JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.BANNER_ADV)
                                val bannerAdvs = bannerAdv?.let { it2 ->
                                    DataUtils.filterAdvPosition(it2, AdvPosition.Player)
                                }
                                bannerAdvs?.let { it3 ->
                                    val adBinderData: MutableList<Any> = mutableListOf()
                                    it3.forEach { it2 ->
                                        if (it2.image_type != 3) {
                                            adBinderData.add(AdImageBinder.Data(it2))
                                        } else {
                                            adBinderData.add(AdTextBinder.Data(it2))
                                        }
                                    }
                                    list.addAll(adBinderData)
                                }
                            }
//                            StyleConsts.SYNOPSIS -> list.add((it[1] as List<Any>)[0])
//                            StyleConsts.LINES -> list.add(routerData)
                            StyleConsts.PARTS -> {
                                val type = item.config?.style ?: 1
                                val levelBinder =
                                    LevelBinder.LevelData(levelData.list, levelIndex - 1, item)
                                list.add(levelBinder)
                                //                                if(levelData.list.size > 1){
//
//                                }
                            }
                            StyleConsts.VIDEO_GRID -> {
                                val type = item.config?.layout?.style ?: 1
                                val pageSize = StyleUtils.getGridCountByType(type)
                                val datas: MutableList<List<Any>> = mutableListOf()

                                val videos = mutableListOf<Any>()
                                likeList?.forEach { it1 ->
                                    if (it1.price_type?.toInt() == 1) {
                                        videos.add(ItemFreeBinder.Free(it1))
                                    } else {
                                        videos.add(ItemPayBinder.Pay(it1))
                                    }
                                }
                                val count = videos.size / pageSize
                                for (i: Int in 0 until count) {
                                    val start: Int = i * pageSize
                                    datas.add(videos.subList(start, start + pageSize))
                                }
                                // 最后一次不满pageSize
                                if ((count * pageSize) < videos.size) {
                                    datas.add(videos.subList(count * pageSize, videos.size))
                                }
                                if (videos.isNotEmpty()) {
                                    list.add(
                                        VideoListBinder.Data(
                                            datas[0].toMutableList(),
                                            type,
                                            datas,
                                            0
                                        )
                                    )
                                }
                            }
                        }
                    }
                }
                listDatas.addAll(list)
                setState {
                    copy(
                        pageStatus = PageStatus.Success,
                        data = list,
                        info = info,
                        router = routerData,
                        level = levelData
                    )
                }
            }.onStart {
                if (isRefresh) {
                    setState { copy(refreshStatus = RefreshStatus.RefreshLoading) }
                } else {
                    setState { copy(pageStatus = PageStatus.Loading) }
                }
            }.catch {
                if (isRefresh) {
                    setState { copy(pageStatus = PageStatus.Error(it)) }
                } else {
                    setState { copy(pageStatus = PageStatus.Error(it)) }
                }
            }.collect()
        }
    }

    /**
     * 换一换
     */
    @Synchronized
    private fun switch(data: VideoListBinder.Data, position: Int) {
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
//            map["site_id"] = GlobalConsts.SITE_ID
            map["order"] = "random"
            map["limit"] = StyleUtils.getGridCountByType(data.type)
            if (state.value.info?.categ_id != null) {
                map["category_id"] = state.value.info?.categ_id!!
            }
            setEffect { PlayerUiEffect.Loading("正在更新") }
            val videos = ApiFactory.getInstance().create(ApiService::class.java).getFilmList(map)
            if (videos.success() && videos.list?.isNotEmpty() == true) {
                val videoData = mutableListOf<Any>()
                videos.list.forEach {
                    if (it.price_type?.toInt() == 1) {
                        videoData.add(ItemFreeBinder.Free(it))
                    } else {
                        videoData.add(ItemPayBinder.Pay(it))
                    }
                }
                val resData = VideoListBinder.Data(
                    videoData,
                    data.type,
                    data.datas,
                    1
                )
                setEffect {
                    PlayerUiEffect.ChangeDataEffect(resData, position)
                }
            } else {
                setEffect {
                    PlayerUiEffect.Toast("没有更多数据啦!")
                }
            }
            setEffect { PlayerUiEffect.Dismiss }

//            if(curData.curPos == 0 && curData.datas?.size == 1){
//                setEffect {
//                    PlayerUiEffect.Toast("没有更多数据啦!")
//                }
//            }else{
//                var curPos = curData.curPos +1
//                val listSize  =curData.datas?.size?:0
//                if(curPos >= listSize ){
//                    curPos = 0
//                }
//                val resData = VideoListBinder.Data(
//                    curData.datas?.get(curPos) as MutableList<Any>,
//                    curData.type,
//                    curData.datas,
//                    curPos
//                )
//                listDatas[position] = resData
//                setEffect {
//                    PlayerUiEffect.ChangeDataEffect(resData,position)
//                }
//            }
        }
    }

    // 金币购买
    private fun buyByGold() {
        val data = state.value.info
        viewModelScope.launch {
//            val info =
//                ApiFactory.getInstance().create(ApiService::class.java).buyVideo(data!!.id)
//            if (info.success()) {
//                setEffect {
//                    PlayerUiEffect.Toast("购买成功")
//                }
//                getPlayerBefore()
//                setEffect {
//                    PlayerUiEffect.Play(data, state.value.router!!, state.value.level!!)
//                }
//            } else {
//                setEffect {
//                    PlayerUiEffect.Toast(info.msg ?: "购买失败")
//                }
//                setEffect {
//                    PlayerUiEffect.PayStatus(false)
//                }
//            }
        }
    }

    /**
     * 刷新数据
     */
    private fun refreshData() {
        viewModelScope.launch {
            combine(
                listOf(
                    getAd(9, Int.MAX_VALUE),
                    getVideoInformation(),
                    getAd(10, Int.MAX_VALUE)
                )
            ) {
                val newData: MutableList<Any> = mutableListOf()
//                it[0]?.forEach { ad ->
//                    if ((ad as AdvertisingInfo).type == 1) {
//                        newData.add(AdImageBinder.Data(ad))
//                    } else {
//                        newData.add(AdTextBinder.Data(ad))
//                    }
//                }
                newData.addAll(it[1] as List<Any>)
//                it[2]?.forEach { ad ->
//                    if ((ad as AdvertisingInfo).type == 1) {
//                        newData.add(AdImageBinder.Data(ad))
//                    } else {
//                        newData.add(AdTextBinder.Data(ad))
//                    }
//                }
                val likeList: List<Any> = it[3] as List<Any>
                if (likeList.isNotEmpty()) {
                    newData.add("猜你喜欢")
                    newData.addAll(likeList)
                }
                setState {
                    copy(
                        refreshStatus = RefreshStatus.RefreshSuccess,
                        data = newData,
                        info = ((it[1] as List<Any>)[0] as SynopsisBinder.SynopsisData).data,
                        router = ((it[1] as List<Any>)[1] as RouteBinder.RouteData),
                        level = ((it[1] as List<Any>)[2] as LevelBinder.LevelData)
                    )
                }
            }.onStart {
                setState { copy(refreshStatus = RefreshStatus.RefreshLoading) }
            }.catch {
                setState { copy(refreshStatus = RefreshStatus.RefreshFail(it)) }
            }.collect()
        }
    }

    /**
     * 获取播前广告
     */
    private fun getPlayerBefore() {
//        val showAdv = style?.playerBeforeAdv?.show ?: false
//        if (showAdv) {
        viewModelScope.launch {
            getAd(AdvType.BeforePlayAdv.ordinal + 1, 1).catch {
                Log.e("URL", "异常")
                playVideo()
            }.collect {
                if (!it.isNullOrEmpty()) {
                    Log.e("URL", "广告")

                    setEffect { PlayerUiEffect.PlayBeforeAd(it[0]) }
                } else {
                    Log.e("URL", "其他")
                    playVideo()
                }
            }
        }
//        } else {
//            playVideo()
//        }
    }

    private fun playVideo() {
        setEffect {
            PlayerUiEffect.Play(
                state.value.info!!,
                state.value.router!!,
                state.value.level!!
            )
        }
    }

    // 获取演员列表
    private fun getActorList(): Flow<List<RoleBean>> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["id"] = mId
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getActorByVideo(map)
            if (bean.success()) {
                emit(bean.list ?: listOf())
            } else {
                emit(listOf())
//                throw Error(bean.message)
            }
        }
    }

    private var likeList: MutableList<VideoBean>? = mutableListOf<VideoBean>()

    // 获取视频相关的信息和演员信息
    private fun getVideoInformation(): Flow<List<Any>> {
        return combine(getVideoInfo(), getActorList()) { info, actorList ->
            val newData: MutableList<Any> = mutableListOf()
            newData.add(SynopsisBinder.SynopsisData(info.video!!, actorList, info.extra))
            setState { copy(watchHistory = info.watch_history) }
            likeList = info.like_list?.list as MutableList<VideoBean>?
            val lineJson = info.video.play_url
            if (lineJson?.isNotEmpty() == true) {
                val lines = JsonUtils.getListData<LineBean>(lineJson)
                if (!lines.isNullOrEmpty()) {
                    newData.add(RouteBinder.RouteData(lines.toMutableList()))
                    // 去
                    val levels = lines[0].urls

                    if (!levels.isNullOrEmpty()) {
                        newData.add(LevelBinder.LevelData(levels.toMutableList(), 0))
                    }
                }
            }
            newData
        }
    }

    // 获取视频信息
    private fun getVideoInfo(): Flow<VideoDetailBean> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["id"] = mId
            val bean = ApiFactory.getInstance().create(ApiService::class.java)
                .getVideoBean(map)
            if (bean.success()) {
                val videoBean = bean.video
                val status = bean.status
                if (videoBean != null && status != null) {
                    dataReport(videoBean.id)
                    videoBean.is_star = status.is_like == 1
                    videoBean.is_collect = status.is_collect
                }
                emit(bean)
            } else {
                throw Error(bean.message)
            }
        }
    }

    /**
     * 数据上报
     */
    private fun dataReport(resId: String?) {
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            val playRecord = DataUtils.dataReport(3).play_record!!
            playRecord.user_id = IMineService.getMineService().getUser().value?.getUserId()
            playRecord.resource_id = resId
            map["play_record"] = playRecord
            try {
                val res = ApiFactory.getInstance().create(CommonApiService::class.java)
                    .dataReport(GlobalConsts.DATA_REPORT + "/v1/basic_record/create", map)
                if (res.success()) {
                    Log.d("DATA_REPORT", "上报成功")
                } else {
                    Log.d("DATA_REPORT", "上报失败")
                }
            } catch (e: Exception) {
                Log.e("Exception", e.message.toString())
            }
        }
    }

    /**
     * 获取广告
     * @param location 显示位置
     */
    private fun getAd(location: Int, limit: Int): Flow<List<AdvertisingBean>?> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["limit"] = limit
            map["library_id"] = GlobalConsts.LIBRARY_ID
            map["type"] = location
            val bean =
                ApiFactory.getInstance().create(ApiService::class.java).getAdvertisingList(map)
            if (bean.success()) {
                emit(bean.list)
            } else {
                emit(mutableListOf())
            }
        }
    }

    var currPosition = 0

    /**
     * 获取暂停广告
     */
    private fun getPauseAd() {
        viewModelScope.launch {
            getAd(AdvType.SuspendPlayAdv.ordinal + 1, 20).catch {}.collect {
                if (!it.isNullOrEmpty()) {
                    if (currPosition < it.size - 1) {
                        currPosition++
                    } else {
                        currPosition = 0
                    }
                    setEffect { PlayerUiEffect.PlayPauseAd(it[currPosition]) }
                }
            }
        }
    }

    /**
     * @desc 兼顾 详情点赞/收藏 和猜你喜欢列表收藏
     * 点赞或者收藏 type 1为点赞 2为收藏 position数据的位置
     */
    private fun collectOrLike(parentPos: Int, type: Int, position: Int?) {
        viewModelScope.launch {
            val oldData: List<Any> = state.value.data
            if (oldData.isEmpty()) {
                return@launch
            }
            var realPos: Int = -1
            if (position == null) {
                oldData.forEachIndexed { index, any ->
                    if (any is SynopsisBinder.SynopsisData) {
                        realPos = index
                        return@forEachIndexed
                    }
                }
            } else {
                realPos = if (parentPos >= 0) {
                    parentPos
                } else {
                    position
                }
            }

            val map: MutableMap<String, Any> = mutableMapOf()
            val data: Any = oldData[realPos]

            val videoInfo: VideoBean? = when (data) {
                is VideoListBinder.Data -> {
                    val childPos = position ?: 0
                    if (data.data.isNotEmpty() && (position ?: 0) < data.data.size) {
                        val child = data.data[childPos]
                        if (child is ItemFreeBinder.Free) {
                            child.info
                        } else if (child is ItemPayBinder.Pay) {
                            child.info
                        } else {
                            return@launch
                        }
                    } else {
                        return@launch
                    }
                }
                is SynopsisBinder.SynopsisData -> {
                    data.data
                }
                else -> return@launch
            }
            map["type"] = 1
            val ids = mutableListOf<String>()
            ids.add(videoInfo!!.id.toString())
            map["resource_id_list"] = ids
            map["resource_id"] = videoInfo!!.id.toString()
            map["belong_app_id"] = Constants.APP_ID

            val bean = if (type == 1) {
                if (!videoInfo.is_star) ApiFactory.getInstance()
                    .create(CommonApiService::class.java)
                    .star(map = map) else ApiFactory.getInstance()
                    .create(CommonApiService::class.java).deleteStar(map = map)
            } else {
                if (videoInfo.is_collect != 1) ApiFactory.getInstance()
                    .create(CommonApiService::class.java)
                    .collect(map = map) else ApiFactory.getInstance()
                    .create(CommonApiService::class.java).deleteCollect(map = map)
            }
            if (bean.success()) {
                var info: VideoBean? = null
                val newData = state.value.data.toMutableList()
                var tip = ""
                if (type == 1) {
                    // 点赞
                    var isStar = false
                    if (data is SynopsisBinder.SynopsisData) {
                        isStar = data.data.is_star
                        newData[realPos] = data.copy(
                            data = data.data.copy(is_star = !data.data.is_star)
                        )
                        info = data.data.copy(is_star = !data.data.is_star)
                    }
                    tip = if (isStar) "已取消" else "谢谢你的喜欢"
                    updateCollectOrLikeData(tip, info, newData)
                } else {
                    // 收藏
                    var isCollect = false
                    if (data is SynopsisBinder.SynopsisData) {
                        isCollect = data.data.is_collect == 1
                        newData[realPos] = data.copy(
                            data = data.data.copy(
                                is_collect = if (data.data.is_collect == 1) {
                                    2
                                } else {
                                    1
                                }
                            )
                        )
                        info = data.data.copy(
                            is_collect = if (data.data.is_collect == 1) {
                                2
                            } else {
                                1
                            }
                        )
                        tip = if (isCollect) "已取消" else "收藏成功,可在\"我的\"页面查看"
                        updateCollectOrLikeData(tip, info, newData)
                    } else {
                        updateLikeVideoLsCollect(
                            parentPos,
                            videoInfo.is_collect == 1,
                            position ?: 0
                        )
                    }
                }
            }
        }
    }

    /**
     * 更新收藏或者喜欢状态·
     */
    private fun updateCollectOrLikeData(
        tip: String,
        info: VideoBean?,
        newData: MutableList<Any>
    ) {
        setEffect {
            PlayerUiEffect.Toast(tip)
        }
        if (info != null) {
            setState { copy(data = newData, info = info) }
        } else {
            setState { copy(data = newData) }
        }
    }

    /**
     * 更新猜你喜欢列表收藏状态
     * @param parentPos 父级位置
     * @param isCollect 是否收藏（旧状态）
     * @param childPos 子列表位置
     */
    private fun updateLikeVideoLsCollect(parentPos: Int, isCollect: Boolean, childPos: Int) {
        try {
            var info: VideoBean? = null
            val newData = state.value.data.toMutableList()
            val newChildData = newData[parentPos]
            var tip = if (isCollect) "已取消" else "收藏成功,可在\"我的\"页面查看"
            val realIsCollect = if (isCollect) {
                2
            } else {
                1
            }
            if (newChildData is VideoListBinder.Data) {
                setEffect {
                    PlayerUiEffect.Toast(tip)
                }
                setState {
                    copy(
                        likeLsCollectStatus = UpdatePlayerCollectStatus(
                            parentPos,
                            childPos,
                            realIsCollect
                        )
                    )
                }
            }
        } catch (e: Exception) {
            LogUtils.e("${e.message}")
        }
    }

    /**
     * 更新播放量
     * @param index 播放第几集
     * @param position 播放的位置
     */
    private fun updatePlayCount(index: Int, position: Float, totalTime: String) {
        if (position <= 0f) {
            return
        }
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            val video = state.value.info
            val router = state.value.router?.index
            map["type"] = 1
            map["play_index"] = (router ?: 0) + 1
            map["resource_id"] = video?.id.toString()
            map["progress"] = position.toString()
            map["percentage"] = totalTime
            map["episode"] = index + 1
            val res = ApiFactory.getInstance().create(CommonApiService::class.java)
                .watchHistory(map = map)
            if (res.success()) {
                Log.d("heat", "updateBrowser:更新数据成功,有历史了$position ")
            } else {
                Log.d("heat", "updateBrowser:更新数据失败")
            }
        }
    }
}
