package com.at.video.player.ui.work.television

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.player.entity.StarVideoInfo
import com.at.video.player.entity.VideoInfo
import com.at.video.player.http.ApiService
import com.at.video.player.ui.work.television.binder.ItemFreeBinder
import com.at.video.player.ui.work.television.binder.ItemPayBinder
import com.melancholy.network.ApiFactory
import com.video.module_widget.binder.AdImageBinder
import com.video.module_widget.binder.AdTextBinder
import com.video.module_widget.binder.BannerBinder
import com.video.module_widget.binder.GapLineBinder
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.constants.StyleConsts
import com.video.module_widget.entity.AdvertisingBean
import com.video.module_widget.entity.HomeStyle
import com.video.module_widget.utils.JsonUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.*
import kotlinx.coroutines.launch
import javax.inject.Inject

/**
 *
 * @Description: WorkViewModel
 * @CreateDate: 2022/6/13 下午2:25
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/13 下午2:25
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

data class TelevisionUiState(
    val pageStatus: PageStatus = PageStatus.Loading,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val data: List<Any> = listOf()
) : IUiState

sealed class TelevisionUiEvent : IUiEvent {
    data class HeaderData(val id: String, val categoryId: String, val style: HomeStyle?) : TelevisionUiEvent()
    object InitLoadData : TelevisionUiEvent()
    object RefreshData : TelevisionUiEvent()
    object LoadMoreData : TelevisionUiEvent()
}
sealed class TelevisionUiEffect : IUiEffect

@HiltViewModel
class TelevisionViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<TelevisionUiState, TelevisionUiEvent, TelevisionUiEffect>(application) {
    private var mId: String = ""
    private var mCategoryId: String = ""
    private var style: HomeStyle? = null
    private var limit: Int = 20
    private var offset: Int = 0
    private var hasMore: Boolean = false

    override fun providerInitialState(): TelevisionUiState {
        return TelevisionUiState()
    }

    override fun handleEvent(event: TelevisionUiEvent) {
        when (event) {
            is TelevisionUiEvent.HeaderData -> {
                mId = event.id
                mCategoryId = event.categoryId
                style = event.style
            }
            is TelevisionUiEvent.InitLoadData -> initLoadData()
            is TelevisionUiEvent.RefreshData -> initLoadData(true)
            is TelevisionUiEvent.LoadMoreData -> { getMoreData() }
        }
    }

    private fun initLoadData(isRefresh: Boolean = false) {
        offset = 0
        hasMore = false
        viewModelScope.launch {
            getActorWorks().onStart {
                if (isRefresh) {
                    setState { copy(refreshStatus = RefreshStatus.RefreshLoading) }
                } else {
                    setState { copy(pageStatus = PageStatus.Loading) }
                }
            }.catch {
                if (isRefresh) {
                    setState { copy(refreshStatus = RefreshStatus.RefreshFail(it)) }
                } else {
                    setState { copy(pageStatus = PageStatus.Error(it)) }
                }
            }.collect {
                if (it.isEmpty()) {
                    setState { copy(pageStatus = PageStatus.Empty, refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle) }
                } else {
                    val workList = it
                    val data: MutableList<Any> = mutableListOf()
                    val newData: MutableList<Any> = mutableListOf()

                    workList.forEach { it1 ->
                        if (it1.price_type?.toInt() == 1) {
                            data.add(ItemFreeBinder.Free(it1))
                        } else {
                            data.add(ItemPayBinder.Pay(it1))
                        }
                    }

                    style?.let {
                        it.content?.forEach { item ->
                            when (item!!.type) {
//                                StyleConsts.SWIPER_ADV -> {
//                                    val carourseAdvs = JsonUtils.getAdvListData<AdvertisingBean>(
//                                        GlobalConsts.CAROUSEL_ADV
//                                    )
//                                    carourseAdvs?.let { advs ->
//                                        newData.add(GapLineBinder.Data(""))
//                                        newData.add(BannerBinder.BannerInfo(advs))
//                                    }
//                                }
//                                StyleConsts.BANNER_ADV -> {
//                                    val bannerAdvs = JsonUtils.getAdvListData<AdvertisingBean>(GlobalConsts.BANNER_ADV)
//                                    val adBinderData: MutableList<Any> = mutableListOf()
//                                    bannerAdvs?.let { advs ->
//                                        advs.forEach { item ->
//                                            run {
//                                                adBinderData.add(GapLineBinder.Data(""))
//                                                if (item.image_type != 3) {
//                                                    adBinderData.add(AdImageBinder.Data(item))
//                                                } else {
//                                                    adBinderData.add(AdTextBinder.Data(item))
//                                                }
//                                            }
//                                        }
//                                        newData.addAll(adBinderData)
//                                    }
//                                }
                                StyleConsts.WORKS -> {
                                    newData.add(GapLineBinder.Data(""))
                                    newData.addAll(data)
                                }
                            }
                        }
                    }

                    setState { copy(pageStatus = PageStatus.Success, data = newData, refreshStatus = if (isRefresh) RefreshStatus.RefreshSuccess else RefreshStatus.RefreshIdle) }
                }
            }
        }
    }

    private fun refreshData() {
        viewModelScope.launch {
            getDataList().onStart {
                setState { copy(refreshStatus = RefreshStatus.RefreshLoading) }
            }.catch {
                setState { copy(refreshStatus = RefreshStatus.RefreshFail(it)) }
            }.collect {
                if (it.isEmpty()) {
                    setState {
                        copy(
                            pageStatus = PageStatus.Empty,
                            refreshStatus = RefreshStatus.RefreshSuccess
                        )
                    }
                } else {
                    setState { copy(refreshStatus = RefreshStatus.RefreshSuccess, data = it) }
                }
            }
        }
    }

    /**
     * 获取更多作品数据
     */
    private fun getDataList(): Flow<List<VideoInfo>> {
        return flow {
//            val bean = ApiFactory.getInstance().create(ApiService::class.java)
//                .getPersonWorks(mId, mCategoryId)
//            if(bean.success()) {
//                emit(bean.data ?: listOf())
//            } else {
//                throw Error(bean.msg)
//            }
        }
    }

    private fun getMoreData() {
        offset += limit
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["person_id"] = mId
            map["category_id"] = mCategoryId
            map["offset"] = offset
            map["limit"] = limit
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getStartWork(map)
            if (bean.success()) {
                hasMore = bean.list[0].video_list?.size!! == limit
                val data: MutableList<Any> = mutableListOf()
                bean.list[0].video_list?.forEach { it1 ->
                    if (it1.price_type?.toInt() == 1) {
                        data.add(ItemFreeBinder.Free(it1))
                    } else {
                        data.add(ItemPayBinder.Pay(it1))
                    }
                }
                val oldData = state.value.data.toMutableList()
                if (data.size > 0) {
                    oldData.addAll(data)
                }
                setState { copy(data = oldData, loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
            } else {
                setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(false)) }
            }
        }
    }

    private fun getActorWorks(): Flow<List<StarVideoInfo>> {
        return flow {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["person_id"] = mId
            map["category_id"] = mCategoryId
            map["offset"] = offset
            map["limit"] = limit
            val bean = ApiFactory.getInstance().create(ApiService::class.java).getStartWork(map)
            if (bean.success()) {
                bean.list[0].video_list?.let {
                    emit(it)
                }
            } else {
                throw Error(bean.message)
            }
        }
    }
}
