package com.at.video.service.ui.service.conversation

import android.app.Application
import androidx.lifecycle.viewModelScope
import com.at.common.global.CacheHelper
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.PageStatus
import com.at.video.export.mine.IMineService
import com.at.video.service.entity.Message
import com.at.video.service.entity.MessageInfo
import com.at.video.service.entity.SendMessageInfo
import com.at.video.service.http.ApiService
import com.melancholy.network.ApiFactory
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.io.File
import java.util.*
import javax.inject.Inject

/**
 *
 * @Description: ConversationViewModel
 * @CreateDate: 2022/6/16 上午9:06
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/16 上午9:06
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

data class ConversationUiState(
    val pageStatus: PageStatus = PageStatus.Empty,
    val loginData:Message? = null,
    val kefuMsg:MessageInfo? = null,
    val sendMessage:SendMessageInfo ? = null,
    val messageData:List<ChartData>? = null,
    val newMsg:List<ChartData>? = null
) : IUiState

sealed class ConversationUiEvent : IUiEvent {
    object Login:ConversationUiEvent()
    data class SessionToWorkOrder(val title:String):ConversationUiEvent()
    data class SendMessage(val content: String):ConversationUiEvent()
    object LatestMessage:ConversationUiEvent()
    data class UploadFile(val file:File):ConversationUiEvent()
    object GetVisitInfo:ConversationUiEvent()
}
sealed class ConversationUiEffect : IUiEffect {
    object LoginFail:ConversationUiEffect()
    object WorkOrderTurnSuccess:ConversationUiEffect()
    data class Toast(val msg: String): ConversationUiEffect()
    object TransformOrder:ConversationUiEffect()
}

@HiltViewModel
class ConversationViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<ConversationUiState, ConversationUiEvent, ConversationUiEffect>(application) {
    companion object{
        const val SHARED_SESSION_ID= "shared_session_id"
        const val SHARED_KEFU_ID= "shared_kefu_id"
        const val SHARED_KEFU_NAME= "shared_kefu_name"
        const val MAX_DATA = 10
    }

    override fun providerInitialState(): ConversationUiState {
        return ConversationUiState()
    }

    override fun handleEvent(event: ConversationUiEvent) {
        when(event){
            is ConversationUiEvent.Login -> login()
            is ConversationUiEvent.SessionToWorkOrder ->sessionToWorkOrder(event.title)
            is ConversationUiEvent.SendMessage -> sendMessage(event.content)
            is ConversationUiEvent.LatestMessage ->getLatestMessage()
            is ConversationUiEvent.UploadFile -> uploadFile(event.file)
            is ConversationUiEvent.GetVisitInfo -> getVisitInfo()
        }
    }

    private fun login(){
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            val usr = IMineService.getMineService().getUser().value
            map["visitor_id"] = usr?.getUserId().toString()
            map["name"] = usr?.getUserName()!!
            val res =ApiFactory.getInstance().create(ApiService::class.java).login(map)
            if(res.code == 6200 &&res.data!=null){
                val msg = res.data!!
                CacheHelper.getInstance().putString(SHARED_SESSION_ID, msg.session_id)
                CacheHelper.getInstance().putString(SHARED_KEFU_ID, msg.kefu_id)
                CacheHelper.getInstance().putString(SHARED_KEFU_NAME, msg.kefu_name)
                setState { copy(loginData = Message(id=msg.session_id, content = msg.content, name = msg.kefu_name, avatar = msg.kefu_avatar, msg_type = 1, time =Date().toString()), kefuMsg = msg) }
            }else{
                setEffect { ConversationUiEffect.LoginFail }
            }
            getLatestMessage(true)
        }
    }

    private fun getVisitInfo(){
        viewModelScope.launch {
            val visitorId = IMineService.getMineService().getUser().value?.getUserId()!!
            val res = ApiFactory.getInstance().create(ApiService::class.java).getVisitInfo(visitId = visitorId.toInt())
            if(res.code == 6200 && res.data != null){
                if(res.data!!.state == 1){
                    setEffect { ConversationUiEffect.Toast("当前会话已结束不能转为工单!") }
                }else{
                    setEffect { ConversationUiEffect.TransformOrder }
                }
            }else{
                setEffect { ConversationUiEffect.Toast(res.msg?:"请求异常") }
            }
        }
    }

    private fun getHistoryData(){
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["visitor_id"] = IMineService.getMineService().getUser().value?.getUserId()!!
            val res = ApiFactory.getInstance().create(ApiService::class.java).historySession(map)
            if(res.code == 6200 && !res.data!!.list.isNullOrEmpty()){
                val messages = res.data!!.list?.map { it -> ChartData(it) }
                setState { copy(messageData = messages) }
            }
        }
    }

    private fun sessionToWorkOrder(title:String){
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["title"] = title
            map["session_id"] = state.value.kefuMsg?.session_id?:""
            val res = ApiFactory.getInstance().create(ApiService::class.java).sessionToWorkOrder(map)
            if(res.code == 6200&&res.data!=null){
                setEffect { ConversationUiEffect.WorkOrderTurnSuccess }
            }else{
                setEffect { ConversationUiEffect.Toast(res.msg!!) }
            }
        }
    }

    private fun sendMessage(content:String,contentType:String =""){
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["from_id"] = state.value.kefuMsg?.visitor_id?:""
            map["to_id"] = CacheHelper.getInstance().getString(SHARED_KEFU_ID,"")!!
            map["content"] = content
            map["msg_type"] = 2
            map["content_type"] = contentType
            map["session_id"] = CacheHelper.getInstance().getString(SHARED_SESSION_ID,"")!!
            val res = ApiFactory.getInstance().create(ApiService::class.java).sendMessage(map)
            if(res.code == 6200&&res.data!!.message!=null){
                setState { copy(sendMessage = res.data) }
            }else{
                setEffect { ConversationUiEffect.Toast(res.msg!!) }
            }
        }
    }

    private fun getLatestMessage(isGetHis:Boolean = false){
        val usr = IMineService.getMineService().getUser().value ?: return
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()

            map["visitor_id"] = state.value.kefuMsg?.visitor_id?:""
            map["kefu_id"] = state.value.kefuMsg?.kefu_id?:""
            map["kefu_name"] = state.value.kefuMsg?.kefu_name?:""
            map["client"] = true
            map["visitor_name"] = usr?.getUserName()!!
            val res = ApiFactory.getInstance().create(ApiService::class.java).getLatestMessage(map)

            if(res.code == 6200 &&!res.data!!.messages.isNullOrEmpty()){
                val messages = res.data!!.messages?.map { it -> ChartData(it) }
                setState { copy(newMsg = messages) }
            }
            if(isGetHis){
                getHistoryData()
            }
        }
    }

    private fun uploadFile(file:File){
        viewModelScope.launch(Dispatchers.IO) {
            val part = MultipartBody.Part.createFormData("file", file.name,
                RequestBody.create(null, file))
            val map: MutableMap<String, String> = mutableMapOf()
            map["type"] = "image"
            val upload = ApiFactory.getInstance().create(ApiService::class.java)
                .updateFile(map,part)
            if(upload.code == 6200 && upload.data!=null) {
                sendMessage("<img width=\"100\" src=\"${Constants.SERVICE_URL + upload.data!!.path}\"/>",
                    Constants.SERVICE_URL + upload.data!!.path
                )
            }else{
                setEffect { ConversationUiEffect.Toast("图片上传失败!") }
            }
        }
    }
}