package com.at.video.service.ui.service.order.add

import android.app.Application
import android.util.Log
import androidx.lifecycle.viewModelScope
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.export.mine.IMineService
import com.at.video.service.entity.FileData
import com.at.video.service.entity.FileInfo
import com.at.video.service.http.ApiService
import com.at.video.service.http.body.ProgressRequestBody
import com.at.video.service.http.listener.ProgressRequestListener
import com.at.video.service.ui.service.conversation.ConversationUiEffect
import com.at.video.service.ui.service.utils.UriUtils
import com.melancholy.network.ApiFactory
import com.melancholy.utils.DateUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import okhttp3.MediaType
import okhttp3.MediaType.Companion.toMediaTypeOrNull
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.io.File
import java.util.*
import javax.inject.Inject

/**
 *
 * @Description: WorkOrderAddViewModel
 * @CreateDate: 2022/6/16 下午5:35
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/16 下午5:35
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

data class WorkOrderAddUiState(
    val pageStatus: PageStatus = PageStatus.Empty,

) : IUiState

sealed class WorkOrderAddUiEvent : IUiEvent {
    data class AddWorkOrder(val title:String,val content:String):WorkOrderAddUiEvent()
    data class UploadFile(val file: File,val fileInfo:FileData,val position: Int = 0):WorkOrderAddUiEvent()
}
sealed class WorkOrderAddUiEffect : IUiEffect {
    data class Toast(val msg:String):WorkOrderAddUiEffect()
    object AddSuccess:WorkOrderAddUiEffect()
    data class UploadFileSuccess(val fileInfo:FileData):WorkOrderAddUiEffect()
    data class UploadFileProgress(val progress:Int,val position:Int):WorkOrderAddUiEffect()
}

@HiltViewModel
class WorkOrderAddViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<WorkOrderAddUiState, WorkOrderAddUiEvent, WorkOrderAddUiEffect>(application) {

    var appendix:MutableList<FileInfo> = mutableListOf()

    override fun providerInitialState(): WorkOrderAddUiState {
        return WorkOrderAddUiState()
    }

    override fun handleEvent(event: WorkOrderAddUiEvent) {
        when(event){
            is WorkOrderAddUiEvent.AddWorkOrder -> addWorkOrder(event.title,event.content)
            is WorkOrderAddUiEvent.UploadFile -> uploadFile(event.file,event.fileInfo,event.position)
        }
    }

    private fun addWorkOrder(title: String,content: String){
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["title"] = title
            map["visitor_id"] = IMineService.getMineService().getUser().value?.getUserId()!!.toString()
            map["content"] = content
            if(appendix.size != 0){
                map["appendix"] = appendix!!
            }
            val res = ApiFactory.getInstance().create(ApiService::class.java).createWorkOrder(map)
            if(res.code == 6200 &&res.data!=null){
                setEffect { WorkOrderAddUiEffect.AddSuccess }
            }else{
                setEffect { WorkOrderAddUiEffect.Toast(res.msg!!) }
            }
        }
    }

    private fun uploadFile(file: File,fileData: FileData,position: Int = 0){
        viewModelScope.launch(Dispatchers.IO) {
//            val part = MultipartBody.Part.createFormData("file", file.name,
//                RequestBody.create(null, file))

            val requestBody = RequestBody.create("multipart/form-data".toMediaTypeOrNull(), file)
            val part = MultipartBody.Part.createFormData("file",file.name, ProgressRequestBody(requestBody,
                object : ProgressRequestListener {
                    override fun onRequestProgress(
                        bytesWritten: Long,
                        contentLength: Long,
                        done: Boolean
                    ) {
                        val progress= UriUtils.getFloatNoMoreThanTwoDigits((bytesWritten / contentLength.toDouble()).toFloat()).toFloat() * 100
                        setEffect { WorkOrderAddUiEffect.UploadFileProgress(progress.toInt(),position) }
                    }
                })
            )
            val type = if(fileData.type == 1) "image" else "file"
            val map: MutableMap<String, String> = mutableMapOf()
            map["type"] = type
            val upload = ApiFactory.getInstance().create(ApiService::class.java)
                .updateFile(map,part)
            if(upload.code == 6200 &&upload.data != null) {
                appendix.add(upload.data!!)
                val fileInfo = FileData(type = fileData.type, path = upload.data!!.path, name = upload.data!!.name, createAt = DateUtils.format(
                    Date(),"yyyy-MM-DD HH:mm:ss"
                ), size = fileData.size)
                setEffect { WorkOrderAddUiEffect.UploadFileSuccess(fileInfo) }
            }else{
                setEffect { WorkOrderAddUiEffect.Toast("图片上传失败!") }
            }
        }
    }
}