package com.at.video.service.ui.service.order.detail

import android.app.Application
import android.os.Environment
import androidx.lifecycle.viewModelScope
import com.at.common.global.Constants
import com.at.common.mvi.BaseAndroidViewModel
import com.at.common.mvi.IUiEffect
import com.at.common.mvi.IUiEvent
import com.at.common.mvi.IUiState
import com.at.common.status.LoadMoreStatus
import com.at.common.status.PageStatus
import com.at.common.status.RefreshStatus
import com.at.video.export.mine.IMineService
import com.at.video.service.entity.FileData
import com.at.video.service.entity.FileInfo
import com.at.video.service.entity.OrderDetailMsgInfo
import com.at.video.service.http.ApiService
import com.melancholy.network.ApiFactory
import com.melancholy.utils.DateUtils
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import okhttp3.MultipartBody
import okhttp3.RequestBody
import java.io.File
import java.io.FileOutputStream
import java.util.*
import javax.inject.Inject

/**
 *
 * @Description: WorkOrderDetailViewModel
 * @CreateDate: 2022/6/17 上午10:16
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/6/17 上午10:16
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */

data class WorkOrderDetailUiState(
    val pageStatus: PageStatus = PageStatus.Empty,
    val refreshStatus: RefreshStatus = RefreshStatus.RefreshIdle,
    val loadMoreStatus: LoadMoreStatus = LoadMoreStatus.LoadMoreIdle,
    val chatData:List<OrderDetailMsgInfo>? = null,
    val sendData:OrderDetailMsgInfo?=null,
) : IUiState

sealed class WorkOrderDetailUiEvent : IUiEvent {
    data class InitData(val workId:String):WorkOrderDetailUiEvent()
    object EndOrder:WorkOrderDetailUiEvent()
    object RevokeOrder:WorkOrderDetailUiEvent()
    data class UploadFile(val file: File,val fileInfo:FileData):WorkOrderDetailUiEvent()
    data class SendMessage(val message:String,val kefuId: String):WorkOrderDetailUiEvent()
    object RefreshData: WorkOrderDetailUiEvent()
    object LoadDataMore: WorkOrderDetailUiEvent()
    object TakePhoto:WorkOrderDetailUiEvent()
    data class DownLoadFile(val fileUrl:String,val fileName:String):WorkOrderDetailUiEvent()
}
sealed class WorkOrderDetailUiEffect : IUiEffect {
    data class Toast(val msg:String):WorkOrderDetailUiEffect()
    data class UploadFileSuccess(val fileInfo:FileData):WorkOrderDetailUiEffect()
    object EndOrderSuccess:WorkOrderDetailUiEffect()
    object RevokeSuccess:WorkOrderDetailUiEffect()
    object TakePhoto:WorkOrderDetailUiEffect()
}

@HiltViewModel
class WorkOrderDetailViewModel @Inject constructor(application: Application) :
    BaseAndroidViewModel<WorkOrderDetailUiState, WorkOrderDetailUiEvent, WorkOrderDetailUiEffect>(
        application
    ) {
    private var mOffset: Int = 0
    private var mLimit: Int = 9
    private var workId:String =""
    var appendix:MutableList<FileInfo> = mutableListOf()

    override fun providerInitialState(): WorkOrderDetailUiState {
        return WorkOrderDetailUiState()
    }

    override fun handleEvent(event: WorkOrderDetailUiEvent) {
        when(event){
            is WorkOrderDetailUiEvent.InitData ->{
                workId = event.workId
                initChatHistory()
            }
            is WorkOrderDetailUiEvent.EndOrder ->endOrder()
            is WorkOrderDetailUiEvent.RevokeOrder -> revokeOrder()
            is WorkOrderDetailUiEvent.RefreshData -> initChatHistory(1)
            is WorkOrderDetailUiEvent.LoadDataMore -> initChatHistory(2)
            is WorkOrderDetailUiEvent.SendMessage -> sendMessage(event.message,event.kefuId)
            is WorkOrderDetailUiEvent.UploadFile -> uploadFile(event.file,event.fileInfo)
            is WorkOrderDetailUiEvent.TakePhoto ->setEffect { WorkOrderDetailUiEffect.TakePhoto }
            is WorkOrderDetailUiEvent.DownLoadFile->download(event.fileUrl,event.fileName)
        }
    }

    private fun download(fileUrl: String,fileName:String) {
     viewModelScope.launch {
         val rootPath = if(Environment.getExternalStorageState().equals(Environment.MEDIA_MOUNTED)){
             Environment.getExternalStorageDirectory().absolutePath + "/"
         }else{
             Environment.getDataDirectory().absolutePath +"/"
         }
         val file = File("$rootPath/$fileName")
         if(file.exists()){
             setEffect { WorkOrderDetailUiEffect.Toast("文件已下载：${file.absolutePath}") }
         }else{
             file.createNewFile()
         }

         val res = ApiFactory.getInstance().create(ApiService::class.java).download(Constants.SERVICE_URL + fileUrl)
         val inputStream = res.byteStream()
         val fos = FileOutputStream(file.absolutePath)
         try {
             var len = 0
             var buffer = ByteArray(2048*2)
            do{
               len = inputStream.read(buffer)
               fos.write(buffer,0,len)
             } while (len  != -1)
         }catch (e:Exception){

         }finally {
             fos.close()
             inputStream.close()
             setEffect { WorkOrderDetailUiEffect.Toast("已下载：${file.absolutePath}") }
         }
     }
    }

    private fun sendMessage(message:String,kefuId:String){
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["work_id"] = workId
            map["from_id"] = IMineService.getMineService().getUser().value?.getUserId()!!.toString()
            map["to_id"] = kefuId
            map["content"] = message
            map["msg_type"] = 2
            if(appendix.size!=0){
                map["appendix"] = appendix
            }
            val res = ApiFactory.getInstance().create(ApiService::class.java).chat(map)

            if(res.code == 6200 &&res.data!=null){
                appendix.clear()
                setState { copy(sendData = res.data) }
            }else{
                setEffect { WorkOrderDetailUiEffect.Toast(res.msg!!) }
            }
        }
    }

    private fun initChatHistory(type:Int = 0){
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["work_id"] = workId
            map["offset"] = mOffset
            map["limit"] = mLimit
            when(type){
                0->  setState { copy(pageStatus = PageStatus.Loading) }
                1->  setState { copy(refreshStatus = RefreshStatus.RefreshLoading,
                    loadMoreStatus = LoadMoreStatus.LoadMoreIdle) }
                2-> setState { copy(loadMoreStatus = LoadMoreStatus.LoadMoreLoading) }
            }
            val res = ApiFactory.getInstance().create(ApiService::class.java).chatHistory(map)
            if(res.code == 6200 &&res.data !=null){
                if(res.data!!.list.isNullOrEmpty()){
                    setState { copy(pageStatus = PageStatus.Empty) }
                }else{
                    val hasMore: Boolean = res.data!!.list!!.size == mLimit
                    if(hasMore) {
                        mOffset += mLimit
                    }
                    val data = if(type == 2) state.value.chatData!! + res.data!!.list!! else res.data!!.list
                    setState { copy(pageStatus = PageStatus.Success, chatData = data,
                        loadMoreStatus = LoadMoreStatus.LoadMoreSuccess(hasMore)) }
                }
            }else{
                setState { copy(pageStatus = PageStatus.Error(Throwable(res.msg))) }
            }
        }
    }

    private fun endOrder(){
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["work_id"] = workId
            val res = ApiFactory.getInstance().create(ApiService::class.java).over(map)
            if(res.data!=null){
                setEffect { WorkOrderDetailUiEffect.EndOrderSuccess}
            }else{
                setEffect { WorkOrderDetailUiEffect.Toast(res.msg!!) }
            }
        }
    }

    private fun revokeOrder(){
        viewModelScope.launch {
            val map: MutableMap<String, Any> = mutableMapOf()
            map["work_id"] = workId
            val res = ApiFactory.getInstance().create(ApiService::class.java).revoke(map)
            if(res.code == 6200 &&res.data!=null){
                setEffect { WorkOrderDetailUiEffect.RevokeSuccess }
            }else{
                setEffect { WorkOrderDetailUiEffect.Toast(res.msg!!) }
            }
        }
    }
    private fun uploadFile(file: File, fileData: FileData){

        viewModelScope.launch(Dispatchers.IO) {
            val part = MultipartBody.Part.createFormData("file", file.name,
                RequestBody.create(null, file))
            val type = if(fileData.type == 1) "image" else "file"
            val map: MutableMap<String, String> = mutableMapOf()
            map["type"] = type
            val upload = ApiFactory.getInstance().create(ApiService::class.java)
                .updateFile(map,part)
            if(upload.code == 6200 &&upload.data != null) {
                appendix.add(upload.data!!)
                val fileInfo = FileData(type = fileData.type, path = upload.data!!.path, name = upload.data!!.name, createAt = DateUtils.format(
                    Date(),"yyyy-MM-DD HH:mm:ss"
                ))
                setEffect { WorkOrderDetailUiEffect.UploadFileSuccess(fileInfo) }
            }else{
                setEffect { WorkOrderDetailUiEffect.Toast("图片上传失败!") }
            }
        }
    }
}