package com.video.module_widget.exception

import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Looper
import android.util.Log
import androidx.core.content.pm.PackageInfoCompat
import androidx.lifecycle.viewModelScope
import com.at.common.global.CacheHelper
import com.melancholy.network.ApiFactory
import com.melancholy.utils.FileUtils
import com.video.module_widget.api.CommonApiService
import com.video.module_widget.constants.GlobalConsts
import com.video.module_widget.dialog.SystemDialog
import com.video.module_widget.utils.DataUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import java.io.File
import java.io.FileOutputStream
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.HashMap

/**
 *@author luoyi
 *@name 崩溃拦截类
 *@desciption 拦截崩溃并上报
 *@since 2023/1/30 下午1:33
 *@version 1.0.0
 **/
object ExceptionHandler {

    /**
     * 拦截异常
     * @param context application 上下文对象
     */
    fun hookException(context: Context) {
        Thread.setDefaultUncaughtExceptionHandler { t, e ->
            e.printStackTrace()
            // 子线程的不处理
            if (Looper.myLooper() != Looper.getMainLooper()) {
                return@setDefaultUncaughtExceptionHandler
            }
            dealOnException(context, e)
            // 保证程序不anr，不崩溃退出
            while (true) {
                try {
                    if (Looper.myLooper() != null && Looper.myLooper() == Looper.getMainLooper()) {
                        Looper.loop()
                    } else {
                        break
                    }
                } catch (exception: Exception) {
                    exception.printStackTrace()
                    dealOnException(context, exception.cause)
                }
            }

        }

    }

    /**
     * 当异常发生时候，处理异常
     * @param context 上下文对象
     * @param e 异常
     */
    @Synchronized
    private fun dealOnException(context: Context, e: Throwable?) {
        collectionExceptionLog(context, e)
        showDialog(context)
    }

    /**
     * 显示程序出错弹出
     * @param context 上下文对象
     */
    private fun showDialog(context: Context) {
        val intent = Intent(context, SystemDialog::class.java)
        intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
        intent.putExtra("Message", "程序出错啦!请退出应用重新进入!")
        context.applicationContext.startActivity(intent)
    }

    /**
     * 收集错误日志
     * @param context 上下文对象
     * @param e 异常
     */
    private fun collectionExceptionLog(context: Context, e: Throwable?) {
        if (e == null) {
            return
        }
        GlobalScope.launch(Dispatchers.IO) {
            val map: MutableMap<String, Any> = mutableMapOf()
            val logRecord = DataUtils.dataReport(1).log_record!!
            logRecord.content = getLogSummary(context, e)
            map["log_record"] = logRecord
            try {
                val res = ApiFactory.getInstance().create(CommonApiService::class.java).dataReport(
                    GlobalConsts.DATA_REPORT + "/v1/basic_record/create", map
                )
                if (res.success()) {
                    Log.d("DATA_REPORT", "上报成功")
                } else {
                    Log.d("DATA_REPORT", "上报失败")
                    logRecord.content?.let {
                        saveFile(context, it)
                    }
                }
            } catch (e: Exception) {
                Log.e("Exception", e.message.toString())
                logRecord.content?.let {
                    saveFile(context, it)
                }
            }
        }

    }

    /**
     * 再次上传异常日志（先检查日志上否上传）
     * @param context 上下文对象
     */
    fun uploadExceptionAgain(context: Context) {
        GlobalScope.launch(Dispatchers.IO) {
            val paths = FileUtils.getExternalFilesDirs(
                context, "log"
            )
            val parent: File =
                if (paths.isNullOrEmpty()) File(context.filesDir, "log") else paths[0]
            val isExists = parent.exists()
            if (!isExists) {
                parent.mkdir()
            }
            val path = File(parent, "crash_log.txt")
            if (!path.exists()) {
                return@launch
            }
            val bfr = path.bufferedReader()
            val content = bfr.use { it.readText() }
            val map: MutableMap<String, Any> = mutableMapOf()
            val logRecord = DataUtils.dataReport(1).log_record!!
            logRecord.content = content
            map["log_record"] = logRecord
            try {
                val res = ApiFactory.getInstance().create(CommonApiService::class.java).dataReport(
                    GlobalConsts.DATA_REPORT + "/v1/basic_record/create", map
                )
                if (res.success()) {
                    Log.d("DATA_REPORT", "上报成功")
                    path.delete()
                } else {
                    Log.d("DATA_REPORT", "上报失败")
                }
            } catch (e: Exception) {
                Log.e("Exception", e.message.toString())
            }
        }
    }

    /**
     * 保存异常日志文件
     * @param context 上下文
     * @param message 异常日志
     */
    private fun saveFile(context: Context, message: String) {
        val paths = FileUtils.getExternalFilesDirs(
            context, "log"
        )
        val parent: File = if (paths.isNullOrEmpty()) File(context.filesDir, "log") else paths[0]
        val isExists = parent.exists()
        if (!isExists) {
            parent.mkdir()
        }
        val path = File(parent, "crash_log.txt")
        if (!path.exists()) {
            path.createNewFile()
        }
        val fos = FileOutputStream(path)
        try {
            fos.write(message.toByteArray())
            fos.write("\n".toByteArray())
            fos.flush()
        } catch (e: Exception) {
        } finally {
            fos.close()
        }
    }


    /**
     * 收集参数信息
     * @param context
     */
    private fun getPhoneInfoMap(context: Context): HashMap<String, String> {
        // 用于存放参数信息
        val info = HashMap<String, String>()
        info["设备型号"] = "${Build.MODEL}<br/>"
        info["设备品牌"] = "${Build.BOARD}<br/>"
        info["硬件名称"] = "${Build.HARDWARE}<br/>"
        info["硬件制造商"] = "${Build.MANUFACTURER}<br/>"
        info["系统版本"] = "${Build.VERSION.RELEASE}<br/>"
        info["系统版本号"] = "${Build.VERSION.SDK_INT}<br/>"

        val pm = context.packageManager
        val pi = pm.getPackageInfo(context.packageName, PackageManager.GET_ACTIVITIES)
        if (pi != null) {
            info["应用版本"] = "${pi.versionName}<br/>"
            info["应用版本号"] = "${PackageInfoCompat.getLongVersionCode(pi)}<br/>"
        }
        return info
    }

    /**
     * 获取日志头信息
     * @return StringBuffer
     */
    private fun getLogHeader(info: HashMap<String, String>): StringBuffer {
        // 用于格式化日期
        val mDateFormat = SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault())
        val sb = StringBuffer()
        sb.append(">>>>时间: ${mDateFormat.format(Date())}<br/>")
        info.entries.forEach {
            sb.append("${it.key}: ${it.value}<br/>")
        }
        return sb
    }

    /**
     * 获取日志概要
     * @param e
     * @return 日志概要
     */
    private fun getLogSummary(context: Context, e: Throwable): String {
        val sb = getLogHeader(getPhoneInfoMap(context)).append("<br/>")
        sb.append("异常类: ${e.javaClass}<br/>")
        sb.append("异常信息: ${e.message}<br/>")
        for (i in e.stackTrace.indices) {
            sb.append("**** ${i + 1}<br/>")
            sb.append("类名: ${e.stackTrace[i].className}<br/>")
            sb.append("方法: ${e.stackTrace[i].methodName}<br/>")
            sb.append("文件: ${e.stackTrace[i].fileName}<br/>")
            sb.append("行数: ${e.stackTrace[i].lineNumber}<br/><br/>")
        }
        return sb.toString().trim()
    }
}