package com.video.module_widget.exception // ktlint-disable package-name

import android.app.Application
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Looper
import androidx.core.content.pm.PackageInfoCompat
import com.melancholy.utils.FileUtils
import com.video.module_widget.dialog.SystemDialog
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.launch
import java.io.File
import java.io.FileOutputStream
import java.text.SimpleDateFormat
import java.util.*
import kotlin.collections.LinkedHashMap

/**
 *
 * @Description: java类作用描述
 * @CreateDate: 2022/12/29 上午9:24
 * @UpdateUser: 更新者
 * @UpdateDate: 2022/12/29 上午9:24
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
class GlobalExceptionHandler private constructor(context: Context) :
    Thread.UncaughtExceptionHandler {

    // 系统默认的 UncaughtException 处理类
    private val mDefaultHandler = Thread.getDefaultUncaughtExceptionHandler()

    // 上下文对象
    private val mContext = context

    // 用于存放参数信息
    private val info = LinkedHashMap<String, String>()

    // 用于格式化日期
    private val mDateFormat = SimpleDateFormat("yyyy-MM-dd HH:mm:ss", Locale.getDefault())

    // 单例模式
    companion object {
        private var instance: GlobalExceptionHandler? = null
        fun getInstance(context: Context): GlobalExceptionHandler? {
            if (instance == null) {
                synchronized(GlobalExceptionHandler::class) {
                    instance = GlobalExceptionHandler(context)
                }
            }
            return instance
        }
    }

    /**
     * 构造初始化
     */
    init {
        // 设置当前类为应用默认处理器
        Thread.setDefaultUncaughtExceptionHandler(this)
    }

    /**
     * 当 UncaughtException 发生时转入该函数
     * @param t
     * @param e
     */
    override fun uncaughtException(t: Thread, e: Throwable) {
        synchronized(GlobalExceptionHandler::class.java) {
//            GlobalScope.launch {
//                val map: MutableMap<String, Any> = mutableMapOf()
//                val logRecord = DataUtils.dataReport(1).log_record!!
//                logRecord.content = getLogSummary(e)
//                map["log_record"] = logRecord
            try {
//                    val res =
//                        ApiFactory.getInstance().create(CommonApiService::class.java).dataReport(
//                            GlobalConsts.DATA_REPORT + "/v1/basic_record/create", map
//                        )
//                    if (res.success()) {
//                        Log.d("DATA_REPORT", "上报成功")
//                    } else {
//                        Log.d("DATA_REPORT", "上报失败")
//                        logRecord.content?.let {
//                            saveFile(it)
//                        }
//                    }
            } catch (e: Exception) {
//                    Log.e("Exception", e.message.toString())
//                    logRecord.content?.let {
//                        saveFile(it)
//                    }
            } finally {
                if (mContext is Application) {
                    val intent = Intent(mContext.applicationContext, SystemDialog::class.java)
                    intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                    intent.putExtra("Message", "程序出错啦!请退出应用重新进入!")
                    mContext.applicationContext.startActivity(intent)
                }

//                }
            }
        }
    }

    // 保存异常到文件
    private fun saveFile(message: String) {
        GlobalScope.launch(Dispatchers.IO) {
            val paths = FileUtils.getExternalFilesDirs(
                mContext,
                "log"
            )
            val parent: File =
                if (paths.isNullOrEmpty()) File(mContext.filesDir, "log") else paths[0]
            val isExists = parent.exists()
            if (!isExists) {
                parent.mkdir()
            }
            val path = File(parent, "crash_log.txt")
            if (!path.exists()) {
                path.createNewFile()
            }
            val fos = FileOutputStream(path)
            try {
                fos.write(message.toByteArray())
                fos.write("\n".toByteArray())
                fos.flush()
            } catch (e: Exception) {
            } finally {
                fos.close()
            }
        }
    }

    /**
     * 自定义错误处理
     * @param e
     * @return 已处理异常信息
     */
    private fun handleException(e: Throwable): Boolean {
        Thread {
            Looper.prepare()
            synchronized(GlobalExceptionHandler::class.java) {
                val intent = Intent(mContext, SystemDialog::class.java)
                intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                intent.putExtra("Message", "程序出错啦")
                intent.putExtra("btnTitle", "重启应用")
                mContext.startActivity(intent)
            }
            Looper.loop()
        }.start()

        /**
         * 这里可以执行一些业务操作
         * 比如保存崩溃日志到文件等等
         */
        return true
    }

    /**
     * 收集参数信息
     * @param context
     */
    private fun putInfoToMap(context: Context) {
        info["设备型号"] = Build.MODEL
        info["设备品牌"] = Build.BOARD
        info["硬件名称"] = Build.HARDWARE
        info["硬件制造商"] = Build.MANUFACTURER
        info["系统版本"] = Build.VERSION.RELEASE
        info["系统版本号"] = "${Build.VERSION.SDK_INT}"

        val pm = context.packageManager
        val pi = pm.getPackageInfo(context.packageName, PackageManager.GET_ACTIVITIES)
        if (pi != null) {
            info["应用版本"] = pi.versionName
            info["应用版本号"] = "${PackageInfoCompat.getLongVersionCode(pi)}"
        }
    }

    /**
     * 获取日志头信息
     * @return StringBuffer
     */
    private fun getLogHeader(): StringBuffer {
        val sb = StringBuffer()
        sb.append(">>>>时间: ${mDateFormat.format(Date())}\n")
        putInfoToMap(mContext)
        info.entries.forEach {
            sb.append("${it.key}: ${it.value}\n")
        }
        return sb
    }

    /**
     * 获取日志概要
     * @param e
     * @return 日志概要
     */
    private fun getLogSummary(e: Throwable): String {
        val sb = getLogHeader().append("\n")
        sb.append("异常类: ${e.javaClass}\n")
        sb.append("异常信息: ${e.message}\n\n")
        for (i in e.stackTrace.indices) {
            sb.append("****堆栈追踪 ${i + 1}\n")
            sb.append("类名: ${e.stackTrace[i].className}\n")
            sb.append("方法: ${e.stackTrace[i].methodName}\n")
            sb.append("文件: ${e.stackTrace[i].fileName}\n")
            sb.append("行数: ${e.stackTrace[i].lineNumber}\n\n")
        }
        return sb.toString().trim()
    }
}
