package com.video.module_widget.utils

import android.Manifest
import android.annotation.SuppressLint
import android.content.ContentResolver
import android.content.Context
import android.content.pm.PackageManager
import android.content.res.Resources
import android.database.Cursor
import android.net.ConnectivityManager
import android.net.NetworkInfo
import android.net.Uri
import android.os.Build
import android.telephony.SubscriptionInfo
import android.telephony.SubscriptionManager
import android.telephony.TelephonyManager
import android.telephony.TelephonyManager.*
import android.util.DisplayMetrics
import android.util.Log
import androidx.appcompat.app.AppCompatActivity
import androidx.core.app.ActivityCompat
import com.melancholy.utils.AppInstance
import java.lang.reflect.Method
import java.net.Inet4Address
import java.net.InetAddress
import java.net.NetworkInterface
import java.net.SocketException
import java.util.*

object PhoneUtils {

    // 获取IMEI码
    fun getIMEI(): String {
        val tm = AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        try {
            return tm.imei
        } catch (e: Exception) {
        }
        return getUniquePsuedoID()
    }

    private fun getUniquePsuedoID(): String {
        val m_szDevIDShort =
            "35" + Build.BOARD.length % 10 + Build.BRAND.length % 10 + Build.CPU_ABI.length % 10 + Build.DEVICE.length % 10 + Build.DISPLAY.length % 10 + Build.HOST.length % 10 + Build.ID.length % 10 + Build.MANUFACTURER.length % 10 + Build.MODEL.length % 10 + Build.PRODUCT.length % 10 + Build.TAGS.length % 10 + Build.TYPE.length % 10 + Build.USER.length % 10
        var serial: String
        try {
            serial = Build::class.java.getField("SERIAL")[null]?.toString() ?: ""
            return UUID(m_szDevIDShort.hashCode().toLong(), serial.hashCode().toLong()).toString()
        } catch (e: Exception) {
            // 获取失败，使用AndroidId
            serial = UUID.randomUUID().toString()
        }
        return UUID(m_szDevIDShort.hashCode().toLong(), serial.hashCode().toLong()).toString()
    }

    /**
     * 显示指标
     */
    private fun getDisplayMetrics(): DisplayMetrics {
        return Resources.getSystem().displayMetrics
    }

    /**
     * 获取整个手机屏幕的大小(包括虚拟按钮)
     * 必须在onWindowFocus方法之后使用
     * @param activity
     * @return
     */
    fun getScreenSize(activity: AppCompatActivity): String {
        val size = IntArray(2)
        val decorView = activity.window.decorView
        size[0] = decorView.width
        size[1] = decorView.height
        return "${size[0]}x${size[1]}"
    }

    fun getAppSize(): String {
        val size = IntArray(2)
        val metrics = getDisplayMetrics()
        size[0] = metrics.widthPixels
        size[1] = metrics.heightPixels
        return "${size[0]}x${size[1]}"
    }

    /**
     * 判断sim卡是否准备好
     *
     * @return `true`: 是<br></br>`false`: 否
     */
    fun isSimCardReady(): Boolean {
        val tm = AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        return tm.simState == TelephonyManager.SIM_STATE_READY
    }

    /**
     * 获取Sim卡运营商名称
     *
     * 中国移动、如中国联通、中国电信
     *
     * @return sim卡运营商名称
     */
    fun getSimOperatorName(): String? {
        val tm = AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        return tm.simOperatorName
    }

    /**
     * 获取Sim卡运营商名称
     *
     * 中国移动、如中国联通、中国电信
     *
     * @return 移动网络运营商名称
     */
    fun getSimOperatorByMnc(): Int {
        val tm = AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        val operator = (tm.simOperator) ?: return 0
        return when (operator) {
            "46000", "46002", "46007" -> 3
            "46001" -> 2
            "46003" -> 1
            else -> 0
        }
    }

    /**
     * 获取Sim卡序列号
     *
     *
     * Requires Permission:
     * [READ_PHONE_STATE][android.Manifest.permission.READ_PHONE_STATE]
     *
     * @return 序列号
     */
    fun getSimSerialNumber(): String? {
        try {
            val tm =
                AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
            val serialNumber = tm.simSerialNumber
            return serialNumber ?: ""
        } catch (e: java.lang.Exception) {
        }
        return ""
    }

    /**
     * 获取Sim卡的国家代码
     *
     * @return 国家代码
     */
    fun getSimCountryIso(): String? {
        val tm = AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        return tm.simCountryIso
    }

    /**
     * 获得卡槽数，默认为1
     *
     * @return 返回卡槽数
     */
    fun getSimCount(): Int {
        var count = 0
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1) {
            try {
                val mSubscriptionManager = AppInstance.getApplication()
                    .getSystemService(Context.TELEPHONY_SUBSCRIPTION_SERVICE) as SubscriptionManager
                if (mSubscriptionManager != null) {
                    count = mSubscriptionManager.activeSubscriptionInfoCountMax
                    return count
                }
            } catch (ignored: java.lang.Exception) {
            }
        }
        try {
            count = getReflexMethod("getPhoneCount")?.toInt() ?: 0
        } catch (ignored: java.lang.Exception) {
        }
        return count
    }

    /**
     * 获取Sim卡使用的数量
     *
     * @return 0, 1, 2
     */
    @SuppressLint("MissingPermission")
    fun getSimUsedCount(): Int {
        var count = 0
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1) {
            try {
                val mSubscriptionManager = AppInstance.getApplication()
                    .getSystemService(Context.TELEPHONY_SUBSCRIPTION_SERVICE) as SubscriptionManager
                count = mSubscriptionManager.activeSubscriptionInfoCount
                return count
            } catch (ignored: java.lang.Exception) {
            }
        }
        val tm = AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        if (tm.simState == TelephonyManager.SIM_STATE_READY) {
            count = 1
        }
        try {
            if (getReflexMethodWithId(
                    "getSimState",
                    "1"
                )!!.toInt() == TelephonyManager.SIM_STATE_READY
            ) {
                count = 2
            }
        } catch (ignored: Exception) {
        }
        return count
    }

    /**
     * 获取多卡信息
     *
     * @return 多Sim卡的具体信息
     */
    @SuppressLint("MissingPermission")
    fun getSimMultiInfo(): List<SimInfo>? {
        val infos: MutableList<SimInfo> = ArrayList()
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP_MR1) {
            // 1.版本超过5.1，调用系统方法
            val mSubscriptionManager = AppInstance.getApplication()
                .getSystemService(Context.TELEPHONY_SUBSCRIPTION_SERVICE) as SubscriptionManager
            var activeSubscriptionInfoList: List<SubscriptionInfo>? = null
            if (mSubscriptionManager != null) {
                try {
                    activeSubscriptionInfoList = mSubscriptionManager.activeSubscriptionInfoList
                } catch (ignored: java.lang.Exception) {
                }
            }
            if (activeSubscriptionInfoList != null && activeSubscriptionInfoList.size > 0) {
                // 1.1.1 有使用的卡，就遍历所有卡
                for (subscriptionInfo in activeSubscriptionInfoList) {
                    val simInfo = SimInfo(
                        subscriptionInfo.carrierName,
                        subscriptionInfo.iccId,
                        subscriptionInfo.simSlotIndex,
                        subscriptionInfo.number,
                        subscriptionInfo.countryIso
                    )
//                    try {
//                        simInfo.mImei =
//                            getReflexMethodWithId("getDeviceId", simInfo.mSimSlotIndex.toString())
//                        simInfo.mImsi = getReflexMethodWithId(
//                            "getSubscriberId",
//                            subscriptionInfo.subscriptionId.toString()
//                        )
//                    } catch (ignored: MethodNotFoundException) {
//                    }
                    infos.add(simInfo)
                }
            }
        } else {
            // 2.版本低于5.1的系统，首先调用数据库，看能不能访问到
            val uri: Uri = Uri.parse("content://telephony/siminfo") // 访问raw_contacts表
            val resolver: ContentResolver = AppInstance.getApplication().contentResolver
            val cursor: Cursor? = resolver.query(
                uri,
                arrayOf(
                    "_id",
                    "icc_id",
                    "sim_id",
                    "display_name",
                    "carrier_name",
                    "name_source",
                    "color",
                    "number",
                    "display_number_format",
                    "data_roaming",
                    "mcc",
                    "mnc"
                ),
                null,
                null,
                null
            )
            if (cursor != null) {
                while (cursor.moveToNext()) {
                    val mCarrierName = cursor.getString(cursor.getColumnIndex("carrier_name"))
                    val mIccId = cursor.getString(cursor.getColumnIndex("icc_id"))
                    val mSimSlotIndex = cursor.getInt(cursor.getColumnIndex("sim_id"))
                    val mNumber = cursor.getString(cursor.getColumnIndex("number"))
                    val mCountryIso = cursor.getString(cursor.getColumnIndex("mcc"))
                    val simInfo = SimInfo(
                        mCarrierName,
                        mIccId,
                        mSimSlotIndex,
                        mNumber,
                        mCountryIso
                    )
                    infos.add(simInfo)
                }
                cursor.close()
            }
        }
        return infos
    }

    private fun isChekSelfPermission(permission: String?): Boolean {
        return (
            ActivityCompat.checkSelfPermission(AppInstance.getApplication(), permission!!)
                != PackageManager.PERMISSION_GRANTED
            )
    }

    private fun isEthernet(): Boolean {
        val cm =
            AppInstance.getApplication().getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val info = cm.getNetworkInfo(ConnectivityManager.TYPE_ETHERNET) ?: return false
        val state = info.state ?: return false
        return state == NetworkInfo.State.CONNECTED || state == NetworkInfo.State.CONNECTING
    }

    fun getNetworkType(): Int {
        if (isChekSelfPermission(Manifest.permission.ACCESS_NETWORK_STATE)) {
            return 0
        }
        if (isEthernet()) {
            return 0
        }
        val cm =
            AppInstance.getApplication().getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val info = cm.activeNetworkInfo
        return if (info != null && info.isAvailable) {
            if (info.type == ConnectivityManager.TYPE_WIFI) {
                2
            } else if (info.type == ConnectivityManager.TYPE_MOBILE) {
                when (info.subtype) {
                    TelephonyManager.NETWORK_TYPE_GSM, TelephonyManager.NETWORK_TYPE_GPRS, TelephonyManager.NETWORK_TYPE_CDMA, TelephonyManager.NETWORK_TYPE_EDGE, TelephonyManager.NETWORK_TYPE_1xRTT, TelephonyManager.NETWORK_TYPE_IDEN -> 1
                    TelephonyManager.NETWORK_TYPE_TD_SCDMA, TelephonyManager.NETWORK_TYPE_EVDO_A, TelephonyManager.NETWORK_TYPE_UMTS, TelephonyManager.NETWORK_TYPE_EVDO_0, TelephonyManager.NETWORK_TYPE_HSDPA, TelephonyManager.NETWORK_TYPE_HSUPA, TelephonyManager.NETWORK_TYPE_HSPA, TelephonyManager.NETWORK_TYPE_EVDO_B, TelephonyManager.NETWORK_TYPE_EHRPD, TelephonyManager.NETWORK_TYPE_HSPAP -> 1
                    TelephonyManager.NETWORK_TYPE_IWLAN, TelephonyManager.NETWORK_TYPE_LTE -> 1
                    TelephonyManager.NETWORK_TYPE_NR -> 1
                    else -> {
                        val subtypeName = info.subtypeName
                        if (subtypeName.equals("TD-SCDMA", ignoreCase = true) ||
                            subtypeName.equals("WCDMA", ignoreCase = true) ||
                            subtypeName.equals("CDMA2000", ignoreCase = true)
                        ) {
                            1
                        } else {
                            0
                        }
                    }
                }
            } else {
                0
            }
        } else 0
    }

    private fun getTelephonyManager(): TelephonyManager {
        return AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
    }

    fun getPhoneType(): String? {
        when (getTelephonyManager().phoneType) {
            PHONE_TYPE_NONE -> return "PHONE_TYPE_NONE"
            PHONE_TYPE_GSM -> return "PHONE_TYPE_GSM"
            PHONE_TYPE_CDMA -> return "PHONE_TYPE_CDMA"
            PHONE_TYPE_SIP -> return "PHONE_TYPE_SIP"
        }
        return ""
    }

    fun getManufacturer(): String = Build.MANUFACTURER

    fun getDeviceProduct(): String = Build.PRODUCT

    fun getDeviceBrand(): String = Build.BRAND

    fun getDeviceModel(): String = Build.MODEL

    fun getDeviceBoard(): String = Build.BOARD

    fun getDeviceName(): String = Build.DEVICE

    fun getDeviceSDK(): Int = Build.VERSION.SDK_INT

    fun getDeviceAndroidVersion(): String = Build.VERSION.RELEASE

    fun toPrintDeviceInfo(): String {
        return "MANUFACTURER:${getManufacturer()}," +
            "PRODUCT:${getDeviceProduct()}," +
            "BRAND:${getDeviceBrand()}," +
            "MODEL:${getDeviceModel()}," +
            "BOARD:${getDeviceBoard()}," +
            "DEVICE:${getDeviceName()}," +
            "SDK:${getDeviceSDK()}," +
            "Android:${getDeviceAndroidVersion()}"
    }

    fun getLocalIPAddress(): String? {
        try {
            val en: Enumeration<NetworkInterface> = NetworkInterface.getNetworkInterfaces()
            while (en.hasMoreElements()) {
                val intf: NetworkInterface = en.nextElement()
                val enumIpAddr: Enumeration<InetAddress> = intf.inetAddresses
                while (enumIpAddr.hasMoreElements()) {
                    val inetAddress: InetAddress = enumIpAddr.nextElement()
                    if (!inetAddress.isLoopbackAddress && inetAddress is Inet4Address) {
                        // return inetAddress.getAddress().toString();
                        return inetAddress.getHostAddress().toString()
                    }
                }
            }
        } catch (ex: SocketException) {
            Log.e("BaseScanTvDeviceClient", "获取本机IP false =$ex")
        }
        return null
    }

    /**
     * 通过反射调取@hide的方法
     *
     * @param predictedMethodName 方法名
     * @return 返回方法调用的结果
     * @throws Exception 方法没有找到
     */
    @Throws(Exception::class)
    private fun getReflexMethod(predictedMethodName: String): String? {
        var result: String? = null
        val telephony =
            AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        try {
            val telephonyClass = Class.forName(telephony.javaClass.name)
            val getSimID: Method = telephonyClass.getMethod(predictedMethodName)
            val ob_phone: Any = getSimID.invoke(telephony)
            if (ob_phone != null) {
                result = ob_phone.toString()
            }
        } catch (e: java.lang.Exception) {
            throw Exception(predictedMethodName)
        }
        return result
    }

    /**
     * 通过反射调取@hide的方法
     *
     * @param predictedMethodName 方法名
     * @param id                  参数
     * @return 返回方法调用的结果
     * @throws MethodNotFoundException 方法没有找到
     */
    @Throws(Exception::class)
    private fun getReflexMethodWithId(predictedMethodName: String, id: String): String? {
        var result: String? = null
        val telephony =
            AppInstance.getApplication().getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager
        try {
            val telephonyClass = Class.forName(telephony.javaClass.name)
            val parameter = arrayOfNulls<Class<*>?>(1)
            parameter[0] = Int::class.javaPrimitiveType
            val getSimID = telephonyClass.getMethod(predictedMethodName, *parameter)
            val parameterTypes = getSimID.parameterTypes
            val obParameter = arrayOfNulls<Any>(parameterTypes.size)
            if (parameterTypes[0].simpleName == "int") {
                obParameter[0] = Integer.valueOf(id)
            } else if (parameterTypes[0].simpleName == "long") {
                obParameter[0] = java.lang.Long.valueOf(id)
            } else {
                obParameter[0] = id
            }
            val ob_phone = getSimID.invoke(telephony, *obParameter)
            if (ob_phone != null) {
                result = ob_phone.toString()
            }
        } catch (e: java.lang.Exception) {
            throw Exception(predictedMethodName)
        }
        return result
    }
}

data class SimInfo(
    val mCarrierName: CharSequence,
    val mIccId: CharSequence,
    val mSimSlotIndex: Int,
    val mNumber: CharSequence,
    val mCountryIso: CharSequence
)

enum class NetworkType {
    NETWORK_ETHERNET, NETWORK_WIFI, NETWORK_5G, NETWORK_4G, NETWORK_3G, NETWORK_2G, NETWORK_UNKNOWN, NETWORK_NO
}
